#ifdef _WIN32
#include "iVolume.h"
#include <windows.h>
#include <fcntl.h>

//virtual
int iVolume::readSectors( bc_u64 start_sector, bc_u8 *data, bc_u64 number_of_sectors  )
{
	HANDLE hFile = (HANDLE)m_ctx;
	LARGE_INTEGER li;
	li.QuadPart = start_sector * m_sectorSize;
	li.QuadPart += m_beginPos;

	if (!SetFilePointerEx( hFile, li, NULL, FILE_BEGIN))
		return GetLastError();

	if ( number_of_sectors * m_sectorSize > 0x80000000 )
		return ERROR_BAD_LENGTH;

	DWORD br, bt;
	br = (DWORD)(number_of_sectors * m_sectorSize);

	if ( !ReadFile( hFile, data, br, &bt, NULL) )
		return GetLastError();

	if ( bt != br )
		return ERROR_BAD_LENGTH;

	return 0;
}

//virtual
int iVolume::writeSectors( bc_u64 start_sector, const bc_u8 *data, bc_u64 number_of_sectors  )
{
	HANDLE hFile = (HANDLE)m_ctx;
	LARGE_INTEGER li;
	li.QuadPart = start_sector * m_sectorSize;
	li.QuadPart += m_beginPos;

	if (!SetFilePointerEx( hFile, li, NULL, FILE_BEGIN))
		return GetLastError();

	if ( number_of_sectors * m_sectorSize > 0x80000000 )
		return ERROR_BAD_LENGTH;

	DWORD br, bt;
	br = (DWORD)(number_of_sectors * m_sectorSize);

	if ( !WriteFile( hFile, data, br, &bt, NULL) )
		return GetLastError();

	if ( bt != br )
		return ERROR_BAD_LENGTH;

	return 0;
}

//virtual
int iVolume::sync()
{
	HANDLE hFile = (HANDLE)m_ctx;
	return FlushFileBuffers( hFile ) ? iv_Success : iv_IOError;
}

int iVolume::open( const char *name, int flags )
{
	DWORD dwDesiredAccess = GENERIC_READ;
	if ( flags == O_RDWR )
		dwDesiredAccess |= GENERIC_WRITE;

	HANDLE hFile = CreateFileA(  name,  dwDesiredAccess , FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL );
	if ( hFile == INVALID_HANDLE_VALUE )
		return -1;

	m_ctx = hFile;
	m_beginPos = 0x200;

	return 0;
}

int iVolume::close()
{
	HANDLE hFile = (HANDLE)m_ctx;
	if ( ( hFile != NULL ) && ( hFile != INVALID_HANDLE_VALUE ) )
		CloseHandle( hFile );

	m_ctx = NULL;

	return 0;
}

s64 iVolume::getSize()
{
	ULARGE_INTEGER uli;
	HANDLE hFile = (HANDLE)m_ctx;

	uli.LowPart = GetFileSize( hFile, &uli.HighPart );
	if ( ( uli.LowPart == (DWORD)-1 ) && ( GetLastError() != NO_ERROR ) )
		return 0;

	return uli.QuadPart;
}
#endif // _WIN32
