/*
 *	Copyright (c) 2012 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	Auto deallocation template class.
 */

#pragma once

#include <malloc.h>

template < class T > class cADB
{
public:
	size_t m_size;
	T *m_body;

	cADB( size_t size = 0 )
	{
		m_size = 0;
		m_body = NULL;

		if ( 0 != size )
			ReAllocate( size );
	}

	cADB( const T *orig, size_t size = -1 )
	{
		m_size = 0;
		m_body = NULL;

		CreateCopy( orig, size );
	}

	~cADB()
	{
		Free();
	}

	T *ReAllocate( size_t size )
	{
		if ( NULL == m_body )
			m_body = (T*)malloc( sizeof(T) * size );
		else
			m_body = (T*)realloc( m_body, sizeof(T) * size );

		if ( NULL != m_body )
			m_size = size;
		else
			m_size = 0;

		return m_body;
	}

	void Free()
	{
		if ( m_body != NULL )
		{
			memset( m_body, 0, sizeof(T) * m_size );
			free( m_body );
			m_body = NULL;
			m_size = 0;
		}
	}

	size_t GetSizeByFirstZero( const T *orig )
	{
		if ( NULL == orig )
			return 0;

		size_t size = 0;

		while( 0 != orig[ size++ ] )
		{
			if ( 0 == size )
				break;
		}
		
		return size;
	}

	bool setAt( size_t pos, T val )
	{
		if ( ( pos < 0 ) || ( pos >= m_size ) )
			return false;

		m_body[pos] = val;

		return true;
	}

	T *AddString( const T *orig, size_t size = -1 )
	{
		if ( -1 == size )
			size = GetSizeByFirstZero( orig );

		size_t oldSize = ( NULL == m_body ? 0 : GetSizeByFirstZero( m_body ) - 1 );

		if ( NULL == ReAllocate( oldSize + size + 1 ) )
			return NULL;

		if ( orig != NULL )
			memcpy( m_body + oldSize, orig, size * sizeof( T ) );
		else
			memset( m_body + oldSize, 0, size * sizeof( T ) );

		m_body[ oldSize + size ] = 0;

		return m_body;
	}

	T *CreateCopy( const T *orig, size_t size = -1 )
	{
		if ( ( orig == m_body ) && ( orig != NULL ) )
			return m_body;

		Free();
		return AddString( orig, size );
	}

	static int getLength( const T *s )
	{
		int len = 0;

		for(;; len++ )
		{
			if ( s[len] == 0 )
				break;
		}

		return len;
	}

	static int find( const T *s, T c, bool bForward = true )
	{
		if ( bForward )
		{
			for( int i = 0; s[i] != 0; i++ )
			{
				if ( s[i] == c )
					return i;
			}

			return -1;
		}
		else
		{
			int len = getLength( s );

			while( len-- )
			{
				if ( s[len] == c )
					break;
			}

			return len;
		}
	}

	int find( T c, bool bForward = true )
	{
		return find( m_body, c, bForward );
	}

	int splitPath( const T * &l, const T * &r, const T *s, T c, bool bForward = true )
	{
		int pos = find( s, c, bForward );

		if ( pos == -1 )
		{
			l = "";
			r = s;
		}
		else
		{
			l = AddString( s, pos );
			r = s + pos + 1;
		}

		return pos;
	}
};
