/*
 *	Copyright (c) 2012 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	Universal srting class.
 */

#pragma once
#include <locale.h>
#include <string.h>
#include <assert.h>
#include "../multi-lib-enc/container/intsize.h"
#include "FatFs/ff.h"
#include "FatFs/integer.h"
#include "log.h"
#include "cADB.h"

#include "../multi-lib-enc/utf8-utf16/convertUTFSize.h"

#ifdef _LFN_UNICODE
#ifndef _UNICODE
#define _UNICODE
#endif
#endif


class cUString
{
public:

	cADB < UTF16 > m_bodyW;
	cADB < char > m_bodyA;
	
	cUString()						{}
	cUString( const UTF16	*s )	{ CreateCopy( s ); }
	cUString( const char	*s )	{ CreateCopy( s ); }

	void Free()
	{
		m_bodyW.Free();
		m_bodyA.Free();
	}

	bool setAt( int pos, char ch )
	{
		return
			m_bodyA.setAt( pos, ch ) &&
			m_bodyW.setAt( pos, (UTF16)ch );
	}

	char	*strA() { return m_bodyA.m_body; }
	UTF16   *strW() { return (UTF16*)m_bodyW.m_body; }
	TCHAR	*strT()
	{
#ifdef _UNICODE
		return (TCHAR*)strW();
#else
		return strA();
#endif
	}

//	static const char *setClocale()
//	{
//		/* Enable the Japanese locale and codepage */
//		char *localeInfo = setlocale(LC_CTYPE, "Japanese_Japan.932");
//		return localeInfo;
//	}

	static const UTF16 *convertUTF8ToWcs( cADB < UTF16 > &bodyW, const char *mbs )
	{
		if ( mbs == NULL )
		{
			LIBLOG_INFO( "Assertion! NULL" );
			return NULL;
		}

		bc_u32 inlen = strlen( mbs ) + 1;
		bc_u32 outlen = convertUTF8toUTF16( (const UTF8*)mbs, inlen, (UTF16 *)NULL, 0 );

		if ( bodyW.CreateCopy( NULL, outlen ) == NULL )
		{
			LIBLOG_INFO( "Assertion! NULL" );
			return NULL;
		}

		bc_u32 requiredSize = convertUTF8toUTF16( (const UTF8*)mbs, inlen, (UTF16 *)bodyW.m_body, outlen );

#ifdef _LITTLE_ENDIAN_DEFINED
		// Little-endian does nothing.
#else
		if ( NULL != bodyW.m_body )
			swap_bytes( (UTF16 *)bodyW.m_body, requiredSize * sizeof( UTF16 ) );
#endif

		return bodyW.m_body;
	}

	static bc_u32 getWcsLen( const UTF16 *wcs )
	{
		bc_u32 len = 0;
		while( wcs[len] != 0 )
			len++;

		return len;
	}

	static const char *convertWcsToUTF8( cADB < char > &bodyA, const UTF16 *wcs )
	{
		if ( wcs == NULL )
		{
			LIBLOG_INFO("Assertion! NULL" );
			return NULL;
		}

		bc_u32 inlen = getWcsLen( wcs ) + 1;

#ifdef _LITTLE_ENDIAN_DEFINED
		// Little-endian does nothing.
#else
		cADB < UTF16 > bodyW;
		bodyW.CreateCopy( wcs, inlen );
		if ( NULL != bodyW.m_bopy )
			swap_bytes( (UTF16 *)bodyW.m_body, inlen );

		wcs = bodyW.m_body;
#endif

		bc_u32 outlen = convertUTF16toUTF8( (const UTF16*)wcs, inlen, (UTF8 *)NULL, 0 );

		if ( bodyA.CreateCopy( NULL, outlen ) == NULL )
		{
			LIBLOG_INFO("Assertion! NULL" );
			return NULL;
		}

		bc_u32 requiredSize = convertUTF16toUTF8( (const UTF16*)wcs, inlen, (UTF8 *)bodyA.m_body, outlen );

		assert( requiredSize == outlen );

		return bodyA.m_body;
	}

	static const char *getUTF8( const UTF16 *wcs )
	{
		static cADB < char > bodyA;
		return convertWcsToUTF8( bodyA, wcs );
	}

	const UTF16* CreateCopy( const UTF16 *orig, size_t size = -1 )
	{
		if ( m_bodyW.CreateCopy( orig, size ) == NULL )
			return NULL;

		if ( convertWcsToUTF8( m_bodyA, m_bodyW.m_body ) == NULL )
			return NULL;

		return m_bodyW.m_body;
	}

	const char* CreateCopy( const char *orig, size_t size = -1 )
	{
		if ( m_bodyA.CreateCopy( orig, size ) == NULL )
			return NULL;

		if ( convertUTF8ToWcs( m_bodyW, m_bodyA.m_body ) == NULL )
			return NULL;

		return m_bodyA.m_body;
	}

	const UTF16* AddString( const UTF16 *orig, size_t size = -1 )
	{
		if ( m_bodyW.AddString( orig, size ) == NULL )
			return NULL;

		if ( convertWcsToUTF8( m_bodyA, m_bodyW.m_body ) == NULL )
			return NULL;

		return m_bodyW.m_body;
	}

	char *AddString( const char *orig, size_t size = -1 )
	{
		if ( m_bodyA.AddString( orig, size ) == NULL )
			return NULL;

		if ( convertUTF8ToWcs( m_bodyW, m_bodyA.m_body ) == NULL )
			return NULL;

		return m_bodyA.m_body;
	}
};
