#include "ifsFat.h"
#include "FatFs/diskio.h"
#include <assert.h>

#ifndef _WIN32
//#include "..\multi-lib-enc\certificate\CertificateLib\publicKeyAlgs\Power\Windefs.h"
typedef struct _FILETIME {
	DWORD dwLowDateTime;
	 DWORD dwHighDateTime;
} FILETIME, *PFILETIME, *LPFILETIME;

#endif


//static 
iVolume *iFileSystem::m_mountedVolumeT[_VOLUMES];

int assign_drives( int i0, int i1 )
{
	assert( 0 );
	return 0;
}

DSTATUS disk_initialize ( BYTE vol )
{
	return disk_status( vol );
}

DSTATUS disk_status( BYTE vol )
{
	iVolume *iVol = iFATFS::getMountedVolume(vol);

	if ( NULL == iVol )
		return STA_NODISK;

	if ( iVol->isWriteProtected() )
		return STA_PROTECT;

	return 0;
}

//DRESULT disk_read( BYTE vol, BYTE* data, DWORD start_sector, BYTE sectors )
DRESULT disk_read( BYTE vol, BYTE* data, DWORD start_sector, UINT sectors )
{
	LIBLOG_INFO( "Begin start sector 0x%x(%d), sectors 0x%x(%d)",  (int)start_sector, (int)start_sector, (int)sectors, (int)sectors );

	iVolume *iVol = iFATFS::getMountedVolume(vol);

	if ( NULL == iVol )
	{
		LIBLOG_INFO("Assertion!" );
		return RES_NOTRDY;
	}

	LIBLOG_INFO();
	iVolume::eResult res = iVol->readSectors( start_sector, data, sectors );
	LIBLOG_INFO();

	if ( iVolume::iv_IOError == res )
	{
		LIBLOG_INFO("Assertion!" );
		return RES_ERROR;
	}

	LIBLOG_INFO("End OK!" );

	return RES_OK;
}

DRESULT disk_write( BYTE vol, const BYTE* data, DWORD start_sector, UINT sectors )
{
	iVolume *iVol = iFATFS::getMountedVolume(vol);

	if ( NULL == iVol )
	{
		LIBLOG_INFO("RES_NOTRDY" );
		return RES_NOTRDY;
	}

	iVolume::eResult res = iVol->writeSectors( start_sector, data, sectors );

	if ( iVolume::iv_IOError == res )
	{
		LIBLOG_INFO("RES_ERROR %d", res );
		return RES_ERROR;
	}

	return RES_OK;
}

DRESULT disk_ioctl ( BYTE vol, BYTE b0, void* p0 )
{
	iVolume *iVol = iFATFS::getMountedVolume(vol);

	if ( NULL == iVol )
		return RES_NOTRDY;

	if ( b0 == CTRL_SYNC )
	{
		iVolume::eResult res = iVol->sync();

		if ( iVolume::iv_IOError == res )
			return RES_ERROR;

		return RES_OK;
	}
	else
	if ( b0 == GET_SECTOR_COUNT )
	{
		bc_u64 sectorQ = iVol->getSectorCount();

		if ( sectorQ != (DWORD)sectorQ )
			return RES_ERROR;

		*((DWORD *)p0) = (DWORD)sectorQ;
		return RES_OK;
	}
	else
	if ( b0 == GET_SECTOR_SIZE )
	{
		bc_u32 sectorSize = iVol->getSectorSize();

		if ( sectorSize != (WORD)sectorSize )
			return RES_ERROR;

		*((WORD *)p0) = (WORD)sectorSize;
		return RES_OK;
	}
	else
	if ( b0 == GET_BLOCK_SIZE ) // Is used for flash memory only for alligment
	{
		*((DWORD *)p0) = 4 * 1024; // We will use alligment as BEstCrypt usually uses for buffering
		return RES_OK;
	}

	return RES_ERROR;
}

tm dosToTm( DWORD &dosTime  )
{
	tm t;
	memset( &t, 0, sizeof( t ) );

	t.tm_year	= ((int) ((dosTime >> 25) & 0x7f) + 1980 - 1900);
	t.tm_mon	= ((int) ((dosTime >> 21) & 0x0f) - 1);
	t.tm_mday	= ((int) (dosTime >> 16) & 0x1f);
	t.tm_hour	= ((int) (dosTime >> 11) & 0x1f);
	t.tm_min	= ((int) (dosTime >> 5) & 0x3f);
	t.tm_sec	= ((int) (dosTime << 1) & 0x3e);

	return t;
}

DWORD tmToDos( tm &t )
{
	DWORD dt = 0;
	dt |= (t.tm_year - 1980 + 1900) << 25;
	dt |= (t.tm_mon + 1 ) <<21;
	dt |= t.tm_mday	<< 16;
	dt |= t.tm_hour	<< 11;
	dt |= t.tm_min << 5;
	dt |= t.tm_sec >> 1;

	return dt;
}

union uFatTime
{
	struct
	{
		WORD time;
		WORD date;
	};

	DWORD fatTime;
};

DWORD get_fattime (void)
{
//#ifdef _WIN32
//	FILETIME fTime;
//	SYSTEMTIME sysTime;
//
//	//GetSystemTime( &sysTime );
//	GetLocalTime( &sysTime );
//	time_t rawtime = time(&rawtime);
//
//	SystemTimeToFileTime( &sysTime, &fTime );
//
//	tm *timeinfo = localtime(&rawtime);
//	DWORD fatTime = tmToDos(*timeinfo);
//
//	uFatTime u;
//
//	FileTimeToDosDateTime( &fTime, &u.date, &u.time );
//
//	assert( fatTime == u.fatTime );
//
//	return u.fatTime;
//#else
	time_t rawtime = time(&rawtime);
	tm *timeinfo = localtime(&rawtime);
	DWORD fatTime = tmToDos(*timeinfo);
	return fatTime;
//#endif
}

static FILETIME TimetToFileTime( time_t t )
{
    //LONGLONG ll = Int32x32To64(t, 10000000) + 116444736000000000;
    bc_u64 ll = (bc_u64)t * 10000000 + 116444736000000000;
    FILETIME ft;
    ft.dwLowDateTime = (bc_u32) ll;
    ft.dwHighDateTime = ll >>32;
    return ft;
}

static time_t FileTimeToTimeT( FILETIME ft )
{
    bc_u64 ll;
    ll = ((bc_u64)ft.dwHighDateTime) << 32;
	ll |= ft.dwLowDateTime;

	ll -= 116444736000000000;
	ll /= 10000000;

	return ( time_t )ll;
}

//
//	Local time note, DOS FAT assume local file time. But BCArchive uses UTC file time always.
//
//bc_file_time getCurrentTime()
//{
//	FILETIME ft;
//	
////	SYSTEMTIME st;
////	GetSystemTime( &st );
////	SystemTimeToFileTime( &st, &ft );
//
//	ft = TimetToFileTime( time( NULL ) );
//	
//	bc_file_time bct;
//	bct.highDateTime = ft.dwHighDateTime;
//	bct.lowDateTime = ft.dwLowDateTime;
//
//	return bct;
//}

WORD getDate( bc_file_time file_time, WORD *pTime )
{
	FILETIME ft;
	ft.dwHighDateTime = file_time.highDateTime;
	ft.dwLowDateTime = file_time.lowDateTime;

	time_t t = FileTimeToTimeT( ft );
	tm *gTime = localtime( &t );

	DWORD fatTime = tmToDos(*gTime);

	uFatTime u;
	u.fatTime = fatTime;

//#ifdef _WIN32
//	// See 'Local time note' above
//	FILETIME localFT;
//	FileTimeToLocalFileTime( &ft, &localFT );
//
//	BOOL ok = FileTimeToDosDateTime( &localFT, &u.date, &u.time );
//	if ( !ok )
//		{ assert(false); return 0; }
//
//	assert( u.fatTime == fatTime );
//#endif

	if ( pTime != NULL )
		*pTime = u.time;

	return u.date;
}

WORD getTime( bc_file_time file_time )
{
	WORD time;
	if ( getDate( file_time, &time ) == 0 )
		return 0;

	return time;
}

bc_file_time getBCTime( WORD date, WORD time )
{
	uFatTime u;
	u.date = date;
	u.time = time;

	tm t = dosToTm( u.fatTime );
	time_t mtime = mktime( &t );
	//tm *lt = localtime( time );

	FILETIME ft = TimetToFileTime( mtime );

#ifdef _WIN32

	FILETIME localFT = {0,0};
	BOOL ok = DosDateTimeToFileTime( date, time, &localFT );
	if ( !ok )
		assert(false);

	// See 'Local time note' above
	FILETIME x;
	LocalFileTimeToFileTime( &localFT, &x ); // &ft );

	assert( x.dwLowDateTime == ft.dwLowDateTime );
#endif

	bc_file_time file_time;
	file_time.highDateTime = ft.dwHighDateTime;
	file_time.lowDateTime = ft.dwLowDateTime;

	return file_time;
}


//virtual
int cFatFileObject::read(
	void *buff,		/* Pointer to data buffer */
	bc_u32 btr,		/* Number of bytes to read */
	bc_u32 *br		/* Pointer to number of bytes read */
	)
{
	static int count = 0;
	LIBLOG_INFO( "%d) Begin, position = %d, size = %d", ++count, (int)m_fil.fptr, (int)btr );

	DWORD pos = m_fil.fptr;

	FRESULT res = f_read( &m_fil, buff, btr, (UINT*)br, 1 );

//	int byteToPrint = 128;
//
//	LIBLOG_INFO("bytes read %d(0x%x), positions old = %d(0x%x), cur %d(0x%x)", (int)(*br), (unsigned int)(*br), (int)pos, (unsigned int)pos, (int)m_fil.fptr, (unsigned int)m_fil.fptr );
//
//	for( int p = 0; p < byteToPrint; p += 128 )
//	{
//		LIBLOG_INFO("from beg %08x, {%s}", (unsigned int)pos + p, bytes2string( (const unsigned char *)buff + p, *br,  128, true ) );
//	}
//	LIBLOG_INFO("from end %08x, {%s}", (unsigned int)m_fil.fptr,  bytes2string( (const unsigned char *)buff, *br, 128, false ) );


	LIBLOG_INFO("%d) End, res = %d, bytes read %d(0x%x)", count, (int)res, (int)(*br), (unsigned int)(*br) );

	return getIFSResult( res );
}
