/*
 *	Copyright (c) 2012 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	FAT File system class.
 */

#pragma once

#include "ifs.h"
#include "FatFs/ff.h"
#include <assert.h>
#include "cADB.h"
#include "equstrings.h"
#include <locale.h>
#include <time.h>

#ifndef FR_OK
#define FR_OK 0
#endif

#include "log.h"

#ifdef _LFN_UNICODE
#ifndef _UNICODE
#define _UNICODE
#endif
#endif


//bc_file_time getCurrentTime();
WORD getDate( bc_file_time file_time, WORD *pTime = NULL );
WORD getTime( bc_file_time file_time );
bc_file_time getBCTime( WORD date, WORD time );

static inline eifsResult getIFSResult( FRESULT res )
{
	return eifsResult( res );
}

class cFatFileObject: public iFileObject
{
public:
	FIL m_fil;

	~cFatFileObject()
	{
		clear();
	}

	cFatFileObject()
	{
		clear();
	}

	void clear()
	{
		memset( &m_fil, 0, sizeof( m_fil ) );
		m_attributes = -1;
		memset( &m_lw, 0, sizeof( m_lw ) );
		m_ct = m_la = m_lw;
	}

	//FRESULT f_read (FIL*, void*, UINT, UINT*);			/* Read data from a file */
	virtual int read(
		void *buff,		/* Pointer to data buffer */
		bc_u32 btr,		/* Number of bytes to read */
		bc_u32 *br		/* Pointer to number of bytes read */
		);

	// FRESULT f_lseek (FIL*, DWORD);						/* Move file pointer of a file object */
	virtual bc_u64 setPointer(
		bc_64 ofs,		/* File pointer from point defined by mode */
		eSeekMode mode = file_begin	/* one of enum above */
		)
	{
		LIBLOG_INFO( "Begin, ofs = %d, mode = %d, curPos = %d(0x%x)", (int)ofs, (int)mode, (int)( m_fil.fptr ), (unsigned int) m_fil.fptr  );

		if ( mode == cur_pos )
			ofs += m_fil.fptr;
		else
		if ( mode == file_end )
			ofs += m_fil.fsize;
		else // ( mode == file_begin )
		{
			// does nothing
		}

		LIBLOG_INFO( "Offset %x:%x", (unsigned int)( ofs >> 32 ), (unsigned int)ofs );

		DWORD dwOffset = (DWORD)ofs;

		if ( dwOffset != ofs )
		{
			LIBLOG_INFO( "Assertion! Offset  Hi = %x", (unsigned int)( ofs >> 32 ) );
			return (bc_64)-1;
		}

		FRESULT res = f_lseek (&m_fil, dwOffset);

		if ( res != FR_OK )
		{
			LIBLOG_INFO( "Assertion! res = %d", (int)res  );
			return (bc_64)-1;
		}

		LIBLOG_INFO( "End  curPos %d(0x%x)", (int)( m_fil.fptr ), (unsigned int) m_fil.fptr );

		return m_fil.fptr;
	}

	//FRESULT f_write (FIL*, const void*, UINT, UINT*);	/* Write data to a file */
	virtual int write(
		const void *buff,	/* Pointer to the data to be written */
		bc_u32 btw,			/* Number of bytes to write */
		bc_u32 *bw			/* Pointer to number of bytes written */
		)
	{
		FRESULT res = f_write ( &m_fil, buff, btw, (UINT*)bw );
		return getIFSResult( res );
	}

	// FRESULT f_sync (FIL*); /* Flush cached data of a writing file */
	virtual int sync()
	{
		FRESULT res = f_sync(&m_fil);
		return getIFSResult( res );
	}

	// FRESULT f_close (FIL*); /* Flush cached data of a writing file */
	virtual int close()
	{
		LIBLOG_INFO( "Begin %s", m_path.m_bodyA.m_body );

		FRESULT res = f_close(&m_fil);

		if ( res == FR_OK )
		{
			FILINFO fno;
			memset( &fno, 0, sizeof( fno ) );

			if ( FR_OK != f_stat( m_path.strT(), &fno ) )
			{
				assert( false );
			}
			else
			{
				if ( m_attributes != -1 )
				{
					fno.fattrib = (BYTE)m_attributes;
					f_chmod( m_path.strT(), fno.fattrib, 0xFF );
				}

				if ( m_lw.lowDateTime != 0 )
				{
					fno.fdate = getDate( m_lw, &fno.ftime );
				}

				f_utime( m_path.strT(), &fno );
			}
		}

		m_path.Free();

		LIBLOG_INFO( "End res = %d", res );

		return getIFSResult( res );
	}

	//FRESULT f_truncate (FIL*);							/* Truncate file */
	virtual int setEndOfFile()
	{
		FRESULT res = f_truncate( &m_fil );
		return getIFSResult( res );
	}

	virtual bc_u64 getSize( bc_u64 *pCurPos = 0 )
	{
		if ( pCurPos != 0 )
			*pCurPos = m_fil.fptr;

		return m_fil.fsize;
	}

	virtual bool isEOF()
	{
		return m_fil.fptr == m_fil.fsize;
	}

};

struct sFilInfo : public FILINFO
{
	char longName[ 512 ];

	sFilInfo()
	{
		memset( this, 0, sizeof( (*this) ) );
		lfname = (TCHAR*)longName;
		lfsize = sizeof( longName ) / sizeof( *longName );
	}
};

class cFatDirectoryObject : public iDirectoryObject, public FF_DIR
{
public:
	sFilInfo m_filInfo;

	cADB < char > m_path;
	//cUString m_path;
	const char *m_findFileName;
	wchar_t *m_folderPathW;
	wchar_t *m_longNameW;

	//static cFatDirectoryObject *createObject( const char *filePath )
	//{
	//	size_t pos = findLast( filePath, "\\/" );

	//	cFatDirectoryObject *dir = new cFatDirectoryObject( filePath, pos );

	//	if ( NULL == dir )
	//		return NULL;

	//	if ( NULL == dir->m_path.CreateCopy( filePath ) )
	//	{
	//		delete dir;
	//		return NULL;
	//	}

	//	if ( -1 == pos )
	//	{
	//		delete dir;
	//		return NULL;
	//	}

	//	dir->m_path.setAt( pos, '\0' );
	//	dir->m_findFileName = dir->m_path.strA() + pos + 1;

	//	return dir;
	//}

	virtual bc_u32 getFileAttributes()		{ return m_filInfo.fattrib; }
	virtual bc_file_time getCreationTime()	{ return getBCTime( m_filInfo.fdate, m_filInfo.ftime ); }
    virtual bc_file_time getLastAccessTime(){ return getBCTime( m_filInfo.fdate, m_filInfo.ftime ); }
    virtual bc_file_time getLastWriteTime()	{ return getBCTime( m_filInfo.fdate, m_filInfo.ftime ); }
	virtual bc_u64 getFileSize()			{ return m_filInfo.fsize; }
	virtual bc_u64 getCompressedFileSize()	{ return m_filInfo.fsize; }

	cFatDirectoryObject( const char *folderPath, int len = -1 )
		: iDirectoryObject( folderPath, len )
	{
		m_folderPathW = NULL;
		m_longNameW = NULL;
	}

	void clearLongName()
	{
		deleteWString( &m_longNameW );
	}

	~cFatDirectoryObject()
	{
		deleteWString( &m_folderPathW );
		clearLongName();
	}

	void deleteWString( wchar_t **ppStr )
	{
		if ( NULL == ppStr )
			return;

		if ( NULL == *ppStr )
			return;

		delete *ppStr;

		*ppStr = NULL;
	}

	wchar_t *createWString( const char *mbs )
	{
		/* Enable the Japanese locale and codepage */
		char *localeInfo = setlocale(LC_CTYPE, "Japanese_Japan.932");

		size_t requiredSize = mbstowcs(NULL, mbs, 0); // C4996
		/* Add one to leave room for the NULL terminator */
		wchar_t *pwc = new wchar_t[ requiredSize + 1];
		if ( pwc == 0 )
			return 0;

		if ( mbstowcs( pwc, mbs, requiredSize + 1 ) != requiredSize )
		{
			delete pwc;
			return NULL;
		}

		return pwc;
	}

	cUString _gfn;

	virtual const char *getFileName()
	{
#ifndef _UNICODE
		if ( ( 0 != m_filInfo.lfsize ) && ( 0 != m_filInfo.lfname[0] ) )
			return m_filInfo.lfname;

		return m_filInfo.fname;
#else
		return cUString::convertWcsToUTF8( _gfn.m_bodyA, (const UTF16 *)getFileNameW() );
#endif
	}

	virtual const bc_u16 *getFileNameW()
	{
#ifdef _UNICODE
		if ( ( 0 != m_filInfo.lfsize ) && ( 0 != m_filInfo.lfname[0] ) )
			return m_filInfo.lfname;

		return m_filInfo.fname;
#else
		return cUString::convertToWCS( _gfn.m_bodyW, getFileName() );
		//if ( m_longNameW == NULL )
		//	m_longNameW = createWString( getFileName() );

		//return m_longNameW;
#endif
	}

	virtual const char *getFolderPath()
	{
		if ( m_folderPath == NULL )
			return NULL;

		if ( m_folderPath[1] == ':' )
			return m_folderPath + 2;

		return m_folderPath;
	}

	virtual const bc_u16 *getFolderPathW()
	{
		if ( m_folderPathW == NULL )
			m_folderPathW = createWString( getFolderPath() );

		return (const bc_u16 *)m_folderPathW;
	}
};

class iFATFS : public iFileSystem, public FATFS
{
public:

	iFATFS( iVolume *volume )
		: iFileSystem( volume )
	{
		m_drv = getFirst();
	}

	/*-----------------------------------------------------------------------*/
	/* Create File System on the Drive                                       */
	/*-----------------------------------------------------------------------*/
	virtual int makeFS()
	{
		LIBLOG_INFO( "Begin %d", m_drv  );

		BYTE sfd = 0;		/* Partitioning rule 0:FDISK, 1:SFD */
		UINT au	= 4 * 1024;	/* Allocation unit size [bytes] */

		FRESULT res = f_mkfs( m_drv, sfd, au );

		LIBLOG_INFO( "End = %d", res  );

		return getIFSResult( res );
	}


	/**
	 * Mount/Unmount file system
	 */
	//FRESULT f_mount (BYTE, FATFS*);						/* Mount/Unmount a logical drive */
	virtual int mountFS()
	{
		LIBLOG_INFO( "Begin %d", m_drv  );

#if _FS_CACHE_FAT
		this->fatBuffer = NULL;
#endif

#if _FS_CACHE_CLUSTERS
		this->cCache = NULL;
#endif

#ifdef _UseTabLock
		this->sobj = 0;
#endif

		FRESULT res = f_mount( m_drv, this );

		if ( FR_OK == res )
			setMountedVolume( m_drv, m_volume );

		LIBLOG_INFO( "End = %d", res  );

		return getIFSResult( res );
	}

	int unMountFS()
	{
		LIBLOG_INFO( "Begin %d", m_drv  );

		enum { uf_allocate = 0, uf_free };

		updateFatSectors( m_drv, this, uf_free );
		cCluserCache_free( this );

		setMountedVolume( m_drv, NULL );

		FRESULT res = f_mount( m_drv, NULL );

		LIBLOG_INFO( "End = %d", res  );

		return getIFSResult( res );
	}

	//const char *getFullPath( cUString *adb, const char *filePath )
	//{
	//	adb->Free();

	//	if ( ( filePath[0] != 0 ) && ( filePath[1] == ':' ) )
	//		adb->CreateCopy( filePath );
	//	else
	//	{
	//		char drvName[3] = "0:";
	//		drvName[0] = '0' + m_drv;

	//		adb->AddString( drvName );

	//		if ( ( filePath[0] == '\\' ) || (  filePath[0] == '/' ) )
	//			filePath++;

	//		adb->AddString( "\\" );

	//		adb->AddString( filePath );
	//	}

	//	return adb->strA();
	//}

	//const char *getFullPath( cUString *adb, const char *dirPath, const char *fileName )
	//{
	//	if ( dirPath[0] == 0 )
	//		return getFullPath( adb, fileName );

	//	getFullPath( adb, dirPath );

	//	if ( fileName[0] != '\\' )
	//		adb->AddString( "\\" );

	//	adb->AddString( fileName );

	//	return adb->strA();
	//}

	/**
	 * Open directory object
	 */
	virtual iDirectoryObject *findOpen( const char *dirPath, const char *fileNamePattern )
	{
		LIBLOG_INFO( "Begin '%s', '%s'", dirPath, fileNamePattern  );

		cUString adb;
		dirPath = getFullPath( &adb, dirPath );

		cFatDirectoryObject *dir = new cFatDirectoryObject( dirPath );

		if ( NULL == dir )
		{
			LIBLOG_INFO( "dir == NULL "  );
			return NULL;
		}
#ifdef _UNICODE
		FRESULT res = f_opendir ( dir, cUString( dirPath ).strT() );
#else
		FRESULT res = f_opendir ( dir, dirPath );
#endif

		if ( FR_OK != res )
		{
			LIBLOG_INFO( "Assertion! f_opendir = %d ", res  );
			m_lastError = getIFSResult( res );
			delete dir;
			return NULL;
		}

		dir->m_findFileName = dir->m_path.CreateCopy( fileNamePattern );

		LIBLOG_INFO( "End"  );
		return dir;
	}

	virtual bool findNextFile( iDirectoryObject *pDirObject )
	{
		LIBLOG_INFO("Begin %x ", (unsigned int)pDirObject );
		if ( NULL == pDirObject )
		{
			LIBLOG_INFO("Assertion! pDirObject == NULL " );
			assert( false );
			return false;
		}

		//cFatDirectoryObject *dir = dynamic_cast < cFatDirectoryObject * > ( pDirObject );
		cFatDirectoryObject *dir = (cFatDirectoryObject *)pDirObject;

		if ( dir == NULL )
		{
			LIBLOG_INFO("Assertion! dir == NULL " );
			assert( false );
			return false;
		}

		dir->clearLongName();

		while( true )
		{
			FRESULT res = f_readdir( (FF_DIR*)dir, &(dir->m_filInfo) );

			if ( FR_OK != res )
			{
				LIBLOG_INFO("Assertion! f_readdir == %d ", res );
				m_lastError = getIFSResult( res );
				return false;

			}

			const char *fileName = dir->getFileName();

			if ( 0 == fileName[0] )
			{
				LIBLOG_INFO("fileName is empty" );
				return false;
			}

			if ( dir->isDirectory() )
			{
				if ( ( strcmp( fileName, "." ) == 0  ) ||
					 ( strcmp( fileName, ".." ) == 0 ) )
						continue;
			}

			if ( equStrings( fileName, dir->m_findFileName ) )
			{
				LIBLOG_INFO("Name found '%s' ", dir->getFileName() );
				return true;
			}

			continue;
		}

		return false;
	}

	virtual void findClose( iDirectoryObject *pDirObject )
	{
		if ( NULL == pDirObject )
			{ assert( false ); return; }

		delete pDirObject;
	}

	/**
	 * Open file
	 * @filePath   path to file.
	 * @mode       one or more from bc_file_open_mode enum.
	 */
	// FRESULT f_open (FIL*, const TCHAR*, BYTE);			/* Open or create a file */
	virtual iFileObject *openFile( const char *filePath, int mode )
	{
		cFatFileObject *fObj = new cFatFileObject();

		if ( NULL == fObj )
		{
			LIBLOG_INFO( "Assertion! fObj is NULL!" );
			assert( false );
			m_lastError = eifs_ALLOCATION_ERR;
			return NULL;
		}

		LIBLOG_INFO( "Begin '%s', mode %d", filePath, mode );

		filePath = getFullPath( &(fObj->m_path), filePath );

		LIBLOG_INFO( "Full path '%s'", filePath );

#ifdef _UNICODE
		FRESULT res = f_open( &fObj->m_fil, fObj->m_path.strT(), getMode( mode ) );
#else
		FRESULT res = f_open( &fObj->m_fil, filePath, getMode( mode ) );
#endif

		m_lastError = getIFSResult( res );

		if ( FR_OK == res )
		{
//#ifdef _DEBUG
//			fObj->setPointer( 2, fObj->cur_pos );
//			fObj->setPointer( 0, fObj->file_begin );
//#endif
			return fObj;
		}

		LIBLOG_INFO( "Assertion! Error %d in opening file '%s'", res, filePath );

		closeFile( fObj );

		return NULL;
	}

	virtual iFileObject *openFile( iDirectoryObject *pDirObject, int mode )
	{
		//cFatDirectoryObject *fatDirObj = dynamic_cast < cFatDirectoryObject * > ( pDirObject );
		cFatDirectoryObject *fatDirObj = ( cFatDirectoryObject * )pDirObject;

		if ( fatDirObj == NULL )
			return NULL;

		cUString adb;
		getFullPath( &adb, fatDirObj->getFolderPath(), fatDirObj->getFileName() );

		return openFile( adb.strA(), mode );
	}

	// FRESULT f_close (FIL*);								/* Close an open file object */
	virtual void closeFile( iFileObject *pFileObject )
	{
		if ( NULL == pFileObject )
		{
			LIBLOG_INFO( "Assertion! Object is NULL!" );
			assert( false );
			return;
		}

		LIBLOG_INFO( "Begin '%s'", ((cFatFileObject*)pFileObject)->m_path.m_bodyA.m_body  );

		int res = pFileObject->sync();

		LIBLOG_INFO( "Sync = %d", res );

		res = pFileObject->close();

		LIBLOG_INFO( "Close = %d", res );

		delete pFileObject;

		LIBLOG_INFO( "End"  );

	}

	//FRESULT f_unlink (const TCHAR*);					/* Delete an existing file or directory */
	virtual int deleteFile( const char *filePath )
	{
		cUString adb;
		filePath = getFullPath( &adb, filePath );

#ifdef _UNICODE
		FRESULT res = f_unlink ( adb.strT() );
#else
		FRESULT res = f_unlink ( filePath );
#endif
		return getIFSResult( res );
	}

	//FRESULT	f_mkdir (const TCHAR*);						/* Create a new directory */
	virtual int createDirectory( const char *dirPath )
	{
		cUString adb;
		dirPath = getFullPath( &adb, dirPath );

#ifdef _UNICODE
		FRESULT	res = f_mkdir ( adb.strT() );
#else
		FRESULT	res = f_mkdir ( dirPath );
#endif
		return getIFSResult( res );
	}

	//FRESULT f_utime (const TCHAR*, const FILINFO*);		/* Change timestamp of the file/dir */
	virtual int setFileTime( const char *path, bc_file_time file_time )
	{
		cUString adb;
		path = getFullPath( &adb, path );

		FILINFO fi;
		fi.fdate = getDate( file_time );
		fi.ftime = getTime( file_time );

#ifdef _UNICODE
		FRESULT	res = f_utime ( adb.strT(), &fi );
#else
		FRESULT	res = f_utime ( path, &fi );
#endif
		return getIFSResult( res );
	}

	//FRESULT f_rename (const TCHAR*, const TCHAR*);		/* Rename/Move a file or directory */
	virtual int renameFile( const char *oldPath, const char *newPath )
	{
		cUString adbO;
		oldPath = getFullPath( &adbO, oldPath );

		cUString adbN;
		newPath = getFullPath( &adbN, newPath );

		LIBLOG_INFO( "Begin '%s', '%s'",  oldPath, newPath );


#ifdef _UNICODE
		FRESULT res = f_rename ( adbO.strT(), adbN.strT() );
#else
		FRESULT res = f_rename (oldPath, newPath);
#endif

		LIBLOG_INFO( "End res = %d",  res );

		return getIFSResult( res );
	}

	//FRESULT f_getfree (const TCHAR*, DWORD*, FATFS**);	/* Get number of free clusters on the drive */
	virtual int getFreeSpace( bc_u64 *pFreeSpaceInBytes )
	{
		cUString adb;
		getFullPath( &adb, "/" );

		DWORD freeClusters = 0;

		FATFS *pFS =  this;
#ifdef _UNICODE
		FRESULT res = f_getfree ( adb.strT(), &freeClusters, &pFS );
#else
		FRESULT res = f_getfree ( adb.strA(), &freeClusters, &pFS );
#endif

		if ( res != FR_OK )
			return res;

		*pFreeSpaceInBytes = freeClusters;

		*pFreeSpaceInBytes *= pFS->csize;

#if _MAX_SS == 512
		*pFreeSpaceInBytes *=	512;
#else
		*pFreeSpaceInBytes *=	pFS->ssize;			/* Bytes per sector (512, 1024, 2048 or 4096) */
#endif

		return res;
	}
};

