/*
 *	Copyright (c) 2015 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	NTFS File system class.
 */

#include "ifsNtfs.h"

/*-----------------------------------------------------------------------*/
/* Create File System on the Drive                                       */
/*-----------------------------------------------------------------------*/
//virtual 
int iNTFS::makeFS()
{
	LIBLOG_INFO( "Begin %d", m_drv  );

	BYTE sfd = 0;		/* Partitioning rule 0:FDISK, 1:SFD */
	UINT au	= 4 * 1024;	/* Allocation unit size [bytes] */

	int res = -1; // f_mkfs( m_drv, sfd, au );

	LIBLOG_INFO( "End = %d", res  );

	return res; //getIFSResult( res );
}

extern int _mountFS( void **pVolumeCtx, void *pVolume, bc_u32 flags ); // flags -> 1 = writeProtected.
extern int _unMountFS( void *volumeCtx );
extern void *_findOpen( void *volumeCtx, const char *dirName );
extern int _findFile( void *inodeCtx, bc_64 *pos, __WIN32_FIND_DATAW *ffd );
extern void * _openFile( void *volumeCtx, const char *filePath );
extern void _closeFile( void *attributeCtx );
extern int _setFileTime( void *attributeCtx, bc_u64 crT, bc_u64 laT, bc_u64 lwT  );
extern int _getFileTime( void *attributeCtx, bc_u64 *crT, bc_u64 *laT, bc_u64 *lwT  );
extern int _readFile( void *attributeCtx, const bc_64 pos, void *b, bc_64 size, bc_64 *sizeRead );
extern int _writeFile( void *attributeCtx, const bc_64 pos, const void *b, bc_64 size, bc_64 *written );
extern bc_64 _getSize( void *attributeCtx );
extern int _truncateFile( void *attributeCtx, bc_64 pos );
extern int _deleteFile( void *volumeCtx, const char *filePath );
extern int _createDirectory( void *volumeCtx, const char *dirPath );
extern int _createFile( void *volumeCtx, const char *filePath );
extern int _setFileTime( void *volumeCtx, const char *filePath, bc_u64 fileTime );
extern int _renameFile( void *volumeCtx, const char *oldPath, const char *newPath );
extern int _getFreeSpace( void *volumeCtx, bc_u64 *pFS );

//static inline void updatePath( const char *&path )
//{
//	while( *path == '\\' )
//		path++;
//}

#define PATH_SEP '/'

static void updatePathADB( const char *&path, cADB < char > &adb )
{
	while( ( *path == '\\' ) || ( *path == PATH_SEP ) )
		path++;

	char *s = adb.AddString( path );

	for( int i = 0; s[i] != '\0'; i++ )
	{
		if ( s[i] == '\\' )
		{
			s[i] = PATH_SEP;
			path = s;
		}
	}
}

#define updatePath( path ) cADB < char > ADB##path; updatePathADB( path, ADB##path )


/**
 * Mount/Unmount file system
 */
//virtual
int iNTFS::mountFS()
{
	LIBLOG_INFO( "Begin %d", m_drv  );
	
	int res = _mountFS( &m_volCtx, (void *)(iVolume *)m_volume, m_volume->isWriteProtected() ? 1 : 0 );

	if ( FR_OK == res )
		setMountedVolume( m_drv, m_volume );

	LIBLOG_INFO( "End = %d", res  );

	return res;
}

int iNTFS::unMountFS()
{
	LIBLOG_INFO( "Begin %d", m_drv  );

	setMountedVolume( m_drv, NULL );

	int res = _unMountFS( m_volCtx );
	m_volCtx = NULL;

	LIBLOG_INFO( "End = %d", res  );

	return res;
}

/**
 * Open directory object
 */
//virtual 
iDirectoryObject *iNTFS::findOpen( const char *dirPath, const char *fileNamePattern )
{
	LIBLOG_INFO( "Begin '%s', '%s'", dirPath, fileNamePattern  );

	updatePath( dirPath );

	if ( strcmp( dirPath, "." ) == 0 )
			dirPath = ""; // Root folder.

	cNtfsDirectoryObject *dir = new cNtfsDirectoryObject( dirPath );

	if ( NULL == dir )
	{
		LIBLOG_INFO( "dir == NULL "  );
		return NULL;
	}

	m_volume->setCurrentReadCachePriority( 1 );
	dir->m_inode = _findOpen( m_volCtx, dirPath );
	m_volume->setCurrentReadCachePriority( 0 );

	if ( dir->m_inode == NULL )
	{
		LIBLOG_INFO( "Assertion! f_opendir == NULL " );
		delete dir;
		return NULL;
	}

	dir->m_findFileName = dir->m_path.CreateCopy( fileNamePattern );

	LIBLOG_INFO( "End"  );
	return dir;
}

//virtual
bool iNTFS::findNextFile( iDirectoryObject *pDirObject )
{
	LIBLOG_INFO("Begin %x ", (unsigned int)pDirObject );
	if ( NULL == pDirObject )
	{
		LIBLOG_INFO("Assertion! pDirObject == NULL " );
		assert( false );
		return false;
	}

	//cFatDirectoryObject *dir = dynamic_cast < cFatDirectoryObject * > ( pDirObject );
	cNtfsDirectoryObject *dir = (cNtfsDirectoryObject *)pDirObject;

	if ( dir == NULL )
	{
		LIBLOG_INFO("Assertion! dir == NULL " );
		assert( false );
		return false;
	}

	dir->clearLongName();

	while( true )
	{
		int res = _findFile( dir->m_inode, &(dir->m_findPos), &(dir->m_findData ) );

		(dir->m_findPos)++;

		if ( FR_OK != res )
		{
			LIBLOG_INFO("Warning! _findFile == %d ", res );
			m_lastError = res;
			return false;
		}

		const char *fileName = dir->getFileName();

		LIBLOG_INFO("_findFile - %s ", fileName );

		if ( 0 == fileName[0] )
		{
			LIBLOG_INFO("fileName is empty" );
			return false;
		}

		if ( dir->isDirectory() )
		{
			if (  strcmp( fileName, "." ) == 0  )
			{
				dir->m_findData.cFileName[0] = 0;
				if (  0 == dir->m_findFileName[0] )
				{
					LIBLOG_INFO("Just open directory attribute" );
					return true;
				}

				continue;
			}
			
			if ( strcmp( fileName, ".." ) == 0 )
				continue;
		}

#ifndef FILE_ATTR_HIDDEN
#define FILE_ATTR_HIDDEN 0x00000002
#define FILE_ATTR_SYSTEM 0x00000004
#endif

		// Skip metedata files.
		if ( ( dir->m_findData.dwFileAttributes & ( FILE_ATTR_HIDDEN | FILE_ATTR_SYSTEM ) )  == (FILE_ATTR_HIDDEN | FILE_ATTR_SYSTEM))
			continue;

		if ( equStrings( fileName, dir->m_findFileName ) )
		{
			LIBLOG_INFO("Name found '%s' ", dir->getFileName() );
			return true;
		}

		continue;
	}

	return false;
}

//virtual
void iNTFS::findClose( iDirectoryObject *pDirObject )
{
	if ( NULL == pDirObject )
		{ assert( false ); return; }

	delete pDirObject;
}

/**
 * Open file
 * @filePath   path to file.
 * @mode       one or more from bc_file_open_mode enum.
 */
//virtual
iFileObject *iNTFS::openFile( const char *filePath, int mode )
{
	LIBLOG_INFO( "Begin '%s', mode %x", filePath, mode );

	updatePath( filePath );

	bool bCreate = false;

	if ( ( mode & eBCDispCreateNew ) != 0 )
	{
		if ( fileExists( filePath ) )
		{
			LIBLOG_INFO( "Warning! Mode 'eBCDispCreateNew' does not allow to open existing file '%s'!", filePath );
			return NULL;
		}

		bCreate = true;
	}
	else
	if ( ( mode & eBCDispCreateAlways ) != 0 )
	{
		if ( fileExists( filePath ) )
		{
			int res = deleteFile( filePath );
			if ( res != 0 )
			{
				LIBLOG_INFO( "Assertion! Error %d in deleting existing file '%s'!", res, filePath );
				return NULL;
			}
		}

		bCreate = true;
	}
	else
	if ( mode == ( eBCModeWrite | eBCDispOpenAlways ) )
	{
		if ( !fileExists( filePath ) )
			bCreate = true;
	}

	if ( bCreate )
	{
		int res = _createFile( m_volCtx, filePath );
		if ( res != 0 )
		{
			LIBLOG_INFO( "Assertion! Error %d in creating file '%s'!", res, filePath );
			return NULL;
		}
	}


	cNtfsFileObject *fObj = new cNtfsFileObject();

	if ( NULL == fObj )
	{
		LIBLOG_INFO( "Assertion! fObj is NULL!" );
		assert( false );
		m_lastError = eifs_ALLOCATION_ERR;
		return NULL;
	}

	fObj->m_attrCtx = _openFile( m_volCtx, filePath );

	fObj->m_mode = (bc_file_open_mode)mode;

	if ( fObj->m_attrCtx != NULL )
	{
		_getFileTime( fObj->m_attrCtx, (bc_u64*)&fObj->m_ct, (bc_u64*)&fObj->m_la, (bc_u64*)&fObj->m_lw );
		return fObj;
	}

	LIBLOG_INFO( "Assertion! Error in opening file '%s'", filePath );

	closeFile( fObj );

	return NULL;
}

//virtual
iFileObject *iNTFS::openFile( iDirectoryObject *pDirObject, int mode )
{
	cNtfsDirectoryObject *fatDirObj = ( cNtfsDirectoryObject * )pDirObject;

	if ( fatDirObj == NULL )
		return NULL;

	cUString adb( fatDirObj->getFolderPath() );

	if ( cUString::getWcsLen( adb.m_bodyW.m_body ) != 0 )
		adb.AddString( "\\" );

	adb.AddString( fatDirObj->getFileName() );

	return openFile( adb.strA(), mode );
}

/* Close an open file object */
//virtual
void iNTFS::closeFile( iFileObject *pFileObject )
{
	if ( NULL == pFileObject )
	{
		LIBLOG_INFO( "Assertion! Object is NULL!" );
		assert( false );
		return;
	}

	LIBLOG_INFO( "Begin '%s'", ((cNtfsFileObject*)pFileObject)->m_path.m_bodyA.m_body  );

	int res = pFileObject->sync();

	LIBLOG_INFO( "Sync = %d", res );

	res = pFileObject->close();

	LIBLOG_INFO( "Close = %d", res );

	delete pFileObject;

	LIBLOG_INFO( "End"  );

}

/* Delete an existing file or directory */
//virtual
int iNTFS::deleteFile( const char *filePath )
{
	updatePath( filePath );

	int res = _deleteFile( m_volCtx, filePath );

	return res;
}

/* Create a new directory */
//virtual
int iNTFS::createDirectory( const char *dirPath )
{
	updatePath( dirPath );

	int res = _createDirectory( m_volCtx, dirPath );
	return res;
}

/* Change timestamp of the file/dir */
//virtual
int iNTFS::setFileTime( const char *path, bc_file_time file_time )
{
	updatePath( path );

	int res = _setFileTime( m_volCtx, path, *(bc_u64*)&file_time );
	return res;
}

/* Rename/Move a file or directory */
//virtual
int iNTFS::renameFile( const char *oldPath, const char *newPath )
{
	LIBLOG_INFO( "Begin '%s', '%s'",  oldPath, newPath );

	updatePath( oldPath );
	updatePath( newPath );

	int res = _renameFile( m_volCtx, oldPath, newPath );

	LIBLOG_INFO( "End res = %d",  res );

	return res;
}

/* Get number of free clusters on the drive */
//virtual
int iNTFS::getFreeSpace( bc_u64 *pFreeSpaceInBytes )
{
	int res = _getFreeSpace( m_volCtx, pFreeSpaceInBytes );

	return res;
}


//virtual
int cNtfsFileObject::close()
{
	LIBLOG_INFO( "Begin %s", m_path.m_bodyA.m_body );

	if ( m_attrCtx == NULL )
	{
		LIBLOG_INFO( "Assertion! Attributes context is NULL" );
		return -1;
	}

	_setFileTime( m_attrCtx, *(bc_u64*)&m_ct, *(bc_u64*)&m_la, *(bc_u64*)&m_lw );

	_closeFile( m_attrCtx );
	m_attrCtx = NULL;

	LIBLOG_INFO( "End" );

	return 0;
}

//virtual
int cNtfsFileObject::write(
	const void *buff,	/* Pointer to the data to be written */
	bc_u32 btw,			/* Number of bytes to write */
	bc_u32 *bw			/* Pointer to number of bytes written */
	)
{
	LIBLOG_INFO( "Begin %s", m_path.m_bodyA.m_body );

	if ( m_attrCtx == NULL )
	{
		LIBLOG_INFO( "Assertion! Attributes context is NULL" );
		return -1;
	}

	if ( ( m_mode & eBCModeWrite ) == 0 )
	{
		LIBLOG_INFO( "Assertion! Mode %x is invalid", m_mode );
		return -1;
	}

	bc_64 btw64 = btw;
	bc_64 bw64 = 0;
	int res = _writeFile( m_attrCtx, m_pos, buff, btw64, &bw64 );
	m_pos += bw64;

	*bw = (bc_u32)bw64;

	LIBLOG_INFO( "End res = %d", res );

	return res;
}

//virtual
int cNtfsFileObject::read(
	void *buff,		/* Pointer to data buffer */
	bc_u32 btr,		/* Number of bytes to read */
	bc_u32 *br		/* Pointer to number of bytes read */
	)
{
	LIBLOG_INFO( "Begin %s", m_path.m_bodyA.m_body );

	if ( m_attrCtx == NULL )
	{
		LIBLOG_INFO( "Assertion! Attributes context is NULL" );
		return -1;
	}

	if ( ( m_mode & eBCModeRead ) == 0 )
	{
		LIBLOG_INFO( "Assertion! Mode %x is invalid", m_mode );
		return -1;
	}

	bc_64 btr64 = btr;
	bc_64 br64 = 0;
	int res = _readFile( m_attrCtx, m_pos, buff, btr64, &br64 );
	m_pos += br64;

	*br = (bc_u32)br64;

	LIBLOG_INFO( "End res = %d", res );

	return res;
}

//virtual
bc_u64 cNtfsFileObject::getSize( bc_u64 *pCurPos ) // == 0
{
	if ( pCurPos != NULL )
		*pCurPos = m_pos;

	return _getSize( m_attrCtx );
}

//virtual
bc_u64 cNtfsFileObject::setPointer(
	bc_64 ofs,		/* File pointer from point defined by mode */
	eSeekMode mode	/* one of enum above  = file_begin */ 
	)
{
	if ( mode == cur_pos )
		ofs += m_pos;
	else
	if ( mode == file_end )
		ofs += getSize();
	else // ( mode == file_begin )
	{
		// does nothing
	}
	
	m_pos = ofs;

	return m_pos;
}

//virtual
int cNtfsFileObject::setEndOfFile()
{
	return _truncateFile( m_attrCtx, m_pos );
}

