/*
 * log.h
 *	Allows Android log from c ( not c++ ) files
 *  Created on: 16.07.2014
 *      Author: pav
 */

#ifndef LOG_H_
#define LOG_H_

char *bytes2string( const unsigned char *buffer, int bufferSize, int sizeToFill, int fromBegin );

#ifdef _ANDROID_LOG

#include <android/log.h>
#define LIBLOG_INFO(format, args...) \
	__android_log_print(ANDROID_LOG_INFO, __FILE__, "%d: %s " #format, __LINE__, __FUNCTION__ , ##args);

#define LIBLOG_ERROR(format, args...) \
    __android_log_print(ANDROID_LOG_ERROR, __FILE__, "%d: %s " #format, __LINE__, __FUNCTION__ , ##args);

#define LIBLOG_WARN(format, args...) \
    __android_log_print(ANDROID_LOG_WARN, __FILE__, "%d: %s " #format, __LINE__, __FUNCTION__ , ##args);

#define LIBLOG_DEBUG(format, args...) \
    __android_log_print(ANDROID_LOG_DEBUG, __FILE__, "%d: %s " #format, __LINE__, __FUNCTION__ , ##args);


#elif defined(BC_LINUX)
#include <stdio.h>

#define __android_log_print(...) (void)0

#define LIBLOG_ERROR(format, args...) \
    fprintf(stderr, "Error: %s:%d:%s(): " #format "\n", __FILE__, __LINE__, __FUNCTION__, ##args);

#define LIBLOG_WARN(format, args...) \
    fprintf(stderr, "Warn: %s:%d:%s(): " #format "\n", __FILE__, __LINE__, __FUNCTION__, ##args);

#define LIBLOG_INFO(format, args...) \
    fprintf(stderr, "Info: %s:%d:%s(): " #format "\n", __FILE__, __LINE__, __FUNCTION__, ##args);

#define LIBLOG_DEBUG(format, args...) \
    fprintf(stderr, "Debug: %s:%d:%s(): " #format "\n", __FILE__, __LINE__, __FUNCTION__, ##args);

#else

#define __android_log_print(...) void(0)
#define LIBLOG_INFO(...) void(0)
#define LIBLOG_ERROR(...) void(0)
#endif


#ifdef __cplusplus
extern "C" {
#endif

int writeToLog( const char *tag,  const char *fmt, int i, int j );
int writeToLogW( const char *tag,  const char *fmt, int i, unsigned short *ws );

#ifdef __cplusplus
}
#endif



#endif /* LOG_H_ */
