/*
 * Copyright 2014 Jetico, Inc. All rights reserved.
 *	Class cFileChannel
 *		Allows access to file by iChannel interface.
 */

#pragma once
#pragma warning (disable: 4996)

#include "iChannel.h"
#include "file.h"

class cFileChannel : public iChannel
{
	const char *m_path;
	bc_file_handle m_handle;

public:

	void setPath( const char *path )
	{
		freePath();

		if ( path != NULL )
			m_path = strdup( path );
	}

	void freePath()
	{
		if ( m_path != NULL )
			free( (void *)m_path );

		m_path = NULL;
	}
	
	cFileChannel( const char *path = NULL )
	{
		m_path = NULL;
		setPath( path );
		m_handle = 0;
	}

	virtual ~cFileChannel()
	{
		close();
		freePath();
	}

	/**
	 * Repositions the offset of the open file.
	 * @param offset        Offset.
	 * @param seek          Describes position, from which offset is defined.
	 * @return              Current position of pointer from the beginning of file 
	 *                      or BC_FILE_ERR, if error occured.
	 */
	virtual bc_u64 setPosition( bc_u64 offset, bc_file_seek seek )
	{
		return BCFileSeek( &m_handle, offset, seek );
	}

	/**
	 * Reads/writes a part of Channel with offset from the beginning of the Channel.
	 * @param buffer        Data buffer.
	 * @param byteCount     Data buffer size in bytes.
	 * @param offset        Number of bytes from the beginning of file 
	 *                      that must be skipped before starting reading.
	 * @return              Number of bytes actually read or BC_FILE_ERR, if error occured.
	 */
	virtual bc_u64 readBytes( void *buffer, bc_u64 byteCount, bc_u64 offset )
	{
		return BCFileReadWithOffset( &m_handle, buffer, byteCount, offset );
	}

	virtual bc_u64 writeBytes( void *buffer, bc_u64 byteCount, bc_u64 offset )
	{
		return BCFileWriteWithOffset( &m_handle, buffer, byteCount, offset );
	}

	/**
	 * Reads/writes a part of Channel at current posotion.
	 * @param buffer        Data buffer.
	 * @param byteCount     Data buffer size in bytes.
	 * @param offset        Number of bytes from the beginning of file 
	 *                      that must be skipped before starting reading.
	 * @return              Number of bytes actually read or BC_FILE_ERR, if error occured.
	 */
	virtual bc_u64 readBytes( void *buffer, bc_u64 byteCount )
	{
		return BCFileRead( &m_handle, buffer, byteCount );
	}

	virtual bc_u64 writeBytes( void *buffer, bc_u64 byteCount )
	{
		return BCFileWrite( &m_handle, buffer, byteCount );
	}

	virtual void sync()
	{
		BCFileSync( &m_handle );
	}

	virtual bool open( int mode ) // See enum BCFileMode
	{
		if ( 0 != m_handle )
			return true;

		return (kBCFileResult_Success == BCFileOpen( m_path, mode, &m_handle));
	}

	virtual void close()
	{
		if ( 0 != m_handle )
			BCFileClose( &m_handle );

		m_handle = 0;
	}

	virtual void deleteFile()
	{
		if ( m_path == 0 )
			return;
		
		BCFileDelete( m_path );
	}
};
