/*
 * Copyright 2014 Jetico, Inc. All rights reserved.
 * User defined password get class.
 *	Created - 04.12.2014
 */

#pragma once

#include "password.h"
#include "helpers.h"

class iChannel;

class cPwdGet
{
	cPwdGet *m_prev;

public:

	static cPwdGet * m_current;
	const char * m_query;

	void init()
	{
		m_keyBlockChannel = NULL;
        m_prev = m_current;
        m_current = this;
    }

	virtual ~cPwdGet()
	{
        m_current = m_prev;
	}

	//
	//	Sample:
	//	cPwdGet pwd( "Enter pasword" );
	//
	//	int cPwdGet::pwdGet()
	//	{
	//		printf( "%s>", m_query );
	//		gets( m_password.buf );
	//		return 0;
	//	}
	//
	
    cPwdGet( const char * query = "" )
	{
		init();
        m_query = query;
        m_mainPartPassword = true;
	}

	//
	//	getPassword
	//		Used several times for one object.
	//		The prevFailed is true if previous call was not success.
	//
    virtual BCContainerResult getPassword ( bool prevFailed = false ) = 0;
    
	//
	//	Version 9 of BwestCrypt for Windows allows to mount from key block backup
	//		The readDBFromKeyBlockBackup reads data block from back up channel.
	//
	virtual iChannel * getKeyBlockChannel()	{
        return m_keyBlockChannel;
    }
    virtual void setKeyBlockChannel(iChannel * _keyBlockChannel) {
        m_keyBlockChannel = _keyBlockChannel;
    }


	//
	//	Version 9 of BwestCrypt for Windows allows to generate additional salt from key files
	//		The generateSaltBuffer processes this channels and generates required salt
	//
	virtual void setKeyFilesTable( iChannel ** channelsT, bc_u32 channelsQ ) {
		m_password.m_keyFilesT = channelsT;
		m_password.m_keyFilesQ = channelsQ;
    }
    
	virtual void setPublicKeyTable( iChannel ** channelsT, bc_u32 channelsQ ) {
		m_password.m_publicKeyT = channelsT;
		m_password.m_publicKeyQ = channelsQ;
    }

	virtual void setSecretKeyTable( iChannel ** channelsT, bc_u32 channelsQ ) {
		m_password.m_secretKeyT = channelsT;
		m_password.m_secretKeyQ = channelsQ;
    }

    BCPassword & getPasswordObject() {
        return m_password;
    }
    
    const char * getPasswordString() const {
        return m_password.buf;
    }

    void setPasswordString( const char *pwdString ) {
		m_password.set( pwdString );
	}
    
    BCContainerContext & getContainerContext() {
        return m_containerContext;
    }
    
    bool isMainVolumePassword() const {
        return m_mainPartPassword;
    }

    void setIsMainVolumePassword(bool _mainPart) {
        m_mainPartPassword = _mainPart;
    }
    
    void clear() {
        init();
        m_password.clear();
    }
    
    bool isEmpty() {
        return strlen(getPasswordString()) == 0;
    }

protected:
    BCContainerContext m_containerContext;
    BCPassword m_password;
    
private:
    iChannel * m_keyBlockChannel;
    bool m_mainPartPassword;
};
