/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 */

#include <string.h>

#include "cipher_kernel.h"
#include "kernel/BCKernel.h"

#include "../certificate/CertificateLib/symmetricAlgs/symmetricAlgs.h"

/// Zero pool for NULL passed as pool parameter.
// static const char s_zero_pool[kBCKernel_PoolSize] = {0};


CipherKernelModule::CipherKernelModule()
{
	memset(&m_alg, 0, sizeof(m_alg));
}

CipherKernelModule::~CipherKernelModule()
{
}


bc_cipher_result CipherKernelModule::lookupAlgId( bc_alg_id alg_id, BCAlgInfo *alg_info ) const
{
    return BCKernelLookupAlgById(alg_id, alg_info);
}

bc_cipher_result CipherKernelModule::createCipherKey( const bc_u8 *key, cipher_key_handle *key_handle )
{
//    bc_cipher_result rc = BCKernelStoreKey((const char*)key, keySize(), &key_handle);
//    return rc;
    return kBCCipher_Success;
}

bc_cipher_result CipherKernelModule::deleteCipherKey( cipher_key_handle *key_handle )
{
//    bc_cipher_result rc = BCKernelFreeKey(key_handle);
//    return rc;
    return kBCCipher_Success;
}

void CipherKernelModule::process(bc_u64 iv, bc_proc_type type, const bc_u8* key, size_t keysize, const bc_u8* pool, void* data, size_t nbytes)
{
    if(keysize < this->keySize()) {
        throw CipherException(kBCCipher_InvalidKey);
    }
    
    if(nbytes % this->blockSize()) {
        throw CipherException(kBCCipher_InvalidBuffer);
    }
    
    bc_cipher_result rc = BCKernelProcessWithKey(m_alg.id, m_mode, type, (const char*)key, keysize, iv, (const char*)pool, (char*)data, nbytes);
    if(rc != kBCCipher_Success) {
        throw CipherException(rc);
    }
}


bc_cipher_result CipherKernelModule::storeKey(const bc_u8* key, size_t keylen, const bc_u8 *pool, bc_key_handle *out_handle)
{
    return BCKernelStoreKey((const char*)key, keylen, pool, out_handle);
}


bc_cipher_result CipherKernelModule::freeKey(bc_key_handle key_handle)
{
    return BCKernelFreeKey(key_handle);
}


void CipherKernelModule::encrypt_ecb( const bc_u8 *SrcBuffer, bc_u8 *DstBuffer, cipher_key_handle key_handle )
{
    HSA sa = (HSA)key_handle;
    if ( sae_ok != sa_encrypt( sa, DstBuffer, SrcBuffer, m_alg.block_size ) )
        throw CipherException( kBCCipher_NotInitialized );
}


void CipherKernelModule::decrypt_ecb( const bc_u8 *SrcBuffer, bc_u8 *DstBuffer, cipher_key_handle key_handle )
{
    HSA sa = (HSA)key_handle;
    if ( sae_ok != sa_decrypt( sa, DstBuffer, SrcBuffer, m_alg.block_size ) )
        throw CipherException( kBCCipher_NotInitialized );
}


void CipherKernelModule::encrypt_cbc( const bc_u8 *IVector, const bc_u8 *SrcBuffer, bc_u8 *DstBuffer, size_t Length, cipher_key_handle key_handle )
{
    HSA sa = (HSA)key_handle;
    if ( sae_ok != sa_encryptCBC( sa, IVector, DstBuffer, SrcBuffer, Length ) )
        throw CipherException( kBCCipher_NotInitialized );
}


void CipherKernelModule::decrypt_cbc( const bc_u8 *IVector, const bc_u8 *SrcBuffer, bc_u8 *DstBuffer, size_t Length, cipher_key_handle key_handle )
{
    HSA sa = (HSA)key_handle;
    if ( sae_ok != sa_decryptCBC( sa, IVector, DstBuffer, SrcBuffer, Length ) )
        throw CipherException( kBCCipher_NotInitialized );
}


bc_cipher_result CipherKernelModule::getAlgList( BCAlgInfo **pAlgsT, size_t *pAlgsQ )
{
	*pAlgsT = NULL;
	*pAlgsQ = 0;

	bc_cipher_result rc = BCKernelListAlgs( pAlgsT, pAlgsQ );
	return static_cast<bc_cipher_result>(rc);
}

void CipherKernelModule::freeAlgList( BCAlgInfo *pAlgsT, size_t pAlgsQ )
{
	BCKernelFreeAlgList( pAlgsT );
}
