/*******************************************************************
 *	Copyright (c) 1994-2012 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	File:		08.10.2012 encmode/bc_alg.cpp derived form driver/bc_alg.c
 *
 *	Description:	algorithm wrapper for module building
 *
 *	Scope:		BestCrypt algorithms drivers
 *
 *	Platforms:	Linux
 *
 *	Author:		Nail R. Kaipov
 *
 *	Created:	10-Nov-1998
 *
 *	Revision:	??-??-2009 Evgeny Yakovlev.
 *			lrw, xts added.
 *			25-jun-2009 evgeny yakovlev.
 *			refactored into single algorithm core
 *			08.10.2012 Alexander Pichuev.
 *			encyption modes realisation same as in kerlen mode driver.
 *
 *******************************************************************/

#include "gf_mul.h"
//#include "bc_cfg.h"
#include "bc_alg.h"
#include "../sector2iv.h"
#include "../cipherbase.h"
#ifdef __APPLE__
#include <malloc/malloc.h>
#else
#include <malloc.h>
#endif
/*
#ifdef _ANDROID_LOG
#include <android/log.h>
#define LIBLOG_INFO(format, args...) \
    __android_log_print(ANDROID_LOG_INFO, __FILE__, "%d: %s " #format, __LINE__, __FUNCTION__ , ##args);
#else
#define LIBLOG_INFO(...) (void)0
#endif
*/
#define POOL_SIZE_BYTES		512

#define XTS_KEY2_OFFSET 	32


typedef enum
{
	IV_4X16_0 = 0,	// 4 sections by 16 bits starting from 0
	IV_1X64_1 = 1,	// 1 section by 64 bits starting from 1
} bc_iv_format_t;


static void shred (bc_u8 **ptr, size_t len)
{
	if (!ptr || !*ptr) return;
	memset(*ptr, 0, len);
	free(*ptr);
	*ptr = NULL;
}


//int bc_alg_test_key (struct bc_algorithm* alg, bc_key_handle kh)
//{
//	if (kh >= KEY_TABLE_LENGTH)
//		return -EINVAL;
//
//        if (!alg->key_table[kh].key_extended)
//		return -EINVAL;
//
//	return 0;
//}
//

//int bc_alg_lock_key (struct bc_algorithm* alg, bc_key_handle kh, int lock)
//{
//	int error;
//
//	error = bc_alg_test_key(alg, kh);
//
//	if (error)
//		return error;
//
//	if (lock)
//		alg->key_table[kh].lock++;
//	else
//		alg->key_table[kh].lock--;
//
//	return 0;
//}

//int bc_alg_free_key (struct bc_algorithm* alg, bc_key_handle kh)
//{
//	int err;
//	struct bc_key_element* key_table = alg->key_table;
//
//	err = bc_alg_test_key(alg, kh);
//	if (err)
//		return err;
//
//	key_table[kh].lock--;
//	if(key_table[kh].lock > 0)
//		return 0;
//
//       
//	shred(&(key_table[kh].key_extended), alg->ops->ext_key_size_bytes);
//	shred(&(key_table[kh].pool), POOL_SIZE_BYTES);
//	shred(&(key_table[kh].lrw_context), sizeof(GfmulCtx));
//	shred(&(key_table[kh].xts_context), alg->ops->ext_key_size_bytes);
//
//	memset(&(key_table[kh]), 0, sizeof(struct bc_key_element));
//
//#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
//	module_put(alg->ops->owner);
//#else
//	__MOD_DEC_USE_COUNT(alg->ops->owner);
//#endif
//	return 0;
//}

bc_cipher_result bc_alg_lock_key ( CipherBase *cipher, struct_bc_key *bc_key, int lock )
{
	if ( !bc_key )
		return kBCCipher_NotInitialized;

	if ( lock )
		bc_key->lock++;
	else
		bc_key->lock--;

	return kBCCipher_Success;
}

bc_cipher_result bc_alg_free_key( CipherBase *cipher, struct_bc_key *bc_key )
{
	if ( !bc_key )
		return kBCCipher_NotInitialized;

	bc_key->lock--;
	if ( bc_key->lock > 0 )
		return kBCCipher_Success;

	cipher->deleteCipherKey( &(bc_key->key_extended) );
	shred(&(bc_key->lrw_context), sizeof(GfmulCtx));
	cipher->deleteCipherKey( &(bc_key->xts_context) );
	shred(&(bc_key->pool), POOL_SIZE_BYTES);
	memset( bc_key, 0, sizeof(struct_bc_key));
       
	return kBCCipher_Success;
}

bc_cipher_result bc_alg_make_key( CipherBase *cipher, const bc_u8* key, size_t keylen, const bc_u8* pool, struct_bc_key** out_handle )
{
	bc_cipher_result	error = kBCCipher_NotInitialized;
	const bc_u8*		key_2 = NULL;
	struct_bc_key*		elem = NULL;
        
    if ((NULL == key) || (NULL == out_handle)) // || (NULL == pool)
        return kBCCipher_InvalidArgument;
        
	// check length
	size_t byteLen = ( cipher->m_alg.key_length + 7 ) / 8;
	if ( keylen < byteLen )
        return kBCCipher_InvalidArgument;

	// keylen = (keylen + 7) >> 3; /* convert to bytes */

	elem = (struct_bc_key*)calloc( sizeof( struct_bc_key ), 1 );
	if ( !elem ) {
		error = kBCCipher_NoMemory;
		goto error_out;
	}

	error = cipher->createCipherKey( key, &elem->key_extended );
	if ( kBCCipher_Success != error )
		goto error_out;

	error = cipher->createCipherKey( &(key[XTS_KEY2_OFFSET]), &elem->xts_context );
	if ( kBCCipher_Success != error )
		goto error_out;

	if ( NULL == pool ) {
		error = kBCCipher_InvalidArgument;
		goto error_out;
	}
	else {
		elem->pool = (bc_u8*)calloc(POOL_SIZE_BYTES, 1);
		if ( !elem->pool ) {
			error = kBCCipher_NoMemory;
			goto error_out;
		}

		memcpy(elem->pool, pool, POOL_SIZE_BYTES);
	}

	elem->lrw_context = (bc_u8*)calloc(sizeof(GfmulCtx), 1 );

	if (!elem->key_extended  || !elem->lrw_context || !elem->xts_context || !key) {
		error = kBCCipher_NoMemory;
		goto error_out;
	}
        
	// get Key2 after Key1 in raw buffer
	key_2 = &key[keylen - cipher->blockSize()]; //alg->ops->block_size_bytes];

	if( cipher->blockSize() == 8 ) { // alg->ops->block_size_bytes == 8 ) {
		GFMUL_InitCtxBlock8( key_2, (GfmulCtx*) elem->lrw_context );
	} else if ( cipher->blockSize() == 16 ) { // alg->ops->block_size_bytes == 16 ) {
		GFMUL_InitCtxBlock16( key_2, (GfmulCtx*) elem->lrw_context );
	} else {
            error = kBCCipher_InvalidKey;
            goto error_out;
	}

	elem->lock = 1;

	*out_handle = elem;

	return kBCCipher_Success;

error_out:
	bc_alg_free_key( cipher, elem );
	return error;
}


//int bc_alg_make_key(struct bc_algorithm* alg, bc_u8* key, size_t keylen, bc_u8* pool, bc_key_handle* out_handle)
//{
//    
//	bc_key_handle		kh;
//	int			error = 0;
//	bc_u8* 		key_2 = NULL;
//	struct bc_key_element* 	table;
//	struct bc_key_element* 	elem;        
//        
//        BC_DEBUG_PRINT(("KeyMake: Entering.\n"));
//        
//        if ((NULL == key) || (NULL == pool) || (NULL == out_handle))
//        {
//            BC_DEBUG_PRINT(("KeyMake: invalid value came."));
//            return -EINVAL;
//        }
//        
//	// check length
//	if (keylen < alg->ops->key_size_bits)
//        {
//            BC_DEBUG_PRINT(("KeyMake: incorrect key size."));
//            return -EINVAL;
//        }
//	keylen = (keylen + 7) >> 3; /* convert to bytes */
//	table = alg->key_table;
//
//        BC_DEBUG_PRINT(("KeyMake: searching for empty entry.\n"));
//
//	// find a place for a key
//	for (kh = 0; kh < KEY_TABLE_LENGTH; ++kh) {
//		if ( table[kh].key_extended == NULL ) break;
//	}
//
//	if (kh >= KEY_TABLE_LENGTH || table[kh].key_extended || table[kh].pool)
//		return -EBUSY;
//
//
//        BC_DEBUG_PRINT(("KeyMake: allocating table entries.\n"));
//        
//	// allocate table entries
//	elem = &table[kh];
//	memset(elem, 0, sizeof(struct bc_key_element));
//	elem->key_extended 	= kmalloc(alg->ops->ext_key_size_bytes, GFP_KERNEL);
//	elem->pool 		= kmalloc(POOL_SIZE_BYTES, GFP_KERNEL);
//	elem->lrw_context	= kmalloc(sizeof(GfmulCtx), GFP_KERNEL);
//	elem->xts_context	= kmalloc(alg->ops->ext_key_size_bytes, GFP_KERNEL);
//
//	if (!elem->key_extended || !elem->pool || !elem->lrw_context || !elem->xts_context || !key) {
//            BC_DEBUG_PRINT(("KeyMake: not enough memory.\n"));
//		error = -ENOMEM;
//		goto error_out;
//	}
//        
//        BC_DEBUG_PRINT(("KeyMake: creating key element.\n"));
//	memset(elem->key_extended, 0, alg->ops->ext_key_size_bytes);
//	memset(elem->lrw_context, 0, sizeof(GfmulCtx));
//        memcpy(elem->pool, pool, POOL_SIZE_BYTES);
//        
//        BC_DEBUG_PRINT(("KeyMake: getting second part of key.\n"));
//        
//	// get Key2 after Key1 in raw buffer
//	key_2 = &key[keylen - alg->ops->block_size_bytes];
//
//	if( alg->ops->block_size_bytes == 8 ) {
//		GFMUL_InitCtxBlock8( key_2, (GfmulCtx*) elem->lrw_context );
//	} else if (alg->ops->block_size_bytes == 16 ) {
//		GFMUL_InitCtxBlock16( key_2, (GfmulCtx*) elem->lrw_context );
//	} else {
//            BC_DEBUG_PRINT(("KeyMake: invalid key length value.\n"));
//            error = -EINVAL;
//            goto error_out;
//	}
//
//        BC_DEBUG_PRINT(("KeyMake: extending key.\n"));
//	alg->ops->key_extend(key, (DWORD *)elem->key_extended);
//	alg->ops->key_extend(&(key[XTS_KEY2_OFFSET]), (DWORD*)elem->xts_context );
//	elem->lock = 1;
//
//	*out_handle = kh;
//        BC_DEBUG_PRINT(("KeyMake: extending completed."));
//#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
//	try_module_get(alg->ops->owner);
//#else
//	__MOD_INC_USE_COUNT(alg->ops->owner);
//#endif
//	return 0;
//
//error_out:
//        BC_DEBUG_PRINT(("KeyMake: freeing data.\n"));
//	shred(&(elem->key_extended), alg->ops->ext_key_size_bytes);
//	shred(&(elem->lrw_context), sizeof(GfmulCtx));
//	shred(&(elem->xts_context), alg->ops->ext_key_size_bytes);
//        shred(&(elem->pool), POOL_SIZE_BYTES);
//        BC_DEBUG_PRINT(("KeyMake: exiting.\n"));
//	return error;
//}

//---------------------------------------------
//	Encryption\decryption
//---------------------------------------------


//#ifdef _LITTLE_ENDIAN
//#define le32_to_cpu(x) (x)
//else
//#define le32_to_cpu(x) (\
//	(((x)>>24)&0xff)\
//	|\
//	(((x)>>8)&0xff00)\
//	|\
//	(((x)<<8)&0xff0000)\
//	|\
//	(((x)<<24)&0xff000000)\
//)
//#endif
//
//inline void transform_iv (bc_u32 *iv, bc_u32 *pool, bc_iv_format_t ver)
//{
//	unsigned int i;
//	if (IV_4X16_0 == ver) {
//		i = (iv[0] & 0x3F) << 1;
//		iv[0] ^= pool[i];
//		iv[1] ^= pool[i+1];
//	} else {
//		i = __le32_to_cpu(iv[0]);
//		i = (i & 0x3F) << 1;
//		iv[0] ^= pool[i];
//		iv[1] ^= pool[i+1];
//	}
//
//}

inline void transform_iv ( bc_u8 *iv, const bc_u8 *pool )
{
	unsigned int i = (iv[0] & 0x3F );
	*(bc_u64*)iv ^= ((bc_u64*)pool)[i];
}

//inline void increment_iv(bc_u16 *iv, bc_iv_format_t ver)
//{
//	u64	iv64, *piv64;
//
//	if (IV_4X16_0 == ver) {
//		iv[0]++;
//		iv[3] = iv[2] = iv[1] = iv[0];
//	} else {
//		piv64 = (u64 *)iv;
//		iv64 = __le64_to_cpu(*piv64);
//		iv64++;
//		*piv64 = __cpu_to_le64(iv64);
//	}
//}

inline void increment_iv(bc_u64 *iv64, bc_iv_format_t ver)
{
	(*iv64)++;

	if (IV_4X16_0 == ver) {
		*iv64 = sector2ivOld( (bc_u16)*iv64 );
	}
}

inline void increment_iv(bc_u8 *iv, bc_iv_format_t ver)
{
	bc_u64	iv64;

	iv64 = sync2iv( iv );

	increment_iv( &iv64, ver);

	iv2sync( iv, iv64 );
}


// see sector2iv
inline bc_iv_format_t guess_iv_format( const bc_u8* iv )
{
	union
	{
		bc_u16 words[4];
		bc_u64 integer;
	} u_iv;

	u_iv.integer = *((bc_u64*)iv);

	if( (u_iv.words[0] == u_iv.words[3]) && (u_iv.words[0] == u_iv.words[1]) && (u_iv.words[0] == u_iv.words[2]) ) return IV_4X16_0;
	else return IV_1X64_1;
}

inline void transform_buf( const bc_u8 *pool, const bc_u8 *src, bc_u8 *dst, size_t len )
{
	size_t len64 = len / sizeof ( bc_u64 );

	unsigned long ui =  (unsigned long)src;

	if ( ( ui & 0x3 ) != 0 )
	{
		//
		//	We have to check alignment here.
		//	Otherwise we'll get crash.
		//	The effect was caught at Android 4.1 when large file was copied from Dropbox
		//
		static bool bReported = false;

		if ( !bReported )
			LIBLOG_INFO( "Warning source is not 64 bit aligned" );

		bReported = true;

		for( size_t i = 0; i < len; i++ )
			dst[i] = src[i]^pool[i];
		return;
	}

	bc_u64 *d = (bc_u64*)dst;
	bc_u64 *s = (bc_u64*)src;
	bc_u64 *p = (bc_u64*)pool;

	while (len64--)
		*d++ = *s++ ^ *p++;
}


static bc_cipher_result encrypt_cbc(
		CipherBase *cipher,
		type_cipher_process_cbc alg_encrypt,
		cipher_key_handle key_handle,
		const bc_u8 *iv,
		bc_iv_format_t iv_ver,
		const bc_u8 *src,
		bc_u8 *dst,
		size_t len,
		const bc_u8* pool
	)
{
	size_t	offset, size;

	bc_u64 ivS = sync2iv( iv );

	for (offset = 0; ; offset += 512) {

		bc_u8 ivT[16];
		iv2sync( ivT, ivS );
		transform_iv( ivT, pool );
		memcpy( ivT+8, ivT, 8 );

		size = len > 512 ? 512 : len;
		len -= size;
		transform_buf( pool, src+offset, dst+offset, size );
		alg_encrypt( cipher, ivT, dst+offset, dst+offset, size, key_handle );

		if (!len) break;

		/* We need to update IVector every 512 bytes */
		increment_iv( &ivS, iv_ver );
	}
	return kBCCipher_Success;
}


static bc_cipher_result decrypt_cbc(
		CipherBase *cipher,
		type_cipher_process_cbc alg_decrypt,
		cipher_key_handle key_handle,
		const bc_u8 *iv,
		bc_iv_format_t iv_ver,
		const bc_u8 *src,
		bc_u8 *dst,
		size_t len,
		const bc_u8* pool
	)
{
	size_t	offset, size;

	bc_u64 ivS = sync2iv( iv );

	for (offset = 0; ; offset += 512) {

		bc_u8 ivT[16];
		iv2sync( ivT, ivS );
		transform_iv( ivT, pool );
		memcpy( ivT+8, ivT, 8 );

		size = len > 512 ? 512 : len;
		len -= size;
		alg_decrypt( cipher, ivT, src+offset, dst+offset, size, key_handle );
		transform_buf( pool, dst+offset, dst+offset, size );

		if (!len) break;

		//  We need to update IVector every 512 bytes
		increment_iv( &ivS, iv_ver );
	}
	return kBCCipher_Success;
}

static bc_cipher_result encrypt_lrw(
		CipherBase *cipher,
		type_cipher_process_ecb alg_encrypt_ecb,// algorithm specific encrypt int ecb mode fptr
		cipher_key_handle key_handle, 		// extended algorithm key
		size_t block_size, 		// size of sequential block to process at a time
		const bc_u8 *iv, 		// initial vector
		const bc_u8 *src, 		// data source to encrypt from
		bc_u8 *dst, 			// data dest to encrypt into
		size_t len, 			// size of data
		const bc_u8* pool, 		// random pool
		bc_u8* lrw_ctx			// lrw mode key context
	)
{
	int            blockSize_words = 0, length_words, j;
	bc_u16 *in_w, *out_w;
	bc_u16 ivLocal[8] = {0}, t_lrw[8];
	bc_u64*	liByteAddress;

	liByteAddress = (bc_u64 *)ivLocal;

	*liByteAddress = *((bc_u64*)iv);

	// make the address from sector number to byte offset
	(*liByteAddress) <<= 9; // *= 512

	blockSize_words = block_size / sizeof(bc_u16);
	length_words    = (int)len / sizeof(bc_u16);

	in_w  = (bc_u16 *)src;
	out_w = (bc_u16 *)dst;

	for( j = 0; j < length_words; j += blockSize_words)
	{
		// calculate t and then IV for next block
		if ( block_size == 8)
		{
			GFMUL_MulBlock8( (bc_u8 *)ivLocal, (bc_u8 *)t_lrw, (GfmulCtx *)lrw_ctx);

			// add 8 (size of block) to ivLocal
			(*liByteAddress) += 8;

			out_w[0] = in_w[0] ^ t_lrw[0];
			out_w[1] = in_w[1] ^ t_lrw[1];
			out_w[2] = in_w[2] ^ t_lrw[2];
			out_w[3] = in_w[3] ^ t_lrw[3];

			alg_encrypt_ecb( cipher, (bc_u8 *)out_w, (bc_u8 *)out_w, key_handle );

			out_w[0] ^= t_lrw[0];
			out_w[1] ^= t_lrw[1];
			out_w[2] ^= t_lrw[2];
			out_w[3] ^= t_lrw[3];

			in_w  = &(in_w[4]);
			out_w = &(out_w[4]);
		}
		else
		{
			GFMUL_MulBlock16( (bc_u8 *)ivLocal, (bc_u8 *)t_lrw, (GfmulCtx *)lrw_ctx);

			// add 16 (size of block) to ivLocal
			(*liByteAddress) += 16;

			out_w[0] = in_w[0] ^ t_lrw[0];
			out_w[1] = in_w[1] ^ t_lrw[1];
			out_w[2] = in_w[2] ^ t_lrw[2];
			out_w[3] = in_w[3] ^ t_lrw[3];
			out_w[4] = in_w[4] ^ t_lrw[4];
			out_w[5] = in_w[5] ^ t_lrw[5];
			out_w[6] = in_w[6] ^ t_lrw[6];
			out_w[7] = in_w[7] ^ t_lrw[7];

			alg_encrypt_ecb( cipher, (bc_u8 *)out_w, (bc_u8 *)out_w, key_handle );

			out_w[0] ^= t_lrw[0];
			out_w[1] ^= t_lrw[1];
			out_w[2] ^= t_lrw[2];
			out_w[3] ^= t_lrw[3];
			out_w[4] ^= t_lrw[4];
			out_w[5] ^= t_lrw[5];
			out_w[6] ^= t_lrw[6];
			out_w[7] ^= t_lrw[7];

			in_w  = &(in_w[8]);
			out_w = &(out_w[8]);
		}

        }
	return kBCCipher_Success;
}


static bc_cipher_result decrypt_lrw(	CipherBase *cipher,
			type_cipher_process_ecb alg_decrypt_ecb,// algorithm specific encrypt int ecb mode fptr
			cipher_key_handle key_extended, 		// extended algorithm key
			size_t block_size, 			// size of sequential block to process at a time
			const bc_u8 *iv, 			// initial vector
			const bc_u8 *src, 			// data source to encrypt from
			bc_u8 *dst, 				// data dest to encrypt into
			size_t len, 				// size of data
			const bc_u8* pool, 			// random pool
			bc_u8* lrw_ctx				// lrw mode key context
)
{
	int            blockSize_words = 0, length_words, j;
	bc_u16 *in_w, *out_w;
	bc_u16 ivLocal[8] = {0}, t_lrw[8];
	bc_u64*	liByteAddress;

	liByteAddress = (bc_u64 *)ivLocal;

	*liByteAddress = *((bc_u64*)iv);

	// make the address from sector number to byte offset
	(*liByteAddress) <<= 9; // *= 512

	blockSize_words = block_size / sizeof(bc_u16);
	length_words    = (int)len / sizeof(bc_u16);

	in_w  = (bc_u16 *)src;
	out_w = (bc_u16 *)dst;

	for( j = 0; j < length_words; j += blockSize_words)
	{
		// calculate t and then IV for next block
		if ( block_size == 8)
		{
			GFMUL_MulBlock8( (bc_u8 *)ivLocal, (bc_u8 *)t_lrw, (GfmulCtx *)lrw_ctx);

			// add 8 (size of block) to ivLocal
			(*liByteAddress) += 8;

			out_w[0] = in_w[0] ^ t_lrw[0];
			out_w[1] = in_w[1] ^ t_lrw[1];
			out_w[2] = in_w[2] ^ t_lrw[2];
			out_w[3] = in_w[3] ^ t_lrw[3];

			alg_decrypt_ecb( cipher, (bc_u8 *)out_w, (bc_u8 *)out_w, key_extended );

			out_w[0] ^= t_lrw[0];
			out_w[1] ^= t_lrw[1];
			out_w[2] ^= t_lrw[2];
			out_w[3] ^= t_lrw[3];

			in_w  = &(in_w[4]);
			out_w = &(out_w[4]);
		}
		else
		{
			GFMUL_MulBlock16( (bc_u8 *)ivLocal, (bc_u8 *)t_lrw, (GfmulCtx *)lrw_ctx);

			// add 16 (size of block) to ivLocal
			(*liByteAddress) += 16;

			out_w[0] = in_w[0] ^ t_lrw[0];
			out_w[1] = in_w[1] ^ t_lrw[1];
			out_w[2] = in_w[2] ^ t_lrw[2];
			out_w[3] = in_w[3] ^ t_lrw[3];
			out_w[4] = in_w[4] ^ t_lrw[4];
			out_w[5] = in_w[5] ^ t_lrw[5];
			out_w[6] = in_w[6] ^ t_lrw[6];
			out_w[7] = in_w[7] ^ t_lrw[7];

			alg_decrypt_ecb( cipher, (bc_u8 *)out_w, (bc_u8 *)out_w, key_extended );

			out_w[0] ^= t_lrw[0];
			out_w[1] ^= t_lrw[1];
			out_w[2] ^= t_lrw[2];
			out_w[3] ^= t_lrw[3];
			out_w[4] ^= t_lrw[4];
			out_w[5] ^= t_lrw[5];
			out_w[6] ^= t_lrw[6];
			out_w[7] ^= t_lrw[7];

			in_w  = &(in_w[8]);
			out_w = &(out_w[8]);
		}

        }
	return kBCCipher_Success;
}

#define SECTOR_SIZE			512
#define XTS_BLOCK_SIZE		16
#define SECTOR_SIZE_POWER 	9
#define BLOCK_LENGTH_POWER	4

static bc_cipher_result encrypt_decrypt_xts(
		CipherBase *cipher,
		type_cipher_process_ecb	EncryptProc,
		type_cipher_process_ecb	EncryptOrDecryptProc,
		bc_u64					SectorNumber, // or IV - Initial Vector
		const bc_u8 *			InBuffer,
		bc_u8*					OutBuffer,
		size_t					BufferLength,
		cipher_key_handle		Context_1,
		cipher_key_handle		Context_2
	)
{
	bc_u64	alpha_J_array[ SECTOR_SIZE / sizeof(bc_u64) ];
	bc_u64	ulSectorCounter, ulBlockCounter, ulBlockMax;
	bc_u64	i, ul0, ul1, ulSpecial;
	bc_u64	ulPP[ XTS_BLOCK_SIZE / sizeof(bc_u64) ];

#define _useFix
#ifndef _useFix
	bc_u64	*ulInBuffer, *ulOutBuffer;

	ulInBuffer  = (bc_u64 *)InBuffer;
	ulOutBuffer = (bc_u64 *)OutBuffer;
#endif

	ulBlockCounter = BufferLength >> BLOCK_LENGTH_POWER;

	for (ulSectorCounter = 0; ulSectorCounter < ((BufferLength >> SECTOR_SIZE_POWER) + 1); ulSectorCounter++)
	{
		alpha_J_array[0] = SectorNumber + ulSectorCounter;
		alpha_J_array[1] = 0;

		EncryptProc( cipher, (bc_u8 *)alpha_J_array, (bc_u8 *)alpha_J_array, Context_2 );

		for (i = 2; i < (SECTOR_SIZE / XTS_BLOCK_SIZE) * 2; i = i + 2 )
		{
			ul0 = alpha_J_array[ i - 2 ];
			ul1 = alpha_J_array[ i - 1 ];

			ulSpecial = 0;

			if (ul1 & (bc_u64)0x8000000000000000ULL)
			{
				ulSpecial = 135;
			}

			ul1 <<= 1;

			if (ul0 & (bc_u64)0x8000000000000000ULL)
			{
				ul1 |= 1;
			}

			ul0 <<= 1;

			ul0 ^= ulSpecial;

			alpha_J_array[i]   = ul0;
			alpha_J_array[i+1] = ul1;
		}

		if (ulBlockCounter < (SECTOR_SIZE / XTS_BLOCK_SIZE))
		{
			ulBlockMax = ulBlockCounter;
		}
		else
		{
			ulBlockMax = SECTOR_SIZE / XTS_BLOCK_SIZE;
		}

		for ( i = 0; i < (ulBlockMax << 1); i += 2 )
		{
#ifdef _useFix
			memcpy( ulPP, InBuffer, 16 );
			InBuffer+=16;
			ulPP[0] ^= alpha_J_array[i];
			ulPP[1] ^= alpha_J_array[i+1];
#else
			ulPP[0] = alpha_J_array[i]   ^ (*ulInBuffer++);
			ulPP[1] = alpha_J_array[i+1] ^ (*ulInBuffer++);
#endif


			EncryptOrDecryptProc( cipher, (bc_u8 *)ulPP, (bc_u8 *)ulPP, Context_1 );

#ifdef _useFix
			ulPP[0] ^= alpha_J_array[i];
			ulPP[1] ^= alpha_J_array[i+1];
			memcpy( OutBuffer, ulPP, 16 );
			OutBuffer+=16;
#else
			*ulOutBuffer++ = alpha_J_array[i]   ^ ulPP[0];
			*ulOutBuffer++ = alpha_J_array[i+1] ^ ulPP[1];
#endif

			ulBlockCounter--;
		}
	}

	return kBCCipher_Success;
}

static bc_cipher_result check_environment( CipherBase *cipher, struct_bc_key *bc_key, const bc_u8 *iv, const bc_u8 *src, const bc_u8 *dst, size_t buflen, bc_mode_id mode )
{
	//LIBLOG_INFO( "Begin %d", buflen );

	if ( 0 == buflen )
	{
		LIBLOG_INFO( "0 == buflen" );
		return kBCCipher_InvalidArgument;
	}

	if ( buflen & ( cipher->blockSize() - 1 ) )
	{
		LIBLOG_INFO( "Invalid size %d", buflen );
		return kBCCipher_InvalidArgument;
	}

	if ( !bc_key )
	{
		LIBLOG_INFO( "Key is NULL" );
		return kBCCipher_NotInitialized;
	}

	if ( !bc_key->key_extended || !bc_key->xts_context || !bc_key->lrw_context || !bc_key->pool )
	{
		LIBLOG_INFO( "Key is not ready" );
		return kBCCipher_InvalidArgument;
	}

	return kBCCipher_Success;
}

#define Dummy_0xfe 254

bc_cipher_result bc_alg_encrypt ( CipherBase *cipher, struct_bc_key *bc_key, const bc_u8 *iv, const bc_u8 *src, bc_u8 *dst, size_t buflen, bc_mode_id mode )
{
	bc_cipher_result err;
	bc_iv_format_t iv_ver;

	if ( cipher->algId() == Dummy_0xfe )
	{
		if ( dst != src )
			memcpy( dst, src, buflen );

		return kBCCipher_Success;
	}

	err = check_environment( cipher, bc_key, iv, src, dst, buflen, mode );
	if ( kBCCipher_Success != err )
	{
		LIBLOG_INFO( "Check environment error = %d", err );
		return err;
	}

	bc_u8* pool = bc_key->pool;

	iv_ver = guess_iv_format(iv);

	bc_cipher_result res;

	switch(mode)
	{
		case kBCMode_CBC:
			res = encrypt_cbc( cipher, CipherBase::encrypt_cbc, bc_key->key_extended, iv, iv_ver, src, dst, buflen, pool);
			break;

		case kBCMode_LRW:
			res = encrypt_lrw( cipher, CipherBase::encrypt_ecb, bc_key->key_extended, cipher->blockSize(), iv, src, dst, buflen, pool, bc_key->lrw_context );
			break;

		case kBCMode_XTS:
			res = encrypt_decrypt_xts( cipher, CipherBase::encrypt_ecb, CipherBase::encrypt_ecb, sync2iv(iv), src, dst, buflen, bc_key->key_extended, bc_key->xts_context);
			break;

		default:
			res = kBCCipher_InvalidArgument;
			break;
	}

	return res;
}

bc_cipher_result bc_alg_decrypt ( CipherBase *cipher, struct_bc_key *bc_key, const bc_u8 *iv, const bc_u8 *src, bc_u8 *dst, size_t len, bc_mode_id mode )
{
	bc_cipher_result err;
	bc_iv_format_t iv_ver;

	if ( cipher->algId() == Dummy_0xfe )
	{
		if ( dst != src )
			memcpy( dst, src, len );

		return kBCCipher_Success;
	}

	err = check_environment( cipher, bc_key, iv, src, dst, len, mode );

	if ( kBCCipher_Success != err )
		return err;

	bc_u8* pool = bc_key->pool;

	iv_ver = guess_iv_format(iv);

	switch(mode)
	{
	case kBCMode_CBC:
		return decrypt_cbc(cipher, CipherBase::decrypt_cbc, bc_key->key_extended, iv, iv_ver, src, dst, len, pool);

	case kBCMode_LRW:
		return decrypt_lrw(cipher, CipherBase::decrypt_ecb, bc_key->key_extended, cipher->blockSize(), iv, src, dst, len, pool, bc_key->lrw_context);

	case kBCMode_XTS:
		return encrypt_decrypt_xts(cipher, CipherBase::encrypt_ecb, CipherBase::decrypt_ecb, sync2iv(iv), src, dst, len, bc_key->key_extended, bc_key->xts_context);

	default:
		return kBCCipher_UnsupportedMode;
	}
}


//int bc_alg_encrypt (struct bc_algorithm* alg, bc_key_handle kh, bc_u8 *iv, bc_u8 *src, bc_u8 *dst, size_t buflen, bc_mode_id mode)
//{
//	int err;
//	int iv_ver;
//	struct bc_key_element* elem;
//
//	if (!buflen)
//		return 0;
//
//	if (buflen & (alg->ops->block_size_bytes - 1))
//		return -EINVAL;
//
//	if( (err = bc_alg_test_key(alg, kh)) )
//		return err;
//
//	iv_ver = guess_iv_format(iv);
//
//	elem = &alg->key_table[kh];
//
//	switch(mode)
//	{
//	case EX_ENCRYPT_CBC_MODE_FLAG:
//		return encrypt_cbc(alg->ops->encrypt_cbc, elem->key_extended, iv, iv_ver, src, dst, buflen, elem->pool);
//
//	case EX_ENCRYPT_LRW_MODE_FLAG:
//		return encrypt_lrw(alg->ops->encrypt_ecb, elem->key_extended, alg->ops->block_size_bytes, iv, src, dst, buflen, elem->pool, elem->lrw_context);
//
//	case EX_ENCRYPT_XTS_MODE_FLAG:
//		return encrypt_decrypt_xts(alg->ops->encrypt_ecb, alg->ops->encrypt_ecb, *((ULONGLONG*)iv), src, dst, buflen, elem->key_extended, elem->xts_context);
//
//	default:
//		return -EINVAL;
//	}
//}


//int bc_alg_decrypt (struct bc_algorithm* alg, bc_key_handle kh, bc_u8 *iv, bc_u8 *src, bc_u8 *dst, size_t len, bc_mode_id mode)
//{
//	int err;
//	int iv_ver;
//	struct bc_key_element* elem;
//
//	if (!len)
//		return 0;
//
//	if (len & (alg->ops->block_size_bytes - 1))
//		return -EINVAL;
//
//	if( (err = bc_alg_test_key(alg, kh)) )
//		return err;
//
//	iv_ver = guess_iv_format(iv);
//
//	elem = &alg->key_table[kh];
//
//	switch(mode)
//	{
//	case EX_ENCRYPT_CBC_MODE_FLAG:
//		return decrypt_cbc(alg->ops->decrypt_cbc, elem->key_extended, iv, iv_ver, src, dst, len, elem->pool);
//
//	case EX_ENCRYPT_LRW_MODE_FLAG:
//		return decrypt_lrw(alg->ops->decrypt_ecb, elem->key_extended, alg->ops->block_size_bytes, iv, src, dst, len, elem->pool, elem->lrw_context);
//
//	case EX_ENCRYPT_XTS_MODE_FLAG:
//		return encrypt_decrypt_xts(alg->ops->encrypt_ecb, alg->ops->decrypt_ecb, *((ULONGLONG*)iv), src, dst, len, elem->key_extended, elem->xts_context);
//
//	default:
//		return -EINVAL;
//	}
//}


//char bc_alg_c[]="$Id: bc_alg.c,v 1.10 2006-04-12 05:32:39 nail Exp $";
