#ifndef __BC_DRV_ALG_H__
#define __BC_DRV_ALG_H__

#include "../intsize.h"
#include "../error.h"
#include "../bctypes.h"

class CipherBase;

//
//
//	Key Management
//
//


/**
 * @brief	Lock algorithm key by handle.
 */
bc_cipher_result bc_alg_lock_key( CipherBase *cipher, struct_bc_key *bc_key, int lock );


/**
 * @brief	Free algorithm key by handle.
 */
bc_cipher_result bc_alg_free_key( CipherBase *cipher, struct_bc_key *bc_key );


/**
 * @brief	Allocate memory and store algorithm key, return user a handle.
 */
bc_cipher_result bc_alg_make_key( CipherBase *cipher, const bc_u8* key, size_t keylen, const bc_u8* pool, struct_bc_key** out_bc_key);

//
//
//	Encryption
//
//


/**
 * @brief	Encrypt block of data.
 */
bc_cipher_result bc_alg_encrypt ( CipherBase *cipher, struct_bc_key *bc_key, const bc_u8 *iv, const bc_u8 *src, bc_u8 *dst, size_t len, bc_mode_id mode );


/**
 * @brief	Decrypt block of data.
 */
bc_cipher_result bc_alg_decrypt ( CipherBase *cipher, struct_bc_key *bc_key, const bc_u8 *iv, const bc_u8 *src, bc_u8 *dst, size_t len, bc_mode_id mode );


#endif

