//
// Copyright (c) 2006, Jetico, Inc., Finland. All rights reserved.
//

#ifndef __GF_MUL
#define __GF_MUL

#include "../intsize.h"

#if defined(__cplusplus)
extern "C"
{
#endif

#define LITTLE_ENDIAN_PLATFORM   1234 /* byte 0 is least significant (i386) */
#define BIG_ENDIAN_PLATFORM      4321 /* byte 0 is most significant (mc68k) */

#define PLATFORM_BYTE_ORDER LITTLE_ENDIAN_PLATFORM

#define BLOCK_SIZE_16 16
#define BLOCK_SIZE_8  8

typedef union
{
    bc_u8 gf_t8k[32 / 2][16][ BLOCK_SIZE_16 ]; // actually we need only a half of table, hence, although 8k in name used - it is 4kb table.

    bc_u8 gf_t2k[16][16][ BLOCK_SIZE_8 ];

} GfmulCtx;


// GFMUL_Basic functions are used internally, as well they can be used for
// testing purposes.
void GFMUL_Basic16( const bc_u8 *a, bc_u8 *b, bc_u8 *p);
void GFMUL_Basic8(  const bc_u8 *a, bc_u8 *b, bc_u8 *p);


// GFMUL_InitCtxBlock functions are used to generate encryption context GfmulCtx
// for encryption algorithms with 16 or 8-bytes blocks.
void GFMUL_InitCtxBlock16( const bc_u8 *Key2, GfmulCtx *Ctx );
void GFMUL_InitCtxBlock8(  const bc_u8 *Key2, GfmulCtx *Ctx );

// GFMUL_MulBlock functions are used to generate T number for LRW encryption mode
// for encryption algorithms with 16 or 8-bytes blocks.
void GFMUL_MulBlock16( bc_u8 *Iv, bc_u8 *T, GfmulCtx *Ctx );
void GFMUL_MulBlock8( bc_u8 *Iv, bc_u8 *T, GfmulCtx *Ctx );


#if defined(__cplusplus)
}
#endif

#endif
