/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 * Library error codes.
 */

#ifndef BC_CONTAINER_ERROR_H
#define BC_CONTAINER_ERROR_H

/// Cipher error codes.
typedef enum {
	kBCCipher_Success = 0,      ///< Operation successful.
	kBCCipher_NotInitialized,   ///< BC kernel is not initialized.
	kBCCipher_NoMemory,         ///< Error allocating memory.
	kBCCipher_UnknownAlg,       ///< Specified alg cannot be found.
	kBCCipher_UnsupportedMode,  ///< Alg doesn't supports specified mode.
	kBCCipher_InvalidKey,       ///< Invalid key specified.
	kBCCipher_InvalidBuffer,    ///< Invalid data buffer specified.
	kBCCipher_InvalidArgument,  ///
	kBCCipher_DeviceBusy,       ///
	kBCCipher_InternalError,    ///< Unknown internal error. In this case usually errno is set and may be checked.
	//-----------------------
	kBCCipher_lastError
} bc_cipher_result;

/// Container functions return codes.
typedef enum {
    kBCContainer_Success = 0,
	kBCContainer_firstError = kBCCipher_lastError,
	//--------------------------
    kBCContainer_UnknownFormat,     // Unknown container format
    kBCContainer_InvalidArgument,   // Invalid function argument
    kBCContainer_InvalidAlgorithm,  // Invalid encryption algorithm
    kBCContainer_InvalidPassword,   // Incorrect user password
	kBCContainer_Damaged,		    // Container data is damaged
	kBCContainer_NoKeySpace,		// No space left in for new password
    kBCContainer_LastKeyBlock,      // Last key block left for volume
    kBCContainer_IOError,           // Read/write error on container file
    kBCContainer_NoMemory,          // Memory allocation error
	kBCContainer_Unsupported,		// Keygen op not supported
    kBCContainer_Cancelled,         // Operation was cancelled by the user
    kBCContainer_InternalError,     // Unknown internal error
	//-----------------------
	kBCContainer_lasError
} BCContainerResult;

// Key block functions return codes.
typedef enum {
    kKeyBlock_Success = 0,
	kKeyBlock_errorNo = 0,
	kKeyBlock_firstError = kBCContainer_lasError,
	//-------------------------
	kKeyBlock_InternalProblem,
	kKeyBlock_InvalidAlgorithm,
	kKeyBlock_IncorrectCreateFlag,
	kKeyBlock_IncorrectFormatVersion,
	kKeyBlock_IncorrectBlockSignature,
	kKeyBlock_DamagedDataBlock,
	kKeyBlock_CancelledByUser,
	kKeyBlock_UserPaswordsMismatch,
	kKeyBlock_IncorrectPassword,
	kKeyBlock_NotEnoughInitKeys,
	kKeyBlock_IncorrectEncrAlgorithm,
	kKeyBlock_IncorrectHashAlgorithm,
	kKeyBlock_IncorrectDataBlockSize,
	kKeyBlock_NotEnoughSpaceForKey,
	kKeyBlock_IncorrectPool,
	kKeyBlock_NoJustGoBack,
	kKeyBlock_BufferTooSmall,
	kKeyBlock_ErrorInGeneratingAsymKey,
	kKeyBlock_HiddenPasswordIsInvalid,
	kKeyBlock_MasterPasswordIsInvalid,
	//-------------------------
	kKeyBlock_lastError,
} BCKeyBlockResult;

/// Human-readable description of the library error code
static inline const char* BCContainerErrorDecription(BCContainerResult error)
{
	switch (error) {
        case kBCContainer_UnknownFormat:    return "Unknown container format";
        case kBCContainer_InvalidArgument:  return "Invalid argument";   
        case kBCContainer_InvalidAlgorithm: return "Invalid encryption algorithm";
        case kBCContainer_InvalidPassword:  return "Invalid password";
	    case kBCContainer_Damaged:          return "Damaged container file";
	    case kBCContainer_NoKeySpace:       return "No space for new password";
        case kBCContainer_IOError:          return "Container file IO error";
        case kBCContainer_NoMemory:         return "No memory";
	    case kBCContainer_Unsupported:      return "Operation not supported";
        case kBCContainer_InternalError:    return "Internal system failure";
		default:                            return "Unknown error"; // FIXIT!
	}
}

#endif

