/*
 * IContainer.h
 *
 *  Created on: Oct 18, 2012
 *      Author: anton
 */

#ifndef BCRYPT_CONTAINER_ICONTAINER_H_
#define BCRYPT_CONTAINER_ICONTAINER_H_

#include <stdint.h>

namespace bcrypt_container{

enum CONT_IO_STATUS{
	CONT_IO_OK = 0,		/* 0: Successful */
	CONT_IO_ERROR,		/* 1: R/W Error */
	CONT_IO_WRPRT,		/* 2: Write Protected */
	CONT_IO_NOTRDY,		/* 3: Not Ready */
	CONT_IO_PARERR,		/* 4: Invalid Parameter */
	CONT_IO_INVCRED		/* 5: Incorrect credentials*/
};

enum IO_CTL{
	IOCTL_SYNC,
	IOCTL_SECTOR_SIZE,
	IOCTL_SECTOR_COUNT
};

class IContainer {
protected:
	IContainer(){};
public:
	virtual ~IContainer(){};

	virtual CONT_IO_STATUS diskInitialize()=0;
	virtual CONT_IO_STATUS diskStatus()=0;
	virtual CONT_IO_STATUS diskRead  (      void* buf, __int64_t firstSector, __int64_t secorCount)=0;
	virtual CONT_IO_STATUS diskWrite (const void* buf, __int64_t firstSector, __int64_t secorCount)=0;
	
	/**
	 * Executes control function on disk
	 * 	params:
	 * IOCTL_SYNC - Make sure that the disk drive has finished pending write process. When the disk I/O module has a write back cache, flush the dirty sector immediately. This command is not used in read-only configuration. Second param is unused
	 * IOCTL_SECTOR_SIZE - Returns sector size of the drive into the WORD variable pointed by buf
	 * IOCTL_SECTOR_COUNT - Returns number of available sectors on the drive into the _int64_t variable pointed by buf. This command is used by only f_mkfs function to determine the volume size to be created.
	 */
	virtual CONT_IO_STATUS diskIOctl (IO_CTL command, void* buf)=0;

};

}; //ns bcrypt_container

#endif /* BCRYPT_CONTAINER_ICONTAINER_H_ */
