//
//  helpers.h
//  container
//
//  Created by Jetico Inc. Oy on 16/10/15.
//  Copyright (c) 2015 Jetico Inc. Oy. All rights reserved.
//

#ifndef __HELPERS_H__
#define __HELPERS_H__

#include <time.h>
#include "volume.h"
#include "../certificate/CertificateLib/SMemory/SBlocks.h"


class iChannel;
class IBCHeader;
class cPwdGet;
struct BCVolume;
struct BCVolumeInfo;

#ifdef __APPLE__
# include <sys/syslimits.h>
# define BC_MAX_PATH PATH_MAX
#else
# define BC_MAX_PATH 4096
#endif


/// @brief Deprecated (Unused) structure
struct BCMountParams
{
    bool first; //  mountReadOnly;             ///< Will container need to be mounted as read-only?
    bool second; // useDefaultMountPoint;      ///< Should we use user-specified mount point or default (ignored on Linux).
    char third[BC_MAX_PATH]; // mountPoint[BC_MAX_PATH];   ///< User-specified mount point.
    
    BCMountParams() : first(false), second(true) {
        third[0] = 0;
    }
};

class BCContainerContext {

public:
    /*** Container creation params ***/
    bc_alg_id algID;                // Algorithm ID
    bc_mode_id modeID;              // Algorithm mode ID
    bc_u64 size;                    // Container body size ( file size = header size + body size)
    char description[50];           // Container description, 21 symbol, will be converted to UTF16
    bc_u8 salt[64];                 // Salt for new container
	int saltSize;
	bc_u16 kgID;
    bc_u32 iterations;              // Iterations count
    bc_u32 hashAlgID;               // Hash algorithm ID
    char path[BC_MAX_PATH];         // In case of container creation here is the new container must be path stored
    bool encryptHeader;             // Encrypt header on creation
    /*** Password request params ***/
    bool askMore;                   // Allows to construct sequence of password requests. For example when asking
                                    // for main volume password before at new password adding time can request
                                    // password from hidden part. 
    
    BCContainerContext() : algID(0), modeID(kBCMode_UDF), size(0), iterations(0), hashAlgID(0), encryptHeader(false), askMore(false) {
        memset(description, 0, 50);
        memset(salt, 0, 64);
		saltSize = 0;
		kgID = -1;
    }
};

class BCHelperContext {

    BCVolume volume;           // BCVolume structure as a result of operation4
    bool m_adoptedHeader;

public:
    iChannel * channel;             // Container channel implementation
	bc_file_mode mode;              // Force congtainer file open mode, overwrites mount options
    BCVolume *outVolume;            // BCVolume structure as a result of operation
    BCVolumeInfo volumeInfo;        // Volume info as a result of operation
    IBCHeader * header;             // Container keygen. May be used as in or out parameter
    cProgressActor * progressActor;
    cPwdGet * pwdGet;

	cPTB	ptbKey;					// Buffer to store plain text key.
    /*BCHelperContext()
        : header(NULL)
        , m_adoptedHeader(false)
        , progressActor(NULL)
        , channel(NULL)
        , pwdGet(NULL) {
            outVolume = &volume;
            mode = kBCFileMode_ReadWrite;
        }*/
                    
    BCHelperContext( iChannel * _channel = NULL, bc_file_mode _mode = kBCFileMode_Read, cPwdGet * _pwdGet = NULL, BCVolume *outPtr = NULL )
		: header(NULL), m_adoptedHeader(false), progressActor(NULL)
	{
		outVolume = outPtr == NULL ? &volume : outPtr;

		channel	= _channel;
		mode	= _mode;
		pwdGet	= _pwdGet;
	}

    BCHelperContext(const BCHelperContext & _other) {
        this->operator=(_other);
    }
    
    BCHelperContext & operator=(const BCHelperContext & _other) {
        channel = _other.channel;
        mode = _other.mode;
        
        if (&_other.volume != _other.outVolume) {
            outVolume = _other.outVolume;
        } else {
            volume = _other.volume;
            outVolume = &volume;
        }
        header = _other.header;
        m_adoptedHeader = true;
        progressActor = _other.progressActor;
        pwdGet = _other.pwdGet;
        
        return *this;
    }
    
	BCContainerResult decryptHeader()
	{
		if ( header == NULL )
			return kBCContainer_InternalError;

		return header->decrypt( ptbKey );
	}

	BCContainerResult encryptHeader()
	{
		if ( header == NULL )
			return kBCContainer_InternalError;
        
		return header->encrypt( ptbKey );
	}

	BCContainerResult storeHeader()
	{
		if ( header == NULL )
			return kBCContainer_InternalError;

		if ( channel == NULL )
			return kBCContainer_Success;

		return header->store( channel );
	}

    ~BCHelperContext()
    {
        if (header != NULL && !m_adoptedHeader) delete header;
    }
    
	bool pwdIsEmpty();
};

class cTimer
{
    clock_t m_start;
    
public:
    cTimer()
    {
        Start();
    }
    
    void Start()
    {
        m_start = clock();
    }
    
    double getTime()
    {
        clock_t clocks = clock() - m_start;
        
        double time = clocks;
        time /= CLOCKS_PER_SEC;
        
        return time;
    }
};

#endif // __HELPERS_H__
