/*
 *	Copyright (c) 2012-2015 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *	Base class for channel interface.
 */

#pragma once

#include "intsize.h"
#include "fileDef.h"

#ifdef _WIN32
#define strdup _strdup
#endif

class iChannel
{
public:
	virtual ~iChannel()
	{
	}

	/**
	 * Repositions the offset of the open file.
	 * @param offset        Offset.
	 * @param seek          Describes position, from which offset is defined.
	 * @return              Current position of pointer from the beginning of file 
	 *                      or BC_FILE_ERR, if error occured.
	 */
	virtual bc_u64 setPosition( bc_u64 offset, bc_file_seek seek ) = 0;

	/**
	 * Reads/writes a part of chanel with offset from the beginning of the chanel.
	 * @param buffer        Data buffer.
	 * @param byteCount     Data buffer size in bytes.
	 * @param offset        Number of bytes from the beginning of file 
	 *                      that must be skipped before starting reading.
	 * @return              Number of bytes actually read or BC_FILE_ERR, if error occured.
	 */
	virtual bc_u64 readBytes( void *buffer, bc_u64 byteCount, bc_u64 offset ) = 0;
	virtual bc_u64 writeBytes( void *buffer, bc_u64 byteCount, bc_u64 offset ) = 0;

	/**
	 * Reads/writes a part of chanel at current posotion.
	 * @param buffer        Data buffer.
	 * @param byteCount     Data buffer size in bytes.
	 * @param offset        Number of bytes from the beginning of file 
	 *                      that must be skipped before starting reading.
	 * @return              Number of bytes actually read or BC_FILE_ERR, if error occured.
	 */
	virtual bc_u64 readBytes( void *buffer, bc_u64 byteCount ) = 0;
	virtual bc_u64 writeBytes( void *buffer, bc_u64 byteCount ) = 0;

	virtual void sync() = 0;

	virtual bool open( int mode ) = 0; // See enum BCFileMode
	// For cFileChanel
	//	if ( NULL == vol->file_handle )
	//	return (kBCFileResult_Success == BCFileOpen(vol->file, vol->mode, &vol->file_handle));

	virtual void close() = 0;
	// For cFileChanel
	//	if ( NULL != vol->file_handle )
	//		BCFileClose( &(vol->file_handle) );

	virtual bc_u64 getSize()
	{
		bc_u64 oldPos = setPosition( 0, kBCFileSeek_Current );
		bc_u64 size = setPosition( 0, kBCFileSeek_End );
		setPosition( oldPos, kBCFileSeek_Begin );
		return size;
	}
    
    virtual bc_u64 truncate( bc_u64 size ) = 0;

	virtual void deleteFile() = 0;

	template < class T > bool readToBlock( T &block )
	{
		bc_u64 size = this->getSize();

		if ( size == 0 )
			return false;

		if ( size >= ( (bc_u64)1 << 32 ) )
			return false;

		bc_u8 *bf = block.Allocate( (int)size );

		if ( bf == NULL )
			return false;

		if ( this->readBytes( bf, size, 0 ) != size )
			return false;

		return true;
	}

};
