#include "password.h"
#include "user.h"
#include "../certificate/CertificateLib/x509pkts/pkcs12_funcs.h"

bool BCPasswordUTF16::set( const char *pwd_utf8, size_t len, bool b_big_endian )
{
	clear();

	if ( (-1) == len ) // Count length by null terminaton.
	{
		for( len = 0; ; len++ )
		{
			if ( len == kBCPasswordMax )
				break;

			if ( pwd_utf8[ len ] == 0 )
				break;
		}
	}

	if ( len >= kBCPasswordMax )
		return false;

	BCUserEncoding toEncoding = b_big_endian ? kBCUser_UTF16BE : kBCUser_UTF16LE;

	size = BCUserConvertString( (const bc_u8*)pwd_utf8, len, NULL, 0, kBCUser_UTF8, toEncoding );

	if ( 0 == size )
		return false;

	buf = new bc_u8[size + 2]; // Zero termination is required...

	if ( NULL == buf )
	{
		size = 0;
		return false;
	}

	if ( size != BCUserConvertString( (const bc_u8*)pwd_utf8, len, buf, size, kBCUser_UTF8, toEncoding ) )
	{
		clear();
		return false;
	}

	buf[size] = 0;
	buf[size+1] = 0;

	//
	//	Paranoid check but BCUserConvertString may add the zero termination symbol ( usual in Windows ).
	//
	while( !buf[size-1] && !buf[size-2] )
	{
		if ( size == 0 )
		{
			clear();
			return false;
		}

		size -= 2;
	}

	size += 2; // Include the last two byte size terminated symbol

	return true;
}

bool BCPassword::generateKey( int hash, const bc_u8 *salt, size_t saltSize, bc_u32 iterations )
{
	bReady = BCPasswordUTF16::generate_key_from_password( hash, this->buf, this->length, salt, saltSize, iterations, (bc_u8*)storage_key, sizeof(storage_key));
	return bReady;
}

//static 
bool BCPasswordUTF16::generate_key_from_password(int hash_id, const char* pwdbuf_utf8, size_t pwdlen, const bc_u8* salt, size_t saltsize, bc_u32 num_iter, bc_u8* out_key, size_t keysize)
{
	BCPasswordUTF16 pwdU16;

	if ( !pwdU16.set( pwdbuf_utf8, pwdlen, true ) ) {
		assert( false );
		return false;
	}

	return generate_key_from_password( hash_id, &pwdU16, salt, saltsize, num_iter, out_key, keysize );
}

bool BCPasswordUTF16::generate_key_from_password(int hash_id, const bc_u8* salt, size_t saltsize, bc_u32 num_iter, bc_u8* out_key, size_t keysize) const
{
	int res = P12_producePseudoRandomBytes((BYTE*)out_key, keysize, (BYTE*)salt, saltsize, num_iter, 
						this->buf, this->size,
						P12_prgm4key, (BYTE)hash_id);

	assert(res == x509e_ok);

	return (res == x509e_ok);
}

void BCPasswordUTF16::clearBuffer( bc_u8 **pp_bf, size_t *p_size )
{
	if ( NULL == pp_bf )
	{
		assert( false );
		return;
	}

	if ( NULL == p_size )
	{
		assert( false );
		return;
	}

	if ( NULL != (*pp_bf) )
	{
		for( size_t i = (*p_size); i--; )
			(*pp_bf)[i] = 0;

		delete (*pp_bf);
	}

	(*p_size) = 0;
	(*pp_bf) = NULL;
}
