/*
 *  Copyright 2012 Jetico, Inc. All rights reserved.
 *
 *	Sector to initial vector conversion.
 *
 *	Sector number is copied to initial vector byte buffer as little-endian sequence. 
 *	All the function give same result for both little- and big-endian cases.
 */
#pragma once

#include "intsize.h"

/**
 * iv2sync - copies 64bit initial vector to 8 byte synchro
 * @sync	8 byte size buffer
 * @iv		64-bit initial vector
 * */
static inline void iv2sync( bc_u8 *sync, bc_u64 iv )
{
#ifdef _LITTLE_ENDIAN_DEFINED
	*(bc_u64*)sync = iv;
#else
	sync[7] = (bc_u8)(iv >> 56);
	sync[6] = (bc_u8)(iv >> 48);
	sync[5] = (bc_u8)(iv >> 40);
	sync[4] = (bc_u8)(iv >> 32);
	sync[3] = (bc_u8)(iv >> 24);
	sync[2] = (bc_u8)(iv >> 16);
	sync[1] = (bc_u8)(iv >> 8);
	sync[0] = (bc_u8)(iv);
#endif
}

/**
 * byte2iv - return 64-bit value with the byte in corresponded position
 * @byte	byte to be moved
 * @pos		position in 64-bit result
 * */
static inline bc_u64 byte2iv( bc_u8 byte, int pos )
{
	bc_u64 iv = byte;
	return iv << ( pos * 8 );
}

/**
 * sync2iv - copies 8 byte synchro to 64bit initial vector
 * @sync	8 byte size buffer
 * */
static inline bc_u64 sync2iv( const bc_u8 *sync )
{
#ifdef _LITTLE_ENDIAN_DEFINED
	return *(bc_u64*)sync;
#else
	bc_u64 iv =
		(bc_u64)( sync[0] ) |
		byte2iv( sync[1], 1 ) |
		byte2iv( sync[2], 2 ) |
		byte2iv( sync[3], 3 ) |
		byte2iv( sync[4], 4 ) |
		byte2iv( sync[5], 5 ) |
		byte2iv( sync[6], 6 ) |
		byte2iv( sync[7], 7 ) ;

	return iv;
#endif
}

/**
 * sector2iv - convert sector number to initial vector
 *	Note, initial vector used as internal value, for encryption 
 *	it has to be converted to synchro byte sequence see iv2sync
 * @sector			64-bit sector number
 * @use_new_synchro	new uses 64-bit sector as initial vector, old uses 16-bit sector
 * @return			64-bit initial vector
 * */
static inline bc_u64 sector2ivOld( bc_u16 sector )
{
	union
	{
		bc_u16 words[4];
		bc_u64 integer;
	} u_iv;

	u_iv.words[0] = u_iv.words[1] = u_iv.words[2] = u_iv.words[3] = sector;

	return u_iv.integer;
}

static inline bc_u64 sector2iv( bc_u64 sector, int use_new_synchro )
{
	if ( use_new_synchro )
		return sector + 1; // The tric is used for cipher to guess iv format see guess_iv_format

	return sector2ivOld( (bc_u16)sector );
}
