/*
 * Copyright 2012 Jetico, Inc. All rights reserved.
 * Windows user auth session implementation.
 */

#include "../../container/user.h"
#include "../../container/intsize.h"
#include <stdio.h>

/**
 * Run verify password session.
 * @param query         Password query, default query will be used if NULL
 * @param callback	    Password verification callback that will be called each time a user tries to enter a password.
 * @param cookie		Password verification callback cookie.
 * @param out_pwd       Password that was successfully verified.
 * @return	            kBCUserCancel if user gave up to provide a correct password, kBCUserOk otherwise.
 */
BCUserResult BCUserVerifyPassword(const char* query, BCVerifyPasswordCallback callback, void* cookie, BCMountParams *mountParams, BCPassword* out_pwd)
{
	printf( "%s>", query );

	gets( out_pwd->buf );

	size_t len = strlen( out_pwd->buf );

	if ( sizeof( out_pwd->buf ) <= len )
		throw CipherException(kBCCipher_InternalError);

	const char *err = (*callback)( out_pwd->buf, len, cookie );

	if ( err == NULL )
		return kBCUserOk;

	printf( "\n%s\n", err );

	return kBCUserCancel;
}


/**
 * Run new password session.
 * @param query         New password query, default query will be used if NULL
 * @param callback	    Password verification callback that will be called each time a user tries to enter a password.
 * @param cookie		Password verification callback cookie.
 * @param out_pwd       Password that was successfully verified.
 * @return	            kBCUserCancel if user gave up to provide a correct password, kBCUserOk otherwise.
 */
BCUserResult BCUserCreatePassword(const char* query, BCVerifyPasswordCallback callback, void* cookie, BCPassword* out_pwd)
{
	assert( false );
	return kBCUserCancel;
}


/**
 * Fire up the seed generation progress dialog.
 * @param total	        Total bytes of seed value that needs to be generated.
 * @return	            Number of bytes of seed generated
 */
unsigned BCUserGenerateSeed(bc_u8* out_buf, unsigned total)
{
	unsigned pos = 0;

	while( true )
	{
		printf( "\nEnter %d symbols for seed>", total - pos );
		fgets( (char*)out_buf + pos, total - pos, stdin );
		out_buf[ total - 1 ] = '\0';
		unsigned len = strlen( (char*)out_buf + pos );
		if ( pos + len == total - 1 )
			return total;
		pos += len;
	}

	return 0;
}

/**
 * Change string encoding.
 * @param msg                   Source message to change encoding of.
 * @param msgSize               Source message size in bytes.
 * @param out_encodedMsg        Buffer to contain target message with changed encoding.
 * @param bufferSize            Buffer length in bytes.
 * @param fromEncoding          Source message encoding.
 * @param toEncoding            Target message encoding.
 * @return                      Number of bytes in output buffer used for converted string or 0 on error, 
 *								If output buffer is NULL it returns number of bytes required to complete conversion.
 */
//size_t BCUserConvertString( const bc_u8* in, size_t inlen, bc_u8* out, size_t outlen, BCUserEncoding fromEncoding, BCUserEncoding toEncoding)
//{
//	size_t size = 0;
//	size_t error = 0;
//
//	if ( kBCUser_UTF8 == fromEncoding )
//	{
//		if ( kBCUser_UTF16LE == toEncoding )
//			size = BCUserUTF8ToUTF16LE( (const char*)in, inlen, out, outlen );
//		else
//		if ( kBCUser_UTF16BE == toEncoding )
//			size = BCUserUTF8ToUTF16BE( (const char*)in, inlen, out, outlen );
//		else
//		{
//			assert( false );
//			error = 1;
//		}
//	}
//	else
//	if ( kBCUser_UTF16LE == fromEncoding )
//	{
//		if ( kBCUser_UTF16BE == toEncoding )
//			size = copy_and_swap( out, outlen, in, inlen );
//		else
//		if ( kBCUser_UTF8 == toEncoding )
//			size = BCUserUTF16LEToUTF8( in, inlen, (char*)out, outlen );
//		else
//		{
//			assert( false );
//			error = 2;
//		}
//	}
//	else
//	if ( kBCUser_UTF16BE == fromEncoding )
//	{
//		if ( kBCUser_UTF16LE == toEncoding )
//			size = copy_and_swap( out, outlen, in, inlen );
//		else
//		if ( kBCUser_UTF8 == toEncoding )
//			size = BCUserUTF16BEToUTF8( in, inlen, (char*)out, outlen );
//		else
//		{
//			assert( false );
//			error = 4;
//		}
//	}
//	else
//	{
//		assert( false );
//		error = 5;
//	}
//
//	if ( 0 != error )
//		size = 0;
//
//	return size;
//}


/**
 * Convert UTF16LE string to UTF8.
 * @param in_utf16      UTF16LE encoded input string. ( BUT :( Linux version according code generates BE here )
 * @param inlen         Input string buffer size.
 * @param out_utf8      Buffer to hold utf8 converted string.
 * @param out_len       Output buffer size.
 * @return              Number of bytes in output buffer used for converted string or 0 on error,
 *						If output buffer is NULL it returns number of bytes required to complete conversion.
 */
//size_t BCUserUTF16LEToUTF8(const bc_u8* in_utf16, size_t inlen, char* out_utf8, size_t outlen)
//{
//	//WideCharToMultiByte( 
//	return 0;
//}

/**
 * Convert UTF16BE string to UTF8.
 * @param in_utf16      UTF16BE encoded input string. ( BUT :( Linux version according code generates BE here )
 * @param inlen         Input string buffer size.
 * @param out_utf8      Buffer to hold utf8 converted string.
 * @param out_len       Output buffer size.
 * @return              Number of bytes in output buffer used for converted string or 0 on error,
 *						If output buffer is NULL it returns number of bytes required to complete conversion.
 */
//size_t BCUserUTF16BEToUTF8(const bc_u8* in_utf16, size_t inlen, char* out_utf8, size_t outlen)
//{
//	//WideCharToMultiByte( 
//	return 0;
//}


///**
// * Same as above but reverse.
// */
//size_t BCUserUTF8ToUTF16LE(const char* in_utf8, size_t inlen, bc_u8* out_utf16, size_t outlen)
//{
//	 size_t sizeInWchar = MultiByteToWideChar( CP_UTF8, MB_ERR_INVALID_CHARS, in_utf8, inlen, (wchar_t *)out_utf16, outlen / 2 );
//	return sizeInWchar * 2;
//}
//
//size_t BCUserUTF8ToUTF16BE(const char* in_utf8, size_t inlen, bc_u8* out_utf16, size_t outlen)
//{
//	size_t sizeInBytes = BCUserUTF8ToUTF16LE( in_utf8, inlen, out_utf16, outlen );
//
//	if ( NULL != out_utf16 )
//		swap_bytes( out_utf16, sizeInBytes );
//
//	return sizeInBytes;
//}

/**
 * Display a message to the user.
 * @param msg       User message text.
 * @param level     User message type level.
 */
void BCUserMessage(const char* msg, BCUserMessageLevel level)
{
	printf( "Error '%s', level %d", msg, level );
}


/**
 * Gets information about a new hidden part. Allows an user to protect existing hidden parts
 * by substracting their extents from the hidden map.
 * @param header            Parameter to pass to protectCallback.
 * @param protectCallback   Used to protect existing hidden part by substracting it's blocks from hiddenMap.
 * @param hiddenMap         Map with extents of future hidden part.
 * @param newPassword       Password for new hidden part.
 */
BCUserResult BCUserGetNewHiddenPartInfo(void *header, BCProtectExistingHiddenPartCallback protectCallback,
                                     BCVerifyPasswordCallback verifyCallback, void* verifyCookie,
                                     SpaceMap *hiddenMap, BCPassword *newPassword)
{
	assert( false );
	return kBCUserCancel;
}

