#
# Copyright 2011 Jetico, Inc. All rights reserved.
# AES-NI 32-bit Implementation 
#

# 
# Older binutils do not support aes-ni instructions.
# We use the following patchwork:
# aesenc	%xmm0, %xmm1:	66 0F 38 DC С8
# aesenclast	%xmm0, %xmm1:	66 0F 38 DD С8
# aesdec	%xmm0, %xmm1:	66 0F 38 DE С8
# aesdeclast	%xmm0, %xmm1:	66 0F 38 DF С8
#

.text
.code32

# Procedure looks at cpu model to see if AES-NI instruction set is supported.
# Input: none
# Output: Support flag in EAX 
.globl _aeshw_is_supported
.align 4
_aeshw_is_supported:
    # call cpuid
    pushl	%ecx		# ECX is clobbered
    movl	$1, %eax
    cpuid

    # Store 25-th bit that indicates aes-ni support
    shrl    $25, %ecx
    andl	$1, %ecx
    movl	%ecx, %eax
    popl	%ecx
    ret

# Decrypt data.
# Input: ESI - Source buffer, EDI - Destination buffer, EDX - key Digest
# Output: none 
.globl _aeshw_decrypt
.align 4
_aeshw_decrypt:
    # prepare input block by xoring with initial round key.
    # %xmm0 will hold the state (plaintext block to encrypt).
    # %xmm1 will hold the round key.
    movdqu	(%esi), %xmm0
    addl    $(56*4), %edx      # use round keys in reversed order
    movdqu	(%edx), %xmm1
    pxor	%xmm1, %xmm0

    # perform first 13 rounds of aes
    subl	$16, %edx
    movdqu	(%edx), %xmm1
    movl	$13, %ecx

    L1: aesdec  %xmm1, %xmm0
    #L1:	.byte	0x66, 0x0f, 0x38, 0xde, 0xc8	# aesdec %xmm0, %xmm1
    subl	$16, %edx
    movdqu	(%edx), %xmm1
    loop	L1

    # perform final aes round on plaintext, store result and return
    aesdeclast %xmm1, %xmm0
    #.byte	0x66, 0x0f, 0x38, 0xdf, 0xc8	# aesdeclast %xmm0, %xmm1
    movdqu	%xmm0, (%edi)
    ret


# Encrypt data.
# Input: ESI - Source buffer, EDI - Destination buffer, EDX - key Digest
# Output: none 
.globl _aeshw_encrypt 
.align 4 
_aeshw_encrypt:
# prepare input block by xoring with initial round key.
# %xmm0 will hold the state (ciphertext block to decrypt).
# %xmm1 will hold the round key.
    movdqu	(%esi), %xmm0
    movdqu	(%edx), %xmm1
    pxor	%xmm1, %xmm0

    # perform first 13 rounds of aes
    addl	$16, %edx
    movdqu	(%edx), %xmm1
    movl	$13, %ecx

    L2: aesenc  %xmm1, %xmm0
    #L2:	.byte	0x66, 0x0f, 0x38, 0xdc, 0xc8	# aesenc %xmm0, %xmm1
    addl	$16, %edx
    movdqu	(%edx), %xmm1
    loop	L2

    # perform final aes round on plaintext, store result and return
    aesenclast %xmm1, %xmm0
    #.byte	0x66, 0x0f, 0x38, 0xdd, 0xc8	# aesenclast %xmm0, %xmm1
    movdqu	%xmm0, (%edi)
    ret

