
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/resource.h>

#include "aes.h"

#define USE_AESHW

static const unsigned char p[] = {0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff};
static const unsigned char c[] = {0x8e, 0xa2, 0xb7, 0xca, 0x51, 0x67, 0x45, 0xbf, 0xea, 0xfc, 0x49, 0x90, 0x4b, 0x49, 0x60, 0x89};
static const unsigned char key[] = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0xc0, 0xd0, 0xe0, 0xf1, 
                            0x01, 0x11, 0x21, 0x31, 0x41, 0x51, 0x61, 0x71, 0x81, 0x91, 0xa1, 0xb1, 0xc1, 0xd1, 0xe1, 0xf};


static int is_supported(void) {
	int cpu_aesni = 0;
	asm ("call _aeshw_is_supported \n\t"
         "movl %%eax, %0 \n\t"
         :"=r" (cpu_aesni)
         );

    return cpu_aesni;
}


static void encrypt(const aes* key, const char* in, char* out, unsigned nbytes) 
{
    unsigned nblocks = nbytes / BLOCK_SIZE_R;
    unsigned i;
    
    for(i = 0; i < nblocks; ++i) {
        char* pout = out + (i * BLOCK_SIZE_R);
        const char* pin = in + (i * BLOCK_SIZE_R);
#ifdef USE_AESHW
        asm volatile ("call  _aeshw_encrypt \n\t"
             :
             : "S" (pin), "D" (pout), "d" (key->e_key)   // pin->%esi, pout->%edi, d_key->%edx
             : "%esi", "%edi", "%edx"
        );
#else
        _encrypt(pin, pout, key);
#endif
    }
}


static void decrypt(const aes* key, const char* in, char* out, unsigned nbytes) {
    unsigned nblocks = nbytes / BLOCK_SIZE_R;
    unsigned i;
    
    for(i = 0; i < nblocks; ++i) {
        char* pout = out + (i * BLOCK_SIZE_R);
        const char* pin = in + (i * BLOCK_SIZE_R);
        
#ifdef USE_AESHW
        asm volatile ("call  _aeshw_decrypt \n\t"
                      :
                      : "S" (pin), "D" (pout), "d" (key->d_key)   // pin->%esi, pout->%edi, d_key->%edx
                      : "%esi", "%edi", "%edx"
        );
#else 
        _decrypt(pin, pout, key);
#endif
    }
}


static void showmem(const char* mem, unsigned nbytes) {
    unsigned i = 0;
    while(i < nbytes) {
        printf("0x%02hhx ", mem[i++]);
        if(0 == (i % 8)) {
            printf("\n");
        }
    }
    
    printf("\n");
}

#define BENCHMARK_MB 10
#define BENCHMARK_BUFSIZ 1024 * 1024 * BENCHMARK_MB
#define BENCHMARK_NTRIES 100


static double benchmark(const aes* ctx) {
    char* mem = (char*) malloc(BENCHMARK_BUFSIZ);
    if(!mem) {
        return 0;
    }
    
    clock_t mean = 0;
    
    unsigned i = 0;
    for(i = 0; i < BENCHMARK_NTRIES; ++i) {
        clock_t start = clock();
        encrypt(ctx, mem, mem, BENCHMARK_BUFSIZ);
        clock_t end = clock();
        mean += (end - start);
    }
    
    free(mem);
    
    mean = mean / BENCHMARK_NTRIES / BENCHMARK_MB;
    printf("mean time is %llu\n", mean);

    double mbps = (double) mean / (double) CLOCKS_PER_SEC;
    return 1.0 / mbps;
//    return 1.0 / ( (double) mean / CLOCKS_PER_SEC );
}


int main(void) 
{
    if(!is_supported()) {
        printf("This CPU does not support AES hardware acceleration\n");
        return EXIT_FAILURE;
    }

    aes ctx;
    if(set_key(key, sizeof(key), both, &ctx) != aes_good) {
        printf("Failed to set the key\n");
        return EXIT_FAILURE;
    }

    char buf[sizeof(p)] = {0};
    encrypt(&ctx, p, buf, sizeof(buf));
    
    if(0 != memcmp(buf, c, sizeof(buf))) {
        printf("Encryption failed\n");
        showmem(p, sizeof(p));
        showmem(c, sizeof(c));
        showmem(buf, sizeof(buf));
       // return EXIT_FAILURE;
    }
    
    decrypt(&ctx, buf, buf, sizeof(buf));
    
    if(0 != memcmp(buf, p, sizeof(buf))) {
        printf("Decryption failed\n");
        showmem(buf, sizeof(buf));
       // return EXIT_FAILURE;
    }

    printf("Decrypted data:\n");
    showmem(buf, sizeof(buf));

    printf("Benchmarking..\n");
    double mbps = benchmark(&ctx);
    printf("%f mb/s\n", mbps);
    
    return EXIT_SUCCESS;
}
