/********************************************************************************************
 * File: bc_rc6.c
 *  
 * Purpose: Use the RC6 algorithm in CBC (or CFB) mode with 16 byte blocks and a 256-bit key.
 *
 * The RC6 algorithm's authors: Ronald L. Rivest and RSA Labs.
 * More information about RC6 at: http://www.rsa.com/rsalabs/rc6/index.html
 *
 * The code is based Dr B. R. Gladman's <brg@gladman.uk.net> algorithm implementation.
 *
 * BestCrypt module by Michael Oestergaard Pedersen <michael@daimi.au.dk>
 *
 ********************************************************************************************/

//#include <windows.h>   /* the string MUST be commented if */
                       /* we compile Windows 95 or NT drivers */
/*
#ifndef __WIN95__
#include <windef.h>
#else
#include <basedef.h>
#endif
*/
#include "bc_rc6.h"

#define f_rnd(i,a,b,c,d)						\
		u = rotl(d * (d + d + 1), 5);			\
		t = rotl(b * (b + b + 1), 5);			\
		a = rotl(a ^ t, u) + KeyAddress[i];		\
		c = rotl(c ^ u, t) + KeyAddress[i + 1]

#define i_rnd(i,a,b,c,d)						\
		u = rotl(d * (d + d + 1), 5);			\
		t = rotl(b * (b + b + 1), 5);			\
		c = rotr(c - KeyAddress[i + 1], t) ^ u;	\
		a = rotr(a - KeyAddress[i], u) ^ t

#define ECB_Encrypt(Data)							\
		a = Data[0]; b = Data[1] + KeyAddress[0];	\
		c = Data[2]; d = Data[3] + KeyAddress[1];	\
		f_rnd( 2,a,b,c,d); f_rnd( 4,b,c,d,a);		\
		f_rnd( 6,c,d,a,b); f_rnd( 8,d,a,b,c);		\
		f_rnd(10,a,b,c,d); f_rnd(12,b,c,d,a);		\
		f_rnd(14,c,d,a,b); f_rnd(16,d,a,b,c);		\
		f_rnd(18,a,b,c,d); f_rnd(20,b,c,d,a);		\
		f_rnd(22,c,d,a,b); f_rnd(24,d,a,b,c);		\
		f_rnd(26,a,b,c,d); f_rnd(28,b,c,d,a);		\
		f_rnd(30,c,d,a,b); f_rnd(32,d,a,b,c);		\
		f_rnd(34,a,b,c,d); f_rnd(36,b,c,d,a);		\
		f_rnd(38,c,d,a,b); f_rnd(40,d,a,b,c);		\
		Data[0] = a + KeyAddress[42]; Data[1] = b;	\
		Data[2] = c + KeyAddress[43]; Data[3] = d;

#define ECB_Decrypt(Data)							\
		d = Data[3]; c = Data[2] - KeyAddress[43];	\
		b = Data[1]; a = Data[0] - KeyAddress[42];	\
		i_rnd(40,d,a,b,c); i_rnd(38,c,d,a,b);		\
		i_rnd(36,b,c,d,a); i_rnd(34,a,b,c,d);		\
		i_rnd(32,d,a,b,c); i_rnd(30,c,d,a,b);		\
		i_rnd(28,b,c,d,a); i_rnd(26,a,b,c,d);		\
		i_rnd(24,d,a,b,c); i_rnd(22,c,d,a,b);		\
		i_rnd(20,b,c,d,a); i_rnd(18,a,b,c,d);		\
		i_rnd(16,d,a,b,c); i_rnd(14,c,d,a,b);		\
		i_rnd(12,b,c,d,a); i_rnd(10,a,b,c,d);		\
		i_rnd( 8,d,a,b,c); i_rnd( 6,c,d,a,b);		\
		i_rnd( 4,b,c,d,a); i_rnd( 2,a,b,c,d);		\
		Data[3] = d - KeyAddress[1]; Data[2] = c;	\
		Data[1] = b - KeyAddress[0]; Data[0] = a;

/********* Use this method for testing only ***********/

VOID
RC6_ECB_Mode( DWORD *KeyAddress,
              DWORD *SrcBuffer,
              DWORD *DstBuffer,
              BOOL Operation)	// TRUE Encrypt, FALSE Decrypt
{
	DWORD a,b,c,d,t,u;
	DWORD Buf[4];

	Buf[0] = SrcBuffer[0];
	Buf[1] = SrcBuffer[1];
	Buf[2] = SrcBuffer[2];
	Buf[3] = SrcBuffer[3];
	if (Operation == TRUE) {
		ECB_Encrypt(Buf)
	} else {
		ECB_Decrypt(Buf)
	}
	DstBuffer[0] = Buf[0];
	DstBuffer[1] = Buf[1];
	DstBuffer[2] = Buf[2];
	DstBuffer[3] = Buf[3];
}

/******************* RC6 encryption *******************/
 
VOID
Encrypt(DWORD *IVector, 
		DWORD *KeyAddress,
		DWORD *SrcBuffer,
		DWORD *DstBuffer,
		DWORD Length )    // in bytes
{
	DWORD i,j,a,b,c,d,t,u;
	DWORD ivLocal[4];

	for(i=0; i<4; i++) ivLocal[i] = IVector[i&1];
	for(i=0; Length > 0; Length -= 16, i=i+4)
	{	
		if (Length < 16)
		{	
			ECB_Encrypt(ivLocal)
			for (j = 0; j < Length; j++)
				((BYTE *)DstBuffer)[(i*4)+j] = ((BYTE *)SrcBuffer)[(i*4)+j] ^ ((BYTE *)ivLocal)[j];
			return;
		}

		#ifdef __CBCMODE
		ivLocal[0] ^= SrcBuffer[i  ];
		ivLocal[1] ^= SrcBuffer[i+1];
		ivLocal[2] ^= SrcBuffer[i+2];
		ivLocal[3] ^= SrcBuffer[i+3];
		#endif

		// Encrypt data with RC6
		ECB_Encrypt(ivLocal)

		#ifdef __CBCMODE
		DstBuffer[i  ] = ivLocal[0];
		DstBuffer[i+1] = ivLocal[1];
		DstBuffer[i+2] = ivLocal[2];
		DstBuffer[i+3] = ivLocal[3];
		#else
		ivLocal[0] ^= SrcBuffer[i  ];
		ivLocal[1] ^= SrcBuffer[i+1];
		ivLocal[2] ^= SrcBuffer[i+2];
		ivLocal[3] ^= SrcBuffer[i+3];
		DstBuffer[i  ] = ivLocal[0];
		DstBuffer[i+1] = ivLocal[1];
		DstBuffer[i+2] = ivLocal[2];
		DstBuffer[i+3] = ivLocal[3];
		#endif
	}
}

/******************* RC6 decryption *******************/

VOID
Decrypt(DWORD *IVector, 
		DWORD *KeyAddress,
		DWORD *SrcBuffer,
		DWORD *DstBuffer,
		DWORD Length )    // in bytes
{
	DWORD i,j,a,b,c,d,t,u;
	#ifdef __CBCMODE
	DWORD ivLocal[4], ivNew[4], buf[4];
	#else
	DWORD ivLocal[4], ivOld[4];
	#endif

	for(i=0; i<4; i++) ivLocal[i] = IVector[i&1];
	for(i=0; Length > 0; Length -= 16, i=i+4)
	{	
		if (Length < 16)
		{	
			ECB_Encrypt(ivLocal)
			for (j = 0; j < Length; j++) {
				((BYTE *)DstBuffer)[(i*4)+j] = ((BYTE *)SrcBuffer)[(i*4)+j] ^ ((BYTE *)ivLocal)[j];
			}
			return;
		}

		#ifdef __CBCMODE
		for(j=0; j<4; j++) buf[j] = ivNew[j] = SrcBuffer[i+j];
		ECB_Decrypt(buf)
		for(j=0; j<4; j++) 
		{
			DstBuffer[i+j] = ivLocal[j] ^ buf[j];
			ivLocal[j] = ivNew[j];
		}
		#else
		ECB_Encrypt(ivLocal)
		ivOld[0] = SrcBuffer[i  ];
		ivOld[1] = SrcBuffer[i+1];
		ivOld[2] = SrcBuffer[i+2];
		ivOld[3] = SrcBuffer[i+3];
		DstBuffer[i  ] = ivLocal[0] ^ SrcBuffer[i  ];
		DstBuffer[i+1] = ivLocal[1] ^ SrcBuffer[i+1];
		DstBuffer[i+2] = ivLocal[2] ^ SrcBuffer[i+2];
		DstBuffer[i+3] = ivLocal[3] ^ SrcBuffer[i+3];
		ivLocal[0] = ivOld[0];
		ivLocal[1] = ivOld[1];
		ivLocal[2] = ivOld[2];
		ivLocal[3] = ivOld[3];
		#endif
	}
}

/***************** RC6 Key Extension ******************/

BOOL
KeyExtend( PUCHAR KeySource,		//32 8-bits bytes - 256-bit key
           PDWORD KeyDestination )
{
	DWORD i,j,k,a,b,l[8],t;
	KeyDestination[0] = 0xb7e15163;

	for(k = 1; k < 44; ++k)
		KeyDestination[k] = KeyDestination[k - 1] + 0x9e3779b9;

	for(k = 0; k < 8; ++k)
		l[k] = ((DWORD *)KeySource)[k];

	t = 7;
	a = b = i = j = 0;

	for(k = 0; k < 132; ++k)
	{
		a = rotl(KeyDestination[i] + a + b, 3); b += a;
		b = rotl(l[j] + b, b);
		KeyDestination[i] = a; l[j] = b;
		i = (i == 43 ? 0 : i + 1);
		j = (j == t ? 0 : j + 1);
	}
	return TRUE;
}


/***************** RC6 Universal ECB mode ******************/


void Encrypt_ECB( BYTE  *Input,        /* 8 bytes of input data              */
                  BYTE  *Output,       /* 8 bytes of output data             */
                  BYTE  *ExtendedKey)
{
	RC6_ECB_Mode( (DWORD *)ExtendedKey, (DWORD *)Input, (DWORD *)Output, TRUE );
}


void Decrypt_ECB( BYTE  *Input,        /* 8 bytes of input data              */
                  BYTE  *Output,       /* 8 bytes of output data             */
                  BYTE  *ExtendedKey)
{
	RC6_ECB_Mode( (DWORD *)ExtendedKey, (DWORD *)Input, (DWORD *)Output, FALSE );
}


