/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 *
 */

#ifndef BC_KERNEL
#error "This file can only be included in kernel code"
#endif // BC_KERNEL

#ifndef BC_KERNEL_BUFFER_H
#define BC_KERNEL_BUFFER_H

#include <IOKit/IOLib.h>

/**
 * Simple utility auto memory buffer.
 * Designed to store user memory data on requests.
 */
class BCAutoBuffer
{
public:
	
	BCAutoBuffer() : m_mem(NULL), m_nbytes(0) {
	}
	
	~BCAutoBuffer() {
		this->free();
	}
	
	bool alloc(size_t nbytes) {
		if(m_mem != NULL) {
			return false;
		}
		
		m_mem = (unsigned char*) IOMalloc(nbytes);
		if(!m_mem) {
			return false;
		}
		
		memset(m_mem, 0, nbytes);
		
		m_nbytes = nbytes;
		return true;
	}
	
	void free() {
		if(m_mem == NULL) {
			return;
		}
		
		memset(m_mem, 0, m_nbytes);
		IOFree(m_mem, m_nbytes);
		m_mem = NULL;
		m_nbytes = 0;
	}
	
	unsigned char* mem() {
		return m_mem;
	}
	
	size_t size() {
		return m_nbytes;
	}
	
	int copyin(user_addr_t addr) {
		if(!m_mem) {
			return kIOReturnNotReady;
		}
		
		return ::copyin(addr, m_mem, m_nbytes);
	}
	
	int copyout(void* addr) {
		if(!m_mem) {
			return kIOReturnNotReady;
		}
		
		return ::copyout(m_mem, (user_addr_t)addr, m_nbytes);
	}
	
	int init_from_user(const void* addr, size_t nbytes) {
		if(m_mem) {
			return kIOReturnNotReady;
		}
		
		if(!this->alloc(nbytes)) {
			return kIOReturnNoMemory;
		}
		
		return this->copyin((user_addr_t)addr);
	}
	
	
private:
	
	// no copying of any kind.
	BCAutoBuffer(BCAutoBuffer const & rhv) {
	}
	
	BCAutoBuffer& operator = (BCAutoBuffer const & rhv) {
		return *this;
	}
	
private:
	
	unsigned char*	m_mem;
	size_t		m_nbytes;
};


#endif

