/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 *
 * Common file between kernel and user space.
 * Provider CTL interface for user client.
 */

#ifndef BC_KERNEL_CTL_H
#define BC_KERNEL_CTL_H


//#include "BCKernel.h"
#include "container/bctypes.h"

#ifdef BC_KERNEL
#define BC_RESULT(code) iokit_vendor_specific_err( (code) )
#endif

#define BC_RESULT_KERNEL_MASK (sys_iokit|sub_iokit_vendor_specific)

enum {
	kBCKernel_CTL = 0xBC000008, // magic ctl code number.
};

	
/// BCController user client method tags.
enum 
{
	kBCCTL_StoreKey = 0,
	kBCCTL_FreeKey,
	kBCCTL_Process,
	kBCCTL_SetDisk,
	kBCCTL_Benchmark,
	//kBCCTL_EnableAESNI,
    
	kBCCTL_Total	// last
};


enum {
	kBCCTL_Magic = 0xBC000008,
};

#pragma pack(push, 4)

// This is to ensure that structures with pointers that are passed to kernel are always the same size for 32-bit app and 64-bit kernel.
typedef union {
	void*		ptr;
	uint64_t	_pad;
} bc_userptr;


// Same of the above but for constant pointers.
typedef union {
	const void*	ptr;
	uint64_t	_pad;
} bc_const_userptr;


// Set key control data
typedef struct 
{
	bc_const_userptr	addr;
	unsigned int		size;
} bc_ctl_key;


// Encrypt/decrypt control data
typedef struct 
{
	bc_alg_id		cipher;
	bc_mode_id		mode;
	uint64_t		iv;
	bc_const_userptr	pool;
	bc_userptr		data;
	unsigned		nbytes;
	int			with_key;	// if 1 union contains key data, if 0 - handle.
	union {
		bc_ctl_key	data;
		bc_key_handle	handle;
	} key;
	bc_proc_type		proc;
} bc_ctl_process;


// Disk init data.
typedef struct 
{
	bc_const_userptr	path;		// container file path
	unsigned		pathlen;	// container file path length.
	bc_key_handle		key;		// key data handle.
	bc_alg_id		alg_id;		// block cipher id.
	bc_mode_id		mode_id;	// block cipher mode id.
	bc_const_userptr	pool;		// random data pool
	int			ro;		// set disk with read-only access. can be overriden if driver sees that it cannot write to container file.
	unsigned		nvecs;		// total number of extent vectors.
	bc_userptr		extents;	// extent array pointer.
	int			auto_mount;	// set autodiskmount property.
} bc_ctl_disk;


// Algorithm benchmarking
typedef struct 
{
	bc_alg_id		alg_id;		// block cipher
	bc_mode_id		mode_id;	// mode
	unsigned		bufsiz_mb;	// benchmarking buffer size in megabytes.
	unsigned		nreps;		// number of encrypt/decrypt sequencies.
} bc_ctl_benchmark;


#pragma pack(pop)

#endif
