/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 *
 */

#ifndef BC_KERNEL_CONTROLLER_CLIENT_H
#define BC_KERNEL_CONTROLLER_CLIENT_H

#ifdef BC_KERNEL // kernel impl.
#include <IOKit/IOUserClient.h>
#include "BCCTL.h"


class BCController;


/**
 * Controller user client.
 * Assepts requests from user space targeted at controller.
 */
class BCControllerUserClient : public IOUserClient
{
OSDeclareDefaultStructors(BCControllerUserClient)
public:
	
	bool start(IOService* provider);
	
	IOReturn clientClose( void );
	IOReturn clientDied( void );
	
	bool initWithTask(task_t owningTask, void* securityToken, UInt32 type, OSDictionary* properties);
	
	IOReturn externalMethod(uint32_t selector, IOExternalMethodArguments* arguments, IOExternalMethodDispatch* dispatch = 0, OSObject* target = 0, void* reference = 0);
	
	
protected:

	// IOUserClient dispatch glue code.
	// Extract parameters and dispatch.
	static IOReturn s_dispatch_ctl(OSObject * target, void * reference, IOExternalMethodArguments * arguments);
	

	
protected:
	
	/// kBCCTL_StoreKey.
	IOReturn store_key(bc_ctl_key* arg, bc_key_handle* out_handle);
	
	/// kBCCTL_FreeKey.
	IOReturn free_key(bc_key_handle key_handle);
	
	/// kBCCTL_Process.
	IOReturn process(bc_ctl_process* arg);
	
	/// kBCCTL_SetDisk
	IOReturn set_disk(bc_ctl_disk* arg);
	
	/// kBCCTL_Benchmark
	IOReturn benchmark(bc_ctl_benchmark* arg, uint64_t* out_time);
	
    /// kBCCTL_EnableAESNI
    //IOReturn enable_aeshw(int do_enable);
    
private:
	
	BCController* m_controller;
};


#endif // BC_KERNEL
#endif // BC_CONTROLLER_USER_CLIENT_H
