/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 */

#ifndef BC_KERNEL_DEFS_H
#define BC_KERNEL_DEFS_H

#include <IOKit/IOLib.h>

#define BC_LOG(fmt, args...)  { IOLog(fmt, ## args); }

#ifdef BC_DEBUG
#define MACH_ASSERT 1
#include <kern/debug.h>
#define BC_LOG_DEBUG(fmt, args...)	IOLog(fmt, ## args)
#define BC_ASSERT(pred)			{ if(! (pred) ) panic("BC Assertion %s failed", __STRINGIFY(pred)); }
#else
#define BC_LOG_DEBUG(fmt, args...)
#define BC_ASSERT(pred)			
#endif // BC_DEBUG


/// Logging for IOKit objects.
#define BC_IOLOG(fmt, args...)		BC_LOG(" [%s:%s] " fmt, this->getName(), __func__, ## args)
#define BC_IOLOG_DEBUG(fmt, args...)	BC_LOG_DEBUG(" [%s:%s] " fmt, this->getName(), __func__, ## args)
#define BC_IOLOG_TRACE			BC_IOLOG_DEBUG("trace\n")


/// Macro for symbols to be visible inside all bc kernel implementation but not outside.
#define BC_KERNEL_PRIVATE __private_extern__


/// Macro to compose ioki vendor specific return code from libkernel codes.
#define BC_RESULT(code) iokit_vendor_specific_err( (code) )


#define BC_NOCOPY(class)	\
private:			\
	class ( class const & rhv ) {}; \
	class & operator = ( class const & rhv ) {};


#endif // BC_DEFS_H