/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 *
 */

#ifndef BC_KERNEL_DEVICE_H
#define BC_KERNEL_DEVICE_H


#ifdef BC_KERNEL
#include <IOKit/storage/IOBlockStorageDevice.h>

class BCDiskDriver;


/**
 * IOKit storage family interface nub.
 * Redirects most requests to BCDiskDriver.
 * Instantiated and registered by BCDiskDriver.
 */
class BCDevice : public IOBlockStorageDevice {
OSDeclareDefaultStructors(BCDevice);
public:
	
	/**
	 * Set some initial state.
	 */
	bool init(OSDictionary * properties = 0);
	
	
	static BCDevice* create();
	
	
	/**
	 * Calling this will publish nub for storage level.
	 * Driver should be absolutely prepared to start servicing requests through published nub *before* calling it.
	 */
	bool attach(IOService* provider);

	
	IOReturn	doEjectMedia(void);
	
	IOReturn	doFormatMedia(UInt64 byteCapacity);
	
	UInt32		doGetFormatCapacities(UInt64* capacities, UInt32 capacitiesMaxCount) const;
	
	
	IOReturn	doLockUnlockMedia(bool doLock);
	
	IOReturn	doSynchronizeCache(void);
	
	
	
	char *	getVendorString(void);
	
	char *	getProductString(void);
	
	char *	getRevisionString(void);
	
	char *	getAdditionalDeviceInfoString(void);
	
	
	
	IOReturn	reportBlockSize(UInt64 *blockSize);
	
	IOReturn	reportMaxValidBlock(UInt64 *maxBlock);
	
	
	
	IOReturn	reportRemovability(bool *isRemovable);
	
	IOReturn	reportEjectability(bool *isEjectable);
	
	IOReturn	reportLockability(bool *isLockable);
	
	
	
	IOReturn	reportMaxReadTransfer (UInt64 blockSize, UInt64 *max);
	
	IOReturn	reportMaxWriteTransfer(UInt64 blockSize,UInt64 *max);
	
	
	
	IOReturn	reportMediaState(bool *mediaPresent,bool *changedState);
	
	IOReturn	reportPollRequirements(bool *pollRequired,
					       bool *pollIsExpensive);
	
	
	IOReturn	getWriteCacheState(bool *enabled);
	
	IOReturn	setWriteCacheState(bool enabled);

	
	IOReturn	reportWriteProtection(bool *isWriteProtected);
	
	IOReturn	doAsyncReadWrite(IOMemoryDescriptor *buffer,
					 UInt64 block, UInt64 nblks,
					 IOStorageCompletion completion);
	
	IOReturn	doAsyncReadWrite(IOMemoryDescriptor *buffer,
					 UInt64 block, UInt64 nblks,
					 IOStorageAttributes *attributes,
					 IOStorageCompletion *completion);
	
	IOReturn doSyncReadWrite(IOMemoryDescriptor *buffer, UInt32 block, UInt32 nblks);

private:
	
	BCDiskDriver*	m_driver;
	bool		m_locked;
};

#endif // BC_KERNEL
#endif // BC_KERNEL_DEVICE_H

