#ifndef __BCKERNEL_DEFS_H__
#define __BCKERNEL_DEFS_H__

#include <stdint.h>
#include "BCKernel.h"

/// Kernel error codes.
typedef enum {
    kBCKernel_Success = 0,
    kBCKernel_NotInitialized,
    kBCKernel_NoMemory,
    kBCKernel_Unsupported,
    kBCKernel_Busy,
    kBCKernel_UnknownAlg,
    kBCKernel_UnsupportedMode,
    kBCKernel_InvalidKey,
    kBCKernel_InvalidBuffer,
    kBCKernel_InternalError,
} bc_kern_result;


/// Map error code to description string.
static inline const char* bc_kernel_perror(bc_kern_result code)
{
    switch (code) {
        case kBCKernel_Success:		return "Success";
        case kBCKernel_NotInitialized:	return "BC Kernel service not initialized";
        case kBCKernel_NoMemory:	return "No memory to perform kernel operation";
        case kBCKernel_UnknownAlg:	return "Unknown algorithm";
        case kBCKernel_UnsupportedMode:	return "Unsupported mode";
        case kBCKernel_InvalidKey:	return "Invalid key";
        case kBCKernel_InvalidBuffer:	return "Invalid buffer";
        case kBCKernel_InternalError:	return "Internal platform error";
        default:			return "Unknown kernel error";
    };
}

typedef struct {
    uint64_t	offset;
    uint64_t	length;
} bc_bvec;


typedef struct {
    unsigned	nvecs;
    bc_bvec*	extents;
} bc_bmap;

#ifndef BC_KERNEL
/// Block cipher definition.
typedef struct bc_alg
{
    int		id;
    char		name[kBCKernel_AlgNameSize];
    size_t		key_length;
    size_t		block_size;
    size_t		key_size[kBCKernel_TotalModes]; // mode-specific key sizes
    int		supported_modes;
} bc_alg;

/*struct bc_alg {
    // precalculated for convinience.
    size_t  key_size;
    size_t  extended_key_size;
    
    const struct bc_cipher* cipher;
    const struct bc_mode* mode;
    
    void* key;
};*/

extern bc_cipher_result bc_kernel_benchmark(bc_alg_id alg_id, bc_mode_id mode_id, unsigned bufsiz_mb, unsigned nreps, uint64_t* mean);

#endif


#endif // __BCKERNEL_DEFS_H__
