/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 */

#ifndef BC_KERNEL_CIPHER_PLUGIN_H
#define BC_KERNEL_CIPHER_PLUGIN_H

#include "crypto/cipher.h"

/// Opaque handle for plugin code.
typedef struct bc_plugin* bc_plugin_ref;


#ifdef __cplusplus
extern "C" {
#endif	
	
	
	/**
	 * Register cipher plugin.
	 * @p cipher	Block cipher definitions.
	 * @p out_ref	Returned new plugin reference on success.
	 * @return	KIOReturnBusy if cipher with this id is already registered.
	 *		kIOReturnNoMemory/kIOReturnInternalError on system failures.
	 */
	extern int bc_register_plugin(const bc_cipher* cipher, bc_plugin_ref* out_ref);
	
	
	/**
	 * Unregister cipher plugin.
	 * @p ref	Plugin reference supplied on registering.
	 * @return	kIOReturnBadArgument if reference does not point to registered plugin.
	 *		kIOReturnBusy if cipher is currently in use.
	 */
	extern int bc_unregister_plugin(bc_plugin_ref ref);
	
	
	
#ifdef __cplusplus
}
#endif

#endif // BC_KERNEL_CIPHER_PLUGIN