/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 *
 * This is a common initialization code code for bc algorithm plugins.
 * It requires a plugin source to export include file called bc_algorithm.h with the following definitions:
 *
 * BC_PLUGIN_NAME	plugin service name. should have BC_ prefix in it.
 * BC_ALGORITHM_NAME	plugin algorithm name string.
 * BC_ALGORITHM_ID	plugin unique id.
 * BC_KEY_LENGTH	raw key length in bits.
 * BC_BLOCK_SIZE	cipher block size in bytes.
 * BC_EXTENDED_KEY_SIZE	cipher extended key size in bytes.
 *
 * Encrypt		CBC encryption routine.
 * Decrypt		CBC decryption routine.
 * Encrypt_ECB		ECB encryption routine.
 * Decrypt_ECB		ECB decryption routine.
 * ExtendKey		Key extention routine.
 */


#include "bc_algorithm.h"	// algorithm plugin standard include.
#include "crypto/cipher.h"
#include "BCKernelPlugin.h"

#include <mach/mach_types.h>
#include <mach/kmod.h>
#include <libkern/libkern.h>


/// Registered plugin ref to be used at unregistering.
static bc_plugin_ref g_ref;

/// Plugin block cipher definition.
static bc_cipher g_cipher = 
{
	BC_ALGORITHM_ID,
	BC_ALGORITHM_NAME,
	BC_KEY_LENGTH,
	BC_BLOCK_SIZE,
	BC_EXTENDED_KEY_SIZE,
	
	KeyExtend,
	Encrypt_ECB,
	Decrypt_ECB,

#ifdef BC_ALGORITHM_NO_CBC
    NULL,
    NULL,
#else
    Encrypt,
    Decrypt,
#endif
};


/// Register plugin.
extern kern_return_t bc_plugin_start(kmod_info_t* ki, void* d) {
	int rc = bc_register_plugin(&g_cipher, &g_ref);
	if(KERN_SUCCESS != rc) {
		return KERN_FAILURE;
	}
	
	return KERN_SUCCESS;
}


/// Unregister plugin.
extern kern_return_t bc_plugin_stop(kmod_info_t* ki, void* d) {
	int rc = bc_unregister_plugin(g_ref);
	if(KERN_SUCCESS != rc) {
		return KERN_FAILURE;
	}
	
	return KERN_SUCCESS;
}

//extern kern_return_t _start(kmod_info_t* ki, void* d);
//extern kern_return_t _stop(kmod_info_t* ki, void* d);

#ifndef BC_KEXT_TARGET
/// osx manual kext glue
KMOD_EXPLICIT_DECL(com.jetico.kext.BC_PLUGIN_NAME, "1.0", bc_plugin_start, bc_plugin_stop)
__private_extern__ kmod_start_func_t* _realmain = bc_plugin_start;
__private_extern__ kmod_start_func_t* _antimain = bc_plugin_stop;
__private_extern__ int _kext_apple_cc = __APPLE_CC__;

#endif
