/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 *
 */

#include "BCKernel.h"

#include <stdio.h>
#include <string.h>

static const char key[] =
{
	// AES 256 bit key1
	0xf8, 0xd4, 0x76, 0xff, 0xd6, 0x46, 0xee, 0x6c, 0x23, 0x84, 0xcb, 0x1c, 0x77, 0xd6, 0x19, 0x5d,
	0xfe, 0xf1, 0xa9, 0xf3, 0x7b, 0xbc, 0x8d, 0x21, 0xa7, 0x9c, 0x21, 0xf8, 0xcb, 0x90, 0x02, 0x89,
	
	// AES 128 bit tweak key2
	0xa8, 0x45, 0x34, 0x8e, 0xc8, 0xc5, 0xb5, 0xf1, 0x26, 0xf5, 0x0e, 0x76, 0xfe, 0xfd, 0x1b, 0x1e
};

static const char plaintext[] =
{
	0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46
};

static uint64_t iv = 1;

// C-text is different from IEEE draft because our IV is multiplied by 512
static const char cyphertext[] =
{
	0x2b, 0x37, 0x9c, 0xcc, 0xa0, 0x40, 0x4b, 0xf7, 0xf3, 0xcd, 0x6a, 0x98, 0x35, 0x17, 0x97, 0x36
};

static const char pool[512] = {0};


int bc_test_aes_lrw()
{
	int rc;
	char buf[sizeof(plaintext)] = {0};
	memcpy(buf, plaintext, sizeof(plaintext));

	bc_alg aes;
	rc = bc_kernel_lookup_alg_name("AES", &aes);
	if(kBCKernel_Success != rc) {
		printf("AES lookup failed: %d\n", rc);
		return -1;
	}
	
	rc = bc_kernel_process_with_key(aes.id, kBCMode_LRW, kBCProcess_Encrypt, key, sizeof(key), iv, pool, buf, sizeof(buf));
	if(kBCKernel_Success != rc) {
		printf("Process failed: %s\n", bc_kernel_perror(rc));
		return -1;
	}
	
	if(0 != memcmp(cyphertext, buf, sizeof(buf))) {
		printf("Encryption test failed\n");
		return 0;
	}
	
	rc = bc_kernel_process_with_key(aes.id, kBCMode_LRW, kBCProcess_Decrypt, key, sizeof(key), iv, pool, buf, sizeof(buf));
	if(kBCKernel_Success != rc) {
		printf("Process failed: %s\n", bc_kernel_perror(rc));
		return -1;
	}
	
	if(0 != memcmp(plaintext, buf, sizeof(buf))) {
		printf("Decryption test failed\n");
		return 0;
	}
	
	return 1;
}
