/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 *
 */


#include "BCKernel.h"

#include <stdio.h>
//#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>

static void print_usage()
{
	printf("bcattach container_file\n");
}


int main(int argc, char** argv) 
{
	if(argc < 2) {
		print_usage();
		return EXIT_FAILURE;
	}

	const char* cpath = argv[1];
	int rc = access(cpath, F_OK);
	if(rc != 0) {
		if(ENOENT == errno) {
			printf("Container file %s does not exist\n", cpath);
			return EXIT_FAILURE;
		} else {
			printf("access call failed with %d\n", errno);
			return EXIT_FAILURE;
		}
	}

	int fd = open(cpath, O_RDONLY);
	if(fd < 0) {
		printf("open failed with %d\n", errno);
		return EXIT_FAILURE;
	}
	
	struct stat cstat;
	rc = fstat(fd, &cstat);
	if(rc != 0) {
		printf("fstat failed with %d\n", errno);
		return EXIT_FAILURE;
	}
	
	rc = close(fd);
	if(rc != 0) {
		printf("close failed with %d\n", errno);
		return EXIT_FAILURE;
	}
	
	uint64_t fsize = cstat.st_size;
	uint64_t nblocks = fsize / kBCKernel_BlockSize;
	if(fsize % kBCKernel_BlockSize) {
		printf("Container file %s size %llu is not alighed to block size. Disk will be truncated to %llu\n", cpath, fsize, nblocks);
	}
	
	unsigned char key[64] = {0};
	unsigned char pool[kBCKernel_PoolSize] = {0};
	bc_key_handle handle;
	
	rc = bc_kernel_store_key(key, sizeof(key), &handle);
	if(kBCKernel_Success != rc) {
		printf("Store key failed with %s\n", bc_kernel_perror(rc));
		return EXIT_FAILURE;
	}
	
	bc_alg noop;
	rc = bc_kernel_lookup_alg_name("NOOP", &noop);
	if(kBCKernel_Success != rc) {
		printf("NOOP cipher not found\n");
		return EXIT_FAILURE;
	}
	
	rc = bc_kernel_set_disk(noop.id, kBCMode_NOOP, handle, cpath, 0, nblocks, pool, 0);
	if(kBCKernel_Success != rc) {
		printf("Disk test failed with %s\n", bc_kernel_perror(rc));
		return EXIT_FAILURE;
	}
	
	rc = bc_kernel_free_key(handle);
	if(kBCKernel_Success != rc) {
		printf("Free key failed with %s\n", bc_kernel_perror(rc));
		return EXIT_FAILURE;
	}
	
	return EXIT_SUCCESS;
}

