/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 */

#include "BCKernel.h"
#include <stdio.h>
#include <stdlib.h>

extern int bc_test_aes_cbc();
extern int bc_test_aes_lrw();
extern int bc_test_aes_xts();
extern int bc_test_noop();

static void print_cipher(bc_alg* alg)
{
	printf("\nCipher id 0x%x:\n"
	       " name:		%s\n"
	       " key length:	%zu\n"
	       " block size:	%zu\n"
	       " CBC/LRW/XTS:	%d/%d/%d\n",
	       alg->id,
	       alg->name,
	       alg->key_length,
	       alg->block_size,
	       bc_alg_is_mode_supported(alg, kBCMode_CBC),
	       bc_alg_is_mode_supported(alg, kBCMode_LRW),
	       bc_alg_is_mode_supported(alg, kBCMode_XTS));
}

int main(int argc, char **argv)
{
	int rc;
	
	size_t total;
	bc_alg* list;
	
	rc = bc_kernel_list_algs(&list, &total);
	if(kBCKernel_Success != rc) {
		printf("No plugins\n");
		return EXIT_FAILURE;
	}
	
	printf("%zu total ciphers\n", total);
	size_t i;
	for(i = 0; i < total; ++i) {
		print_cipher(list + i);
	}
	
	bc_kernel_free_alg_list(list);
	
	rc = EXIT_SUCCESS;
    
	if(!bc_test_aes_cbc()) {
		printf("AES-CBC test failed\n");
        rc = EXIT_FAILURE;
	} else {
        printf("AES-CBC test passed\n");
    }

	if(!bc_test_aes_lrw()) {
		printf("AES-LRW test failed\n");
        rc = EXIT_FAILURE;
	} else {
        printf("AES-LRW test passed\n");
    }
	
	if(!bc_test_aes_xts()) {
		printf("AES-XTS test failed\n");
        rc = EXIT_FAILURE;
	} else {
        printf("AES-XTS test passed\n");
    }
	
    return rc;
}
