/*
 *  Copyright 2010 Jetico, Inc. All rights reserved.
 *
 */

#include "test_data.h"
#include <stdio.h>

int bc_load_test_data(const char* file, bc_test_data* out_test)
{
	memset(out_test, 0, sizeof(*out_test));
	
	FILE* data = fopen(file, "br");
	if(!data) {
		return -1;
	}
	
	int rc;
	
	// read data size and bytes
	rc = fread(&out_test->nbytes, data, sizeof(out_test->nbytes));
	if(rc != 0) {
		return rc;
	}
	
	out_test->data = (unsigned char*) malloc(out_test->nbytes);
	if(!out_test->data) {
		return -1;
	}
	
	rc = fread(out_test->data, data, out_test->nbytes);
	if(rc != 0) {
		return rc;
	}
	
	
	// read key size and bytes.
	rc = fread(&out_test->keysize, data, sizeof(out_test->keysize));
	if(rc != 0) {
		return rc;
	}
	
	out_test->key = (unsigned char*) malloc(out_test->keysize);
	if(!out_test->key) {
		return -1;
	}
	
	rc = fread(out_test->key, data, out_test->keysize);
	if(rc != 0) {
		return rc;
	}
	
	
	// read iv.
	rc = fread(&out_test->iv, data, sizeof(out_test->iv));
	if(rc != 0) {
		return rc;
	}
	
	
	// read pool if test data provides it.
	if(!feof(data)) {
		out_test->pool = malloc(kBCPoolSize);
		if(!out_test->pool) {
			return -1;
		}
		
		rc = fread(out_test->pool, data, kBCPoolSize);
		if(rc != 0) {
			return rc;
		}
	}

	fclose(data);
	return 0;
}


void bc_free_test_data(bc_test_data* test)
{
	if(test->data) {
		free(test->data);
	}
	
	if(test->key) {
		free(test->key);
	}
	
	if(test->pool) {
		free(test->pool);
	}
}
	
	
