/*
 * Copyright 2015 Jetico, Inc. All rights reserved.
 *
 * Key files processing implementation.
 */

#include "keyFiles.h"
#include "../certificate/CertificateLib/hashAlgs/hashAlgs.h"
#include "../container/standard.h"

static int makeChannelDigestInternal( int hashAlgId, BYTE *digest, iChannel *channel, DWORD maxSize )
{
	bc_u64 size = min( maxSize, channel->getSize() );	

	if ( size == 0 )
		return 0;

	cPTB ptb;
	if ( ptb.Allocate( (int)size ) == NULL )
		return -1;
    
    if (channel->readBytes( ptb.buffer, (DWORD)size ) != size)
        return -1;

	return ha_makeDigestEx( hashAlgId, digest, ptb.buffer, (DWORD)size );
}

static int makeChannelDigest( int hashAlgId, BYTE *digest, iChannel *channel, DWORD maxSize = 1024 * 1024 )
{
	if ( !channel->open( 1 ) )
		return -1;

	int res = makeChannelDigestInternal( hashAlgId, digest, channel, maxSize );

	channel->close();

	return res;
}

static int compare( BYTE *bf, int p0, int p1, int size )
{
	return memcmp( bf + p0 * size, bf + p1 * size, size );
}

static bool swap( BYTE *bf, int p0, int p1, int size )
{
	BYTE * p = new BYTE[ size ];
	if ( p == NULL )
		return false;

	memcpy( p, bf + p0 * size, size );
	memcpy( bf + p0 * size, bf + p1 * size, size );
	memcpy( bf + p1 * size, p, size );

	return true;
}

static bool sortBuffer( BYTE *bf, int bfSize, int sortSize )
{
	if ( ( bfSize % sortSize ) != 0 )
		return false;
	
	// Booble sort..
	for( int q = bfSize / sortSize; q > 1; q-- )
	{
		for( int i = 0; i < q - 1; i++ )
		{
			if ( compare( bf, i, i + 1, sortSize ) > 0 )
				if ( !swap( bf, i, i + 1, sortSize ) )
					return false;
		}
	}

	return true;
}

static bool fillHashTable( cPTB &ptbHashes, iChannel **keyFilesT, int keyFilesQ, int hashAlgId )
{
	int digestSize;
	if ( ha_getDigestSize( &digestSize, hashAlgId ) != hae_ok )
		return false;

	if ( ( hashAlgId == -1 ) || ( digestSize == 0 ) )
		return false;
			
	for( int i = 0; i < keyFilesQ; i++ )
	{
		BYTE *digest = ptbHashes.AddBytes( NULL, digestSize );

		if ( digest == NULL )
			return false;

		int res = makeChannelDigest( hashAlgId, digest, keyFilesT[i] );

		if ( res != 0 )
			return false;
	}
	
	return sortBuffer( ptbHashes.buffer, ptbHashes.bufferSize, digestSize );
}

//
//	generateSaltBuffer
//		Walks table of channels and generates new salt to ptbSalt
//
bool generateSaltBuffer( cPTB &ptbSalt, iChannel **keyFilesT, int keyFilesQ, int hashAlgId, const bc_u8 *saltBuffer, int saltSize )
{
	if ( keyFilesQ == 0 )
		return false;

	ptbSalt.Free();

	if ( !fillHashTable( ptbSalt, keyFilesT, keyFilesQ, hashAlgId ) )
		return false;

	if ( ptbSalt.WriteBytes( saltBuffer, saltSize ) != saltSize )
		return false;

	return true;
}
