/*
 * Copyright 2011 Jetico, Inc. All rights reserved.
 */

#ifndef KGGHOST_DBHEADER_H
#define KGGHOST_DBHEADER_H

#include "../../container/intsize.h"

#pragma pack(push, 1)

#define DB_REENCRYPTED_LABEL			"RE_KeyGenID"
#define DB_HEADER_SIGNATURE 			"LOCOS94 "
#define DB_HEADER_VOLUME_LABEL			"BC_KeyGenID"
#define BCV8_VERSION_TAG				3 // PAV::Really BCV8 uses 8, BCV7 first used version field as 7 ( see BC v.7.10 - 31.10.2003 )


enum {
	kBCV8_HeaderSize	= 1024,
	kBCV8_InitialDBSize	= 1024 * 4,
	kBCV8_MaximumDBSize	= kBCV8_InitialDBSize * 4,
	kBCV8_JmpCodeSize	= 3,
	kBCV8_SignatureSize	= 8,
	kBCV8_IdLength		= 4,
	kBCV8_OccupiedTag	= 28,
	kBCV8_VolumeLabel	= 11,
	kBCV8_DescriptionLength	= 42,
    kBCV8_SparcedPositionLength = 8,
	kBCV8_DigestSize32	= 32,
	kBCV8_DigestSize64	= 64,
	kBCV8_MaxKeys		= 64,
	kBCV8_BlockSize		= 16, 
	kBCV8_IVSize		= 16,
	kBCV8_HiddenPartMapIV	= 1,
	
	kBCV8_InitialDataBlockSize	= 4096,
	kBCV8_DataBlockExtentionUnit	= 4096,
	kBCV8_MaximumExtKeyBlockSize	= kBCV8_DataBlockExtentionUnit * 4,
	kBCV8_MaximumDataBlockSize	= kBCV8_InitialDataBlockSize + kBCV8_MaximumExtKeyBlockSize,
    kBCV8_HiddenPartSpaceMapSize	= 4096
};


/**
 * Single key slot info.
 * All information is initialized and interpreted only by key generators except for size field where we assume 0 to be a free slot.
 */
typedef struct {
	bc_u16		size;	// encoded key data size.
	bc_16		type;	// encoding type. signed 16 bit value.
	bc_u32		param;	// optional key parameter.
} BCKeySlot;


/**
 * V8 datablock header format.
 * Real header size is 1kb and the rest of the data container padding and digest for header encryption.
 */
typedef struct db_header 
{
	bc_u8		jmpCode[kBCV8_JmpCodeSize];		// On old linux version holds container lock flag.
	bc_u8		signature[kBCV8_SignatureSize];		// Format version signature, i.e. LOCOSXX
	bc_u8		cid[kBCV8_IdLength];			// Container id for central management authentication.
	bc_u8		occupiedTag[kBCV8_OccupiedTag];		// Unused?
	char		volumeLabel[kBCV8_VolumeLabel];		// "BC_KeyGenID" for this version
	
	// bc_u32		keyGenId;				// Key generator identifier
	// bc_u32		version;				// Container version
	bc_u16	wKeyGenId;		// Key generator ID 5 identifier...
	bc_u16	wVersion;		// Key generator ID 5 version...

	union {
		bc_u32	iterations;	// Key generator ID 5, allows iteration other then 256, but int does never uses the version field.
							// The version field is used by linux but in case to check does this container uses new suncro.
		bc_u32	version;	// Container version
	};

	char		description[kBCV8_DescriptionLength];	// Description string.
    bc_u64      sparced_position;
	bc_u64		data_start;				// Encrypted body data start.
	bc_u64		data_length;				// Encrypted body data size.
	bc_u32		alg_id;					// The algorithm used for container encryption
	bc_u32		mode_id;				// Encryption mode for container algorithm
	bc_u32		hash_id;				// Hash algorithm used for encryption keys generation
	BCKeySlot	keymap[kBCV8_MaxKeys];

	bc_mode_id getMode() {
		return (bc_mode_id)mode_id;
	}

	//
	//	IsEncrypted
	//		The function allows ambiguity so we have to check if cont 
	//		info is really encrypted after encryption!!!
	//
	bool IsEncrypted();

	//
	//	KG-Ghost v5 allows to change iteration
	//	1. It changes key gen identifier, so old key generator cannot open this container.
	//		Otherwise it will report invalid password.
	//	2. Uses the version field as iteration count. But keyGenId DWORD files uses as WORD wKeyGenId, WORD wVesrion.
	//	3. Key generator ID - 5 does not allow to encrypt header, otherwise enumeration becames imposible.
	//
	//	See http://nsk.jetico.com/redmine/projects/best-crypt-for-windo/wiki/Wiki
	//
	bool setKeyGenIdV5( bc_u32 count );
	bc_u32 getIterationCount();
	void setVersion( bc_u16 ver );
	int getVersion();

	bool useNewSyncro();

} db_header;


/**
 * Enclosing structure for header encryption and verification.
 */
typedef struct db_enc_block32 {
	db_header	header;
	//bc_u8		padding[(sizeof(db_header) + kBCV8_BlockSize - 1) & ~(kBCV8_BlockSize - 1)]; // Padding data to alight header up to 16 byte boundary.	
	bc_u8		padding[4];
	
	// ---------------------------------------------------------------------------------------------------------------
	// Data before this point can be encrypted and should be a multiple of kBCV8_BlockSize bytes in size.
	
	bc_u8		digest[kBCV8_DigestSize32];		// header digest.
	bc_u8		iv[kBCV8_IVSize];			// random initial vector for header encryption.
} db_enc_block32;

typedef struct db_enc_block64 {
	db_header	header;
	//bc_u8		padding[(sizeof(db_header) + kBCV8_BlockSize - 1) & ~(kBCV8_BlockSize - 1)]; // Padding data to alight header up to 16 byte boundary.	
	bc_u8		padding[4];
	
	// ---------------------------------------------------------------------------------------------------------------
	// Data before this point can be encrypted and should be a multiple of kBCV8_BlockSize bytes in size.
	
	bc_u8		digest[kBCV8_DigestSize64];		// header digest.
	bc_u8		iv[kBCV8_IVSize];			// random initial vector for header encryption.
} db_enc_block64;

#pragma pack(pop)

#endif
