/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 * Implementation uses a thin wrapper over certificate lib hash algorithms.
 */

#include "hash.h"

#include "../../certificate/CertificateLib/hashAlgs/cHashAlg.h"
#include "../../certificate/CertificateLib/hashAlgs/hashAlgs.h"
#include "../../certificate/CertificateLib/hashAlgs/SHA1/Sha1.h"
#include "../../certificate/CertificateLib/hashAlgs/SHA256/sha256.h"
#include "../../certificate/CertificateLib/x509pkts/pkcs12_funcs.h"
#include "../../certificate/CertificateLib/PGPPkts/pktdefs.h"

#include <assert.h>
#include <time.h>


static cHashAlg* get_hash(int id)
{
	cHashAlg* palg;
	if(hae_invalidAlgorithmID == ha_createAlgorithm(&palg, id)) {
		assert(0);
	}

	return palg;
}

template < class T > bool setFirst( T *tab, int id )
{
	for( unsigned i = 1; i < tab->total(); i++ )
	{
		if ( tab->get(i) == id )
		{
			tab->set( i, tab->get( 0 ) );
			tab->set( 0, id );
			return true;
		}
	}

	return false;
}

namespace kgghost {

	void calculate_digest(int hash_id, const void* message, size_t nbytes, bc_u8* out_digest)
	{
		cHashAlg* alg = get_hash(hash_id);
		if(!alg) {
			assert(0);
		}

		alg->makeDigest((BYTE*)out_digest, (BYTE*)message, nbytes);
        ha_free(alg);
	}


	void shred(void* data, size_t nbytes)
	{
		unsigned i;
		bc_u8* buf = (bc_u8*) data;

		if ((!data) || (nbytes <= 0)) return;
		srand( (unsigned int)time( NULL ) );
		for(i=0; i < nbytes; i++)
		{
			buf[i] = (BYTE)rand();
		}

		SHA256_RandomGenerate((BYTE*)data, nbytes, (BYTE*)data, nbytes);
	}

	void generate_key(int hash_id, const bc_u8* seed, size_t seedlen, bc_u8* out_key, size_t keylen)
	{
		int res = P12_producePseudoRandomBytes((BYTE*)out_key, keylen, NULL, 0, 1, 
						       (BYTE*)seed, seedlen, 
						       P12_prgm4key, (BYTE)hash_id);
		assert(res == x509e_ok);
	}

	BCEnumHashes::~BCEnumHashes()
	{
		if ( bf != NULL )
			delete bf;
	}

	BCEnumHashes::BCEnumHashes( bool bV8Only, int defId )
	{
		bf = NULL;

		int *T = NULL;
		int Q = 0;

		if ( bV8Only )
		{
			static int stT[] = {
				pgphaMD5,
				pgphaSHA1,
				pgphaRIPEMD160,
				pgphaSHA256,
			};

			T = stT;
			Q = maxLen( stT );
		}
		else
			ha_getIDsOfAvailableAlgAlgorithms( &Q, &T );

		kTotalHashes = 0;
		bf = new int [ Q ];

		if ( bf != NULL )
		{
			hashes = bf;
			for( ; kTotalHashes < (unsigned)Q; kTotalHashes++ )
				hashes[ kTotalHashes ] = T[ kTotalHashes ];

			if ( defId != -1 )
				setFirst( this, defId );
		}
	}

	BCEnumIterations::BCEnumIterations()
	{
		iterationT[0] = 16 * 1024;
		iterationT[1] = 256;
	}
}
