#ifndef _WIN32
#include <dirent.h>
#endif

//#include "PKE_block.h"
//#include "kg_errors.h"
//#include "alg_utils.h"
#include "datablock.h"

#include "../../certificate/CertificateLib/publicKeyAlgs/publicKeyAlgs.h"
#include "../../certificate/CertificateLib/SMemory/cPassword.h"
#include "../../certificate/CertificateLib/x509pkts/pkcs12_funcs.h"
#include "../../certificate/CertificateLib/PGPPkts/Packets.h"
#include "../../certificate/CertificateLib/RNG/cRNG.h"
#include <list>

#include "../../container/file.h"
#include "pke.h"

//#include <common/types.h>
//#include <common/file.h>

using namespace kgghost;
//using namespace bdk;

//class cSecretKeyStorage
//{
//	HPKA m_hSka;
//
//public:
//
//	cSecretKey()
//	{
//		m_hSka = NULL;
//	}
//	
//	~cSecretKey()
//	{
//		freeKey();
//	}
//
//	void freeKey()
//	{
//		if ( NULL == m_hSka )
//			return;
//
//		ska_free( m_hSka );
//
//		m_hSka = NULL;
//	}
//
//	void setKey( HPKA hSka )
//	{
//		freeKey();
//		m_hSka = hSka;
//	}
//
//	HPKA getKey()
//	{
//		return m_hSka;
//	}
//};
//
//static cSecretKeyStorage SKS;

#if 0
// pke key file database level
namespace kgghost
{

	/**
	 * @brief	PKE key file entry type
	 */
	enum pkedb_entry_type
	{
		ETYPE_PUBLIC, /// < Public key file (certificate)
		ETYPE_SECRET, /// < Secret key file (p12 packet)
	};


	/**
	 * @brief	Key file database entry
	 */
	struct pkedb_entry
	{
		/* type of the key file */
		pkedb_entry_type type;

		/* name of the key file */
		std::string fname;


		/**
		 * @brief	Dump key file data in external buffer
		 */
		void load_entry_data(cPTB& ptb);
	};


	/**
	 * @brief	PKE key file database index
	 */
	struct pkedb_index
	{
		typedef std::list<pkedb_entry> index_list;
		typedef index_list::iterator index_iter;

		/* list of secret key entries */
		index_list list_secret;

		/* list of public key entries */
		index_list list_public;


		pkedb_index()
		{
			reset();
		}


		~pkedb_index()
		{
			reset();
		}


		/**
		 * @brief	Add new key file entry
		 */
		int add_entry(const char* name, unsigned int namelen);


		/**
		 * @brief	Reset index status
		 */
		void reset()
		{
			list_secret.clear();
			list_public.clear();
		}
	};


} // pke key file database level


void pkedb_entry::load_entry_data(cPTB& ptb)
{
#ifdef _WIN32
	//BCFile file(fname, BC_FILE_READ);

	bc_file_handle handle;
	bc_file_result res = BCFileOpen( fname.c_str(), kBCFileMode_Read, &handle );

	if ( kBCFileResult_Success != res )
	{
		assert( false );
		return;
	}

	bc_u8 buf[0xff];
	bc_u64 nread = 0;

	ptb.ClearBuffer();
	do
	{
		//nread = file.read(buf, sizeof(buf), sizeof(*buf));
		nread = BCFileRead( &handle, (void*)buf, sizeof(buf) * sizeof(*buf)  );

		bc_u32 size = u64tou32( nread, sizeof( buf ) );

		ptb.WriteBytes((const BYTE*)buf, size);

	} while (nread == sizeof(buf));

	BCFileClose( &handle );
#else
	// Channel implementation is required.
#endif
}


int pkedb_index::add_entry(const char* name, unsigned int namelen)
{
#ifdef _WIN32
	const char* ext = name + namelen - 3; // file extention pointer

	pkedb_entry_type type;
	if(strncmp(ext, "p12", 3) == 0)
		type = ETYPE_SECRET;
	else if(strncmp(ext, "cer", 3) == 0)
		type = ETYPE_PUBLIC;
	else
		return kKeyBlock_IncorrectFormatVersion;

	pkedb_entry new_entry;
	new_entry.type = type;
	new_entry.fname.assign( BCDBGetPath() );
	new_entry.fname.append( "//", 1 );
	new_entry.fname.append(name, namelen);

	if(type == ETYPE_SECRET)
		list_secret.push_back(new_entry);
	else
		list_public.push_back(new_entry);

	return kKeyBlock_errorNo;
#else
	return kKeyBlock_InternalProblem;
#endif

}

// read all files in db and store their name in index
static int pkedb_index_all(pkedb_index& index)
{
#ifdef _WIN32
	//DIR* db = opendir("/home/inso/.bcdb");
	bc_dir_handle db = BCDirOpen( BCDBGetPath() );
	
	if( NULL == db )
		return kKeyBlock_InternalProblem;

	bc_dirent dir;

	int res;
	//while( (dir = readdir(db)) != NULL )
	while( BCDirRead( &dir, db ) )
	{
//		if(dir->d_type != DT_REG)
		if(!BCDirIsRegularFile( &dir ) )
			continue;

		res = index.add_entry(dir.d_name, strlen(dir.d_name));

		if( kKeyBlock_IncorrectFormatVersion == res )
			continue; // skip junkfiles
		else if ( kKeyBlock_errorNo != res )
			return res;
	}

	//closedir(db);
	BCDirClose( db );

	return kKeyBlock_errorNo;
#else
	return kKeyBlock_InternalProblem;
#endif
}

#endif // #if 0

//
// 	key parsing by addressing files in pkedb
//	deals with key formats
//
namespace kgghost
{
	// pgp key id buffer size
	static const unsigned int PGPKeyIdSize = 8;

	// local key id buffer size
	static const unsigned int LocalKeyIdSize = 20;

	/**
	 * @brief	Fixed size buffer for x509 key id
	 */
	struct x509_key_id
	{
		bc_u8 data[LocalKeyIdSize];
	};


	/**
	 * @brief	Fixed size buffer for PGP key id
	 */
	struct PGP_key_id
	{
		bc_u8 data[PGPKeyIdSize];
	};
}


/**
 * @brief
 *	Parse x509 loaded secret key file, verify it's correctness and return it's local key id.
 *
 * @param key_file_data
 *	Loaded key file buffer
 *
 * @param out_id
 *	Local key id to initialize
 *
 * @return
 *	x509e_ok if parsing was successful, another x509e code on error
 */

static int parse_x509_id_secret( const bc_u8 *ptr, unsigned int size, x509_key_id& out_id, cUniversalString *us = NULL, cPassword *pwd = NULL, cPTB *ptbSKOut = NULL)
{
	cPTB ptb;
	cPTB AttrBuf;
	P12_attrSet attrSet;
	x509_struct localKeyId;

	int res = P12_PFX_PDU_parseEx( ptr, size, &AttrBuf, pwd, &ptb );
	if ( ( res != x509e_ok ) && ( res != x509e_emptyParameters ) && ( res != ppe_invalidPassword  ) )
		return res;
	
	if ( ptbSKOut != NULL )
	{
		if ( res != x509e_ok )
			return res;

		ptb.Swap( ptbSKOut );
	}

	res = attrSet.parse( AttrBuf.buffer, AttrBuf.bufferSize, NULL, NULL, asn1tag_CONSTRUCTED_SET );
	if ( res != x509e_ok )
		return res;

	res = attrSet.getLocalKeyId( &localKeyId );
	if ( res != x509e_ok )
		return res;

	memcpy(out_id.data, localKeyId.B, localKeyId.L);

	if ( us != NULL )
	{
		x509_struct s;
		if ( attrSet.getFriendlyName( &s ) == ppe_ok )
			us->SetUnicode( s.B, s.L );
	}

	return x509e_ok;
}


/**
 * @brief
 *	Parse x509 loaded public key file, verify it's correctness and return it's local key id.
 *
 * @param key_file_data
 *	Loaded key file buffer
 *
 * @param out_id
 *	Local key id to initialize
 *
 * @return
 *	x509e_ok if parsing was successful, another x509e code on error
 */
static int parse_x509_id_public(cPTB& key_file_data, struct x509_key_id& out_id)
{
	int res;
	bc_u8* keyID;
	int keyIDSize = 0;

	x509_certificate crt;

	res = crt.parse( key_file_data.buffer, key_file_data.bufferSize );
	if( res != x509e_ok )
	{
		return res;
	}

	res = crt.tbsCertificate.getSubjectKeyID( &keyID, &keyIDSize );

	if( res == x509e_extensionIsNotPresent )
	{
		cPTB ptb;

		keyIDSize = 0;
		res = crt.tbsCertificate.generateSubjectKeyID( NULL, &keyIDSize );
		if ( res != pkae_bufferSizeTooSmall )
			return res;

		if ( ptb.Allocate( keyIDSize ) == NULL )
			return res;

		res = crt.tbsCertificate.generateSubjectKeyID( ptb.buffer, &keyIDSize );
		if ( res != x509e_ok )
			return res;

		keyID = ptb.buffer;
	}

	memcpy(out_id.data, keyID, LocalKeyIdSize);
	return res;
}


static int parse_x509_id( cPTB& key_data, struct x509_key_id& out_id, bool bSecret )
{
	if ( bSecret )
		return parse_x509_id_secret(key_data.buffer, key_data.bufferSize, out_id );
	else
		return parse_x509_id_public(key_data, out_id);
}

static int parse_pgp_id( cPTB& key_data, const bc_u8 *out_id, bool bSecret )
{
	return ppe_keyNotFound;

	//if ( res != x509e_ok ) 
	//	return res;

	//if ( memcmp( pgp_key_id.data, id_buffer, PGPKeyIdSize ) != 0 )
	//	return ppe_keyNotFound;

	//return x509e_ok;
}

/**
 * @brief
 *	Parse pgp loaded secret key file, verify it's correctness and return it's local key id.
 *
 * @param key_file_data
 *	Loaded key file buffer
 *
 * @param out_id
 *	Local key id to initialize
 *
 * @return
 *	x509e_ok if parsing was successful, another x509e code on error
 */
//static int parse_pgp_id_secret(cPTB& key_file_data, struct x509_key_id& out_id)
//{
//	return kKeyBlock_IncorrectFormatVersion;
//}


/**
 * @brief
 *	Parse x509 loaded public key file, verify it's correctness and return it's local key id.
 *
 * @param key_file_data
 *	Loaded key file buffer
 *
 * @param out_id
 *	Local key id to initialize
 *
 * @return
 *	x509e_ok if parsing was successful, another x509e code on error
 */
//static int parse_pgp_id_public(cPTB& key_file_data, struct x509_key_id& out_id)
//{
//	return kKeyBlock_IncorrectFormatVersion;
//}


//static int parse_pgp_id(cPTB& key_data, struct x509_key_id& out_id, pkedb_entry_type type)
//{
//	if( type == ETYPE_PUBLIC )
//		return parse_pgp_id_public(key_data, out_id);
//	else
//		return parse_pgp_id_secret(key_data, out_id);
//}


//static bool get_x509_key_by_id( cPTB& key_buffer, const x509_key_id& key_id, bool secret)
//{
//	// load pkedb index
//	pkedb_index index;
//	if( kKeyBlock_errorNo != pkedb_index_all( index, channelT, channelQ ))
//		return false;
//
//	key_buffer.ClearBuffer();
//
//	cPTB file_data;
//	struct x509_key_id local_key_id;
//
//	pkedb_index::index_iter entry, end;
//
//	if( secret )
//	{
//		entry = index.list_secret.begin();
//		end = index.list_secret.end();
//	}
//	else
//	{
//		entry = index.list_public.begin();
//		end = index.list_public.end();
//	}
//
//	int res;
//	for(; entry != end; ++entry)
//	{
//		file_data.SeekToBegin();
//
//		// load file data
//		entry->load_entry_data(file_data);
//
//		// parse x509 key data by entry type
//		res = parse_x509_id(file_data, local_key_id, entry->type);
//		if( res != x509e_ok )
//			continue;
//
//		// compare parsed id
//		if ( memcmp(local_key_id.data, key_id.data, LocalKeyIdSize) != 0 )
//			continue;
//
//		// id is correct, store key data and exit
//		key_buffer.WriteBytes(file_data.buffer, file_data.bufferSize);
//		return true;
//	}
//
//	return false;
//}


//static bool get_pgp_key_by_id( cPTB& key_buffer, const x509_key_id& key_id, const iChannel** channelT, int channelQ, bool secret)
//{
//	return false;
//}


static bool get_key_by_id(
		cPTB& out_key_buffer, type_kff& out_kff,
		const bc_u8* id_buffer, unsigned int id_size,
		iChannel** const channelT, int channelQ,
		bool secret )
{
	if ( id_size < PGPKeyIdSize )
		return false;

	out_key_buffer.ClearBuffer();

	cPTB ptbKey;

	int res;
	for( int i = 0; i < channelQ; i++ )
	{
		ptbKey.SeekToBegin();

		// load file data
		if ( !channelT[i]->readToBlock( ptbKey ) )
			continue;

		// Tries to find key with with this id at PGP key ring
		if ( id_size == PGPKeyIdSize )
		{
			if ( parse_pgp_id( ptbKey, id_buffer, secret ) == x509e_ok )
			{
				// id is correct, store key data and exit
				out_kff = kffPGP;
				out_key_buffer.WriteBytes(ptbKey.buffer, ptbKey.bufferSize);
				return true;
			}
		}

		// x509 local key id 
		if ( id_size != LocalKeyIdSize )
			continue;

		// parse as x509 key
		x509_key_id local_key_id;

		res = parse_x509_id( ptbKey, local_key_id, secret );

		if( ( res == x509e_ok ) &&
			( memcmp(local_key_id.data, id_buffer, LocalKeyIdSize ) == 0 ) )
		{
			// id is correct, store key data and exit
			out_kff = kffX509;
			out_key_buffer.WriteBytes(ptbKey.buffer, ptbKey.bufferSize);
			return true;
		}
	}

	return false;
}

//static bool get_key_by_id(
//		cPTB& out_key_buffer, type_kff& out_kff,
//		const bc_u8* id_buffer, unsigned int id_size,
//		const iChannel** channelT,
//		int channelQ,
//		bool secret )
//{
//	int res;
//
//	// try pgp key
//	if(id_size >= PGPKeyIdSize)
//	{
//		res = get_pgp_key_by_id( out_key_buffer, *((x509_key_id*)(id_buffer) ), channelT, channelQ, secret );
//		if( res )
//		{
//			out_kff = kffX509;
//			return true;
//		}
//	}
//
//	// try x509 key
//	if(id_size >= LocalKeyIdSize)
//	{
//		bool ok = get_x509_key_by_id(out_key_buffer, *((x509_key_id*)(id_buffer + PGPKeyIdSize) ), channelT, channelQ, secret);
//		if( ok )
//		{
//			out_kff = kffX509;
//			return true;
//		}
//	}
//
//	return false;
//}

static bool ParseP12BuffAndGetSecretKey( HPKA* pSka, BYTE *buff, int buffSize, cPassword *pwd, cUniversalString *us = NULL )
{
	cPTB ptbKey;
	x509_key_id lkid;

	int res = parse_x509_id_secret( buff, buffSize, lkid, us, pwd, &ptbKey  );

	x509_privateKeyInfo pki;
	if ( pki.parse( ptbKey.buffer, ptbKey.bufferSize ) != x509e_ok )
		return false;

	if ( ska_createAlgorithm_X509( pSka, &pki ) != x509e_ok )
		return false;

	return true;
}


bool ParseSecretKey( BYTE *buff, int buffSize, type_kff kff, cPassword *pwd, HPKA* pSka )
{
	if ( kff == kffX509 )
	{
		return ParseP12BuffAndGetSecretKey( pSka, buff, buffSize, pwd );
	}
	else
	if ( kff == kffPGP )
	{
		int res = PKT_CreateSecretKeyAlgFromPacket( pSka, ( sSecretKey * )buff, buffSize, pwd->buffer );

		return res == pkae_ok;
	}
	else
		return false;
}

const int lkid_size = 20; // size of SHA1 digest.

//
// table contains pair
//	[thumbprint, lkid]
//	...
//

static const bc_u8 *findThumbprintByLKID( const bc_u8 *bf, unsigned int bfSize, const bc_u8* lkid )
{
	for( unsigned int i = 0; i*2*lkid_size < bfSize; i+= 2*lkid_size )
	{
		if ( memcmp( bf + i * 2 * lkid_size + lkid_size, lkid, lkid_size ) == 0 )
			return bf + i * 2 * lkid_size;
	}

	return NULL;
}

static bool addSecretKeys( cPTB &thumbprint2lkidTable, const BCPassword *pwd_in )
{
	if ( pwd_in->m_secretKeyQ == 0 )
		return false;

	cPassword pwd( pwd_in->buf );

	bool bAdded = false;

	for( int i = 0; i < pwd_in->m_secretKeyQ; i++ )
	{
		cPTB pdu;
		if ( !pwd_in->m_secretKeyT[i]->readToBlock( pdu ) )
			continue;

		cPTB ptbSK;
		kgghost::x509_key_id thumbprint;
		int res = parse_x509_id_secret( pdu.buffer, pdu.bufferSize, thumbprint, NULL, &pwd, &ptbSK );
		if ( res != x509e_ok )
			continue;

		x509_privateKeyInfo pki;
		if ( pki.parse( ptbSK.buffer, ptbSK.bufferSize ) != x509e_ok )
			continue;

		HPKA ska;
		if ( ska_createAlgorithm_X509( &ska, &pki ) != x509e_ok )
			continue;

		kgghost::x509_key_id lkid;
		int size = sizeof( lkid.data );

		if ( ska_generateLocalKeyID( ska, lkid.data, &size ) == sae_ok )
		{
			thumbprint2lkidTable.WriteBytes( thumbprint.data, lkid_size );
			thumbprint2lkidTable.WriteBytes( lkid.data, lkid_size );
			bAdded = true;
		}

		ska_free( ska );
	}

	return bAdded;
}

//
//	getThumbprintByLocalKeyId
//	  Secret key PFX_PDU has thumbprint instead of local key id.
//	   So this function:
//		Parses secret keys with password and adds theirs thumbprint and lkid pairs to thumbprint2lkidTable
//		Finds thumbprint at thumbprint2lkidTable.
//		Returns thumbprint pointer  or NULL.
//
static const bc_u8* getThumbprintByLocalKeyId( const BCPassword *pwd_in, const bc_u8 *lkid )
{
	static cPTB ptbTable; // Static table with [thumbprint,lkid] pairs.

	// Defends from unlimited growing.
	if ( ptbTable.bufferSize >= 1024 * 1024 )
		ptbTable.Free();

	// Checks in existing table first
	const bc_u8* thumbprint = findThumbprintByLKID( ptbTable.buffer, ptbTable.bufferSize, lkid );
	if ( thumbprint != NULL )
		return thumbprint;

	if ( !addSecretKeys( ptbTable, pwd_in ) )
		return NULL;

	// Checks in updated table
	thumbprint = findThumbprintByLKID( ptbTable.buffer, ptbTable.bufferSize, lkid );
	
	return thumbprint;
}

void pke_set_t::reset()
{
	if ( pka != NULL )
		pka_free( pka );

	if ( ska != NULL )
		ska_free( ska );

	ska = pka = NULL;
	kff = kffNone;
}

int pke_set_t::produce_from_id(const bc_u8* idBuf, unsigned int idSize, const BCPassword *pwd_in )
{
	reset();

	cPTB ptbSK;

	if ( idSize < PGPKeyIdSize )
		return kKeyBlock_IncorrectDataBlockSize;

	// get secret key buffer by key id
	if ( !get_key_by_id( ptbSK, kff, idBuf, PGPKeyIdSize, pwd_in->m_secretKeyT, pwd_in->m_secretKeyQ, true ) )
	{
		// x509 id lays after PGP id.
		idBuf += PGPKeyIdSize;
		idSize -= PGPKeyIdSize;

		if ( idSize < LocalKeyIdSize )
			return kKeyBlock_IncorrectDataBlockSize;

		if ( !get_key_by_id( ptbSK, kff, idBuf, LocalKeyIdSize, pwd_in->m_secretKeyT, pwd_in->m_secretKeyQ, true ) )
		{
			const bc_u8 *thumbprint = getThumbprintByLocalKeyId( pwd_in, idBuf );
			if ( ( thumbprint != NULL ) &&
				  !get_key_by_id( ptbSK, kff, thumbprint, LocalKeyIdSize, pwd_in->m_secretKeyT, pwd_in->m_secretKeyQ, true ) )
					return kKeyBlock_InternalProblem;
		}
	}

	// parse secret key buffer to initialize secret key algorithm
	cPassword pwd( pwd_in->buf );
	if(!ParseSecretKey( ptbSK.buffer, ptbSK.bufferSize, kff, &pwd, &ska))
		return kKeyBlock_InternalProblem;

	// restore public key algorithm from secret key algorithm
	if(ska_derivePublicKeyAlgorithm(&pka, ska) != pkae_ok)
		return kKeyBlock_InternalProblem;

	return kKeyBlock_errorNo;
}


//
//	Keyblock level
//
namespace kgghost
{

	/**
	 * @brief 	Public Key Encrypted key block
	 */
	struct PKE_KeyBlock
	{
		BYTE pgpKeyId[PGPKeyIdSize];		// pgp key ID, if certificate was used as public key the field is filled by random data
		BYTE localKeyId[LocalKeyIdSize];	// certificate public key local key id

		BYTE pkBuff[1];				// public key encrypted key block.
							// Real buffer size is defined by secret key size.
							// Block has been additionally covered to hide MPI info.
							// The stored buffer is pure product of public key encrypt operation.
	};



}


static int getEncBlockSize_Public( HPKA pka, unsigned int *pBlockSize, unsigned int *pMaxSize = NULL )
{
	int blockSize;
	blockSize = pka_getBlockSize( pka );

	//
	// RSA_PublicKeyEncryptionBlock_Format generates error
	// if dataSize > outBufSize - 10
	//
	// if ( keyLength > blockSize - 10 )
	//	return -1;
	//
	if ( pMaxSize )
		*pMaxSize = blockSize - 10;

	int encBuffSize = 0;
	int res = pka_encrypt( pka, NULL, &encBuffSize, NULL, blockSize );

	if ( res != pkae_bufferSizeTooSmall )
		return -1;

	if ( pBlockSize )
		*pBlockSize = blockSize;

	return encBuffSize;
}


//
//	The encrypted public key buffer is really MPI buffer.
//	The first MPI bytes with size are visible. So we have to hide them in key block.
//	The idea is : the MPI buffer will be xorred with local key ID generated by MD5 instead of SHA1
//
static bool coverBuffer( HPKA pka, BYTE *buff, int buffSize )
{
	// Generates MD5 hash instaed of usual SHA1
	int hashSize = 0;
	int res = pka_generateLocalKeyID( pka, NULL, &hashSize, pgphaMD5 );

	if ( res != x509e_bufferTooSmall )
		return false;

	cPTB ptb;
	if ( ptb.Allocate( hashSize ) == NULL )
		return false;

	res = pka_generateLocalKeyID( pka, ptb.buffer, &hashSize, pgphaMD5 );

	if ( res != pkae_ok )
		return false;

	BYTE *cover = ptb.buffer;
	int coverSize = hashSize;

	for( int i = 0; i < buffSize; i++ )
		buff[i] ^= (BYTE)i ^ cover[ i % coverSize ];

	return true;
}

namespace kgghost
{
	//
	//	pke_encode
    //		encodes key into key block by public key packet.
	//
	//		1. Checks encrypted and plaintext buffer sizes
	//		2. Prepare RSA formated encrypted block
	//		3. Fills key ID buffers
	//		4. Encrypt the block by public key
	//		5. Cover the encrypted MPI by local key ID
	//
	//	Note, the encrypted information may be splitted to several parts,
	//	because maxExcrypted size may be less then size of encrypted key info
	//
	//	Returns:
	//		0 if success
	//		1 if kbSize is to small to encode key.
	//		otherwise pke_ .. error. 
	//
    unsigned pke_encode( const key_data_t* key, const bc_u8* pkPkt, unsigned pkPktSize, bc_u8* kb_data, unsigned &kbsize ) //, HPKA ska  )
	{
		x509_certificate cer;
		int res = cer.parse( (BYTE*)pkPkt, pkPktSize );
		if ( res != pkae_ok ) return res;

		pke_set_t set;
		res = pka_createAlgorithm_X509( &set.pka, &cer.tbsCertificate.subjectPKInfo );
		if ( res != pkae_ok ) return res;

		unsigned int blockSize;
		unsigned int splitSize;

		int size = getEncBlockSize_Public( set.pka, &blockSize, &splitSize );

		if ( size == -1 )
			return x509e_undefinedAlgorithm;

		int blockQ = ( sizeof( key_data_t )+ splitSize - 1 ) / splitSize;

		if ( kbsize < PGPKeyIdSize + LocalKeyIdSize + size * blockQ  )
		{
			kbsize = PGPKeyIdSize + LocalKeyIdSize + size * blockQ;
			return x509e_bufferTooSmall;
		}

		PKE_KeyBlock *kb = (PKE_KeyBlock*)kb_data;

		int keyIDSize = LocalKeyIdSize;

		// The PGP part has to be randomized early.
		res = pka_generateLocalKeyID( set.pka, kb->localKeyId, &keyIDSize );
		if ( res != pkae_ok ) return res;

		bc_u8 *encPos = kb->pkBuff;
		int encSize = size;
		const bc_u8 *keyBuff = (const bc_u8 *)key;
		int filledSize = 0;

		for( int blockN = 0; blockN < blockQ; blockN++ )
		{
			cPTB ptb;
			if ( ptb.Allocate( blockSize ) == NULL )
				return false;

			const bc_u8 *curPos = keyBuff + splitSize * blockN;

			unsigned int curSize = sizeof( key_data_t ) - splitSize * blockN;
			if ( curSize > splitSize )
				curSize = splitSize;

			res = RSA_PublicKeyEncryptionBlock_Format( true, ptb.buffer, blockSize, curPos, curSize  );
			if ( res != pkae_ok ) return res;

			res = pka_encrypt( set.pka, encPos + blockN * encSize, &encSize, ptb.buffer, blockSize );
			if ( res != pkae_ok ) return res;

			//// Just for test...
			//if ( ska != NULL )
			//{
			//	int decSize = encSize;
			//	cPTB dec;
			//	dec.Allocate( decSize );
			//	res = ska_decrypt( ska, dec.buffer, &decSize, encPos + blockN * encSize, encSize );
			//}

			filledSize += encSize;
		}

		assert( kbsize == filledSize + PGPKeyIdSize + LocalKeyIdSize );

		if ( !coverBuffer( set.pka, encPos, filledSize ) )
			return x509e_undefinedAlgorithm;

    	return x509e_ok;
	}

	//
	//	decodeKey
	//		1. Checks buffers sizes
	//		2. Discover the covered MPI buffer,
	//		3. Decrypt MPI buffer by secret key
	//		4. Parse formatted RSA block
	//		5. Copy data from fromatted block to key buffer
	//
	// Decode key from key block using password
	unsigned pke_decode(const bc_u8* kbEncoded, unsigned kbSize, const BCPassword* pwd_in, CipherBase* cipher, int hash, key_data_t* out_key)
	{
		LIBLOG_INFO("kbSize %d", kbSize );

		int res;
		pke_set_t set;

		res = set.produce_from_id( kbEncoded, kbSize, pwd_in );
		if( res != kKeyBlock_errorNo )
			return false; // returns ERROR_INVALID_ALGORITHM here if password was incorrect

		HPKA pka = set.pka;
		HPKA ska = set.ska;

		unsigned int blockSize;
		PKE_KeyBlock* kb = (PKE_KeyBlock*)kbEncoded;

		unsigned int encSize = getEncBlockSize_Public( pka, &blockSize );

		if ( encSize == 0 )
			return false;

		unsigned int size = encSize + PGPKeyIdSize + LocalKeyIdSize;
		if ( size > kbSize )
			return false;

		BYTE *encPos = kb->pkBuff;
		unsigned int encTailsize = size - (PGPKeyIdSize + LocalKeyIdSize);

		cPTB ptb;
		if ( ptb.WriteBytes( encPos, encTailsize ) == 0 )
			return false;

		if ( !coverBuffer( pka, ptb.buffer, ptb.bufferSize ) )
			return false;

		// The encrypted key size may be more then maximum allowed for the algorithm
		// so plain text key may be splited to several parts.
		// We have to decrypt and concatenate all the parts.

		int pos = 0;
		cPTB ptbKey;

		do
		{
			cPTB ptbDec;
			if ( ptbDec.Allocate( blockSize ) == NULL )
				return false;

			if ( ska_decrypt( ska, ptbDec.buffer, (int*)&size, ptb.buffer + pos, encSize ) != pkae_ok )
				return false;

			const BYTE *dataPos;
			int dataSize;

			if ( RSA_PublicKeyEncryptionBlock_Parse( true, ptbDec.buffer, blockSize, &dataPos, &dataSize ) != pkae_ok )
				return false;

			if ( ptbKey.WriteBytes( (BYTE*)dataPos, dataSize ) != dataSize )
				return false;

			if ( ptbKey.bufferSize > sizeof(key_data_t) )
				return false;

			key_data_t *keyInfo = ( key_data_t * )ptbKey.buffer;

			if ( keyInfo->size <= ptbKey.bufferSize )
				break;

			// go to the next encrypted buffer.
			pos += encSize;
		}
		while( true );

		memcpy((BYTE*)out_key, ptbKey.buffer, sizeof( key_data_t ) ); //ptbKey.bufferSize);
		return true;
	}

    bool pke_verify(const bc_u8* kb_data, unsigned kb_size, const BCPassword* pwd, CipherBase* cipher, int hash)
    {
		// public key verification is not possible because corresponded secret key usually does not exist on local computer.
	    return false;
    }

} // namespace kgghost

/*
unsigned int CPKEProcessor::encoded_key_size( unsigned int keyLength )
{
	if ( pka == NULL )
		return -1;

	int maxDecodedSize;
	int encSize = getEncBlockSize_Public( pka, NULL, &maxDecodedSize );

	if ( encSize == -1 )
		return -1;

	int blockQ = ( keyLength + maxDecodedSize - 1 ) / maxDecodedSize;

	return blockQ * encSize + pgp_keyIDSize + localKeyIDSize;
	return 0;
}

bool CPKEProcessor::encode(bc_u8* kbPlain, unsigned int & kbSize, key_info_t const & key, cPassword const & init_pwd)
{
	return false; // stub
}


CPKEProcessor::~CPKEProcessor()
{
}
*/




namespace kgghost
{
}




//
//
//	REFACTOR
//
//



#if 0
	static int _encrypt( HPKA pka, bc_u8 *encPos, int *pEncSize, bc_u8 *buffer, int blockSize )
	{
		int res = pka_encrypt( pka, encPos, pEncSize, buffer, blockSize );

		if ( res == pkae_randomBufferRequired )
		{
			cPTB pool;
			if ( pool.Allocate( blockSize ) == NULL )
				return false;

			ShredData( pool.buffer, pool.bufferSize );

			cRNG::setPool( pool.buffer, pool.bufferSize );

			res = pka_encrypt( pka, encPos, pEncSize, buffer, blockSize );

			cRNG::clearPool();
		}

		return res;
	}
static int getKeyIDTbl( cPTB *ptbTbl, BYTE *keyPkt, int keyPktSize, bool secret = false )
{
	int size = 0;

	int res = pgp_EnumKeyIDs( NULL, &size, keyPkt, keyPktSize, secret );

	if ( res == ppe_bufferTooSmall )
	{
		if ( ptbTbl->Allocate( size ) == NULL )
			return ppe_memoryAllocation;

		res = pgp_EnumKeyIDs( ptbTbl->buffer, &size, keyPkt, keyPktSize, secret );
	}

	return res;
}


static wchar_t *getAlgInfo( wchar_t *wStr, int wStrMaxLen, HPKA ska )
{
	BYTE lkId[ localKeyIDSize ];

	int size = sizeof( lkId );

	ska_generateLocalKeyID( ska, lkId, &size );

	wchar_t strID[ 256 ];
	bytes2string( strID, maxLen( strID ), lkId, sizeof( lkId ) );

	FormatT(
		wStr, wStrMaxLen,
		LoadStringOnceT( hMainInst, IDS_szS_D_BitsID_S ),
		getWCStr( PKT_GetPublicKeyAlgName( ska_getAlgID( ska ) ) ),
		ska_getModuleSize( ska ) * 8,
		strID );

	return wStr;
}


wchar_t curName[ 1024 ];

enum { arOK, arContinue, arError };

static int askRes( HWND hWnd, HPKA *pSKA, HPKA ska, const wchar_t *newName, bool silent )
{
	// If pSKA is NULL simple report that key with the password is found
	if ( pSKA == NULL )
	{
		wcsncpy( curName, newName, maxLen( curName ) );
		return arOK;
	}

	// First secret key found
	if ( *pSKA == NULL )
	{
		wcsncpy( curName, newName, maxLen( curName ) );
		*pSKA = ska;
		return arContinue;
	}

	// Another secret key found

	// We cannot choose key in silent mode, so report errror.
	if ( silent )
		return arError;

	wchar_t text[ 2048 ];
	wchar_t strCur[ 256 ];
	wchar_t strNew[ 256 ];

	FormatT_S( text, maxLen( text ),
		hMainInst, IDS_OtherSKSamePasswordFound_SSSS_Q,
		curName, getAlgInfo( strCur, maxLen( strCur ), *pSKA ),
		newName, getAlgInfo( strNew, maxLen( strNew ), ska )
	);

	int res = MessageBoxT( hWnd, text, LoadStringOnceT( hMainInst, IDS_KeyGen ), MB_ICONQUESTION | MB_YESNOCANCEL );

	if ( res == IDYES )
	{
		ska_free( ska );
		ska = NULL;
		return arOK;
	}
	else
	if ( res == IDNO )
	{
		ska_free( *pSKA );
		*pSKA = ska;
		wcsncpy( curName, newName, maxLen( curName ) );
		return arContinue;
	}
	else
	{
		ska_free( *pSKA );
		*pSKA = NULL;
		ska_free( ska );
		ska = NULL;
		return arError;
	}
}

//
//		Walks all p12 file and implement password
//		if pSKA is NULL simple return true it means that key with same password exists in BCDB
//		if pSKA is not NULL walks all keys and report if another key has same password
//			allows to choose the new and continue searching
//			or use the previose and stop.
//
static bool EnumP12FileAndGetSecretKey( HWND hWnd, wchar_t *folder, cPassword *pPwd, HPKA *pSKA, bool silent )
{
	if ( ( pSKA != NULL ) && ( *pSKA != NULL ) )
	{
		ska_free( *pSKA );
		*pSKA = NULL;
	}

	wchar_t path[ MAX_PATH ];
	wcscpy( path, folder );
	int len = (int)wcslen( path );
	if ( len == 0 )
		return false;
	if ( path[ len - 1 ] != L'\\' )
	{
		path[ len++ ] = L'\\';
		path[ len ] = L'\0';
	}

	wcscpy( path + len, L"*.p12" );

	WIN32_FIND_DATAW findFileData;

	HANDLE hFind = FindFirstFileT( path, &findFileData );

	if ( hFind == INVALID_HANDLE_VALUE )
		return false;

	cUniversalString friendlyNameNew;

	bool found = false;

	do
	{
		wcscpy( path + len, findFileData.cFileName );
		cFileView fv( path );

		if ( fv.m_err )
			continue;

		HPKA ska = NULL;

		if ( ParseP12BuffAndGetSecretKey( &ska, fv.m_ptr, fv.m_size, pPwd, &friendlyNameNew ) )
		{
			friendlyNameNew.MakeFromUnicodeString();

			int res = askRes( hWnd, pSKA, ska, friendlyNameNew.wcBuffer, silent );

			if( res == arOK )
			{	found = true; break; }
			else
			if( res == arContinue )
			{	found = true; continue; }
			else // ask error result
			{	found = false;	break;	}

		}
	}
	while( FindNextFileT( hFind, &findFileData ) );

	FindClose( hFind );

	return found; // *pSKA != NULL;
}

static wchar_t *setName( wchar_t *name, int nameSize, char *bf, int bfSize )
{
	int len = ( bfSize >= nameSize ? nameSize - 1 : bfSize );
	_mbstowcs( name, len, bf, -1, false, CP_ACP );
	name[ len ] = '\0';
	return name;
}

static bool EnumPGPKeyRingAndGetSecretKey( HWND hWnd, wchar_t *keyRingFile, cPassword *pPwd, HPKA *pSKA, bool silent )
{
	cFileView fv( keyRingFile );

	if ( fv.m_err )
		return false;

	BYTE *skrBuff = fv.m_ptr;
	int skrBuffSize = fv.m_size;

	cPTB ptbTbl;

	int res = getKeyIDTbl( &ptbTbl, skrBuff, skrBuffSize, true );

	if ( res != pkae_ok )
		return false;

	int Q = ptbTbl.bufferSize / pgp_keyIDSize;

	cPTB ptbSecKey;

	bool found = false;

	for( int i = 0; i < Q; i++ )
	{
		BYTE *keyID = ptbTbl.buffer + i * pgp_keyIDSize;

		sSecretKey *sk;
		int skSize = 0;

		BYTE *userID;
		int userIDSize;

		res = pgp_FindKeyByID( (BYTE **)&sk, &skSize, keyID, skrBuff, skrBuffSize, true, NULL, NULL, &userID, &userIDSize  );

		if ( res != pkae_ok )
			return false;

		HPKA ska = NULL;
		res = PKT_CreateSecretKeyAlgFromPacket( &ska, sk, skSize, pPwd->buffer );

		if ( res != ppe_ok )
			continue;

		wchar_t newName[ 1024 ];
		setName( newName, maxLen( newName ), (char *)userID, userIDSize );

		int res = askRes( hWnd, pSKA, ska, newName, silent );

		if( res == arOK )
		{	found = true; break; }
		else
		if( res == arContinue )
		{	found = true; continue; }
		else // ask error result
		{	found = false;	break;	}
	}

	return found;
}


//
//	FindSecretKeyInBCDB
//		Walks all p12 file and all PGP secret keys in local secret key ring
//		if pSKA is NULL simple return true it means that key with same password exists in BCDB
//		if pSKA is not NULL walks all keys and report if another key has same password
//			Both functions EnumP12FileAndGetSecretKey
//			allows to choose the new and continue searching
//			or use the previose and stop.
//
static int FindSecretKeyInBCDB( HWND hWnd, cPassword *pPwd, HPKA *pSKA, type_kff *pKff )
{
	wchar_t path[ MAX_PATH ];

	if ( !KM_GetDataBasePath( path, maxLen( path ), KM_GetDataBasePath_Flag_Folder ) )
		return false;

	if ( EnumP12FileAndGetSecretKey( hWnd, path, pPwd, pSKA, false ) )
	{
		if ( pKff != NULL ) *pKff = kffX509;
		return true;
	}
	else
	if ( !KM_GetDataBasePath( path, maxLen( path ), KM_GetDataBasePath_Flag_PGPSecretKeyRing ) )
		return false;

	// path is secret key ring file in BCDB.

	if ( EnumPGPKeyRingAndGetSecretKey( hWnd, path, pPwd, pSKA, false ) )
	{
		if ( pKff != NULL ) *pKff = kffPGP;
		return true;
	}

	return false;
}
//
//	openSecretKey
//		if keyIDSBuffer is NULL, so it walks all secret key in BCDB and implements the password
//		otherwise it opens secret key with required key ID.
//
bool cParamsPKE::openSecretKey( HWND hWnd, cPassword *pwd, BYTE *keyIDSBuffer, bool checkPwdOnly )
{
	if ( KM_Dll.error )
		return false;

	cPTB ptb;

	if ( keyIDSBuffer != NULL )
	{
		if ( ska != NULL )
			ska_free( ska );

		if ( pka != NULL )
			pka_free( pka );

		ska = pka = NULL;

		if ( KM_GetPKBlockByID( hWnd, keyIDSBuffer, pgp_keyIDSize, true, &ptb, &kff ) )
		{
			if ( !ParseSecretKey( ptb.buffer, ptb.bufferSize, kff, pwd, &ska ) )
			{
				if ( !checkPwdOnly )
					MessageBoxT( hWnd, IH.LoadStringOnceT( IDS_INVALID_PWD ), IH.LoadStringTwiceT( IDS_OpenSecretKey ) , MB_ICONSTOP );

				return false;
			}
		}
		else
		if ( KM_GetPKBlockByID( hWnd, keyIDSBuffer + pgp_keyIDSize, localKeyIDSize, true, &ptb, &kff ) )
		{
			if ( !ParseSecretKey( ptb.buffer, ptb.bufferSize, kff, pwd, &ska ) )
			{
				if ( !checkPwdOnly )
					MessageBoxT( hWnd, IH.LoadStringOnceT( IDS_INVALID_PWD ), IH.LoadStringTwiceT( IDS_OpenSecretKey ) , MB_ICONSTOP );

				return false;
			}
		}
		else
		{
			if ( checkPwdOnly )
				return false;

			const wchar_t *initialSKPath = L""; // = NULL;

			while( true )
			{
				ptb.Free();

				if ( !KM_BrowseForPKBlock( hWnd, initialSKPath, true, &ptb, &kff ) || ( ptb.bufferSize == 0 ) )
					return false;

				if ( ParseSecretKey( ptb.buffer, ptb.bufferSize, kff, pwd, &ska ) )
					break;

				DlgMessageBoxT( hMainInst, hWnd, IDS_INVALID_PWD, MB_ICONSTOP, IH.LoadStringOnceT( IDS_OpenSecretKey ) );
			}
		}
	}
	else
	if ( FindSecretKeyInBCDB( hWnd, pwd, ( checkPwdOnly ? NULL : &ska ), &kff ) )
	{
		if ( checkPwdOnly )
			return true;
	}
	else
	if ( !checkPwdOnly )
	{
		const wchar_t *initialSKPath = L""; // = NULL;

		while( true )
		{
			ptb.Free();

			if ( !KM_BrowseForPKBlock( hWnd, initialSKPath, true, &ptb, &kff ) || ( ptb.bufferSize == 0 ) )
				return false;

			if ( ParseSecretKey( ptb.buffer, ptb.bufferSize, kff, pwd, &ska ) )
				break;

			DlgMessageBoxT( hMainInst, hWnd, IDS_INVALID_PWD, MB_ICONSTOP, IH.LoadStringOnceT( IDS_OpenSecretKey ) );
		}
	}
	else
		return false;

	if ( ska_derivePublicKeyAlgorithm( &pka, ska ) != pkae_ok )
		return false;

	return true;
}
//
//	encodeKey
//		1. Checks encrypted and plaintext buffer sizes
//		2. Prepare RSA formated encrypted block
//		3. Fills key ID buffers
//		4. Encrypt the block by public key
//		5. Cover the encrypted MPI by local key ID
//
//	Note, the encrypted information may be splitted to several parts,
//	because maxExcrypted size may be less then size of encrypted key info
//
bool CPKEProcessor::encode_key(char* kb_data, unsigned int kb_data_size, const char* key_buf, unsigned int key_buf_size)
{
	int blockSize;
	int splitSize;

	if ( pka == NULL )
		return false;

	int size = getEncBlockSize_Public( pka, &blockSize, &splitSize );

	if ( size == -1 )
		return false;

	int blockQ = ( keyBuffSize + splitSize - 1 ) / splitSize;

	if ( pgp_keyIDSize + localKeyIDSize + size * blockQ > kbEncryptedSize )
		return false;

	// Fill key ID buffers
	if ( kff == kffPGP )
	{
		assert( ptbKeyID.bufferSize == pgp_keyIDSize );
		memcpy( kbEncrypted, ptbKeyID.buffer, pgp_keyIDSize );

		int keyIDSize = 0;
		int res = pka_generateLocalKeyID( pka, NULL, &keyIDSize );

		if ( res != x509e_bufferTooSmall )
			return false;

		assert( keyIDSize == localKeyIDSize );

		keyIDSize = localKeyIDSize;

		res = pka_generateLocalKeyID( pka, kbEncrypted + pgp_keyIDSize, &keyIDSize );

		if ( res != pkae_ok )
			return false;
	}
	else // x509
	{
		// The part has to be randomized early.

		if ( localKeyIDSize != ptbKeyID.bufferSize )
		{
			assert( false );
			return false;
		}

		memcpy( kbEncrypted + pgp_keyIDSize, ptbKeyID.buffer, localKeyIDSize );
	}

	int filledSize = pgp_keyIDSize + localKeyIDSize;

	BYTE *encPos = kbEncrypted + pgp_keyIDSize + localKeyIDSize;
	int encSize = size;

	for( int blockN = 0; blockN < blockQ; blockN++ )
	{
		cPTB ptb;
		if ( ptb.Allocate( blockSize ) == NULL )
			return false;

		BYTE *curPos = keyBuff + splitSize * blockN;

		//
		//	Version 1.02.8 - 07.04.2008
		//
		//	int curSize = keyBuffSize % splitSize;
		//
		int curSize = keyBuffSize - splitSize * blockN;
		if ( curSize > splitSize )
			curSize = splitSize;

		if ( RSA_PublicKeyEncryptionBlock_Format( true, ptb.buffer, blockSize, curPos, curSize  ) != pkae_ok )
			return false;

		int res = _encrypt( pka, encPos + blockN * encSize, &encSize, ptb.buffer, blockSize );

		if ( res != pkae_ok )
			return false;

		filledSize += encSize;
	}

	assert( filledSize <= kbEncryptedSize );

	if ( !coverBuffer( pka, encPos, filledSize - ( pgp_keyIDSize + localKeyIDSize ) ) )
		return false;

    	return true;
}
#endif
