/*
 *	Copyright (c) 2012 Jetico, Inc., Finland
 *	All rights reserved.
 *
 *  Public key encryption for block encoding and decoding
 */

#ifndef KGGHOST_PKE_H
#define KGGHOST_PKE_H

#include "keyblock.h"
#include "../../certificate/CertificateLib/x509pkts/x509_structs.h"
//#include "../../certificate/CertificateLib/SMemory/cPassword.h"

namespace kgghost {

    // Encode key into key block by public key packet.
	//	Returns:
	//		0 if success
	//		1 if kbSize is to small to encode key.
	//
    unsigned pke_encode( const key_data_t* key, const bc_u8* pkPkt, unsigned pkPktSize, bc_u8* kb_data, unsigned &kbsize );//, HPKA ska = NULL );
    
    // Decode key from key block using password
    unsigned pke_decode(const bc_u8* kbdata, unsigned kbsize, const BCPassword* pwd,
                        CipherBase* cipher, int hash, key_data_t* out_key);

    // Verify password without decoding the key
    bool pke_verify(const bc_u8* kbdata, unsigned kbsize, const BCPassword* pwd,
                    CipherBase* cipher, int hash);

	enum type_kff
	{
		kffNone	= 0,
		kffX509 = 1,
		kffPGP = 2,
	};

	/**
	 * @brief	A set of relative public key encryption algorithms.
	 * @more	Keeps a pair of public and secret key algorithms derived from common key id.
	 *		Common key ids can be pgp key ids or key manager local key ids.
	 */
	struct pke_set_t
	{
		/* public key alg */
		HPKA 		pka;

		/* secret key alg */
		HPKA 		ska;

		/* key id type algs were generated from */
		type_kff 	kff;


		pke_set_t() : pka(NULL), ska(NULL), kff(kffNone)
		{
		}


		~pke_set_t()
		{
			reset();
		}


		/**
		 * @brief	Set uninitialized invariant.
		 */
		void reset();

		/**
		 * @brief	Produce pke alg set from key id.
		 *
		 * @p idBuf	Id buffer
		 * @p idSize	Id buffer size.
		 * @p pwd	User password that was used to encode key block.
		 *
		 * @return	kKeyBlock_errorNo on success
		 *		ERROR_INVALID_ALGORITHM if id decoding was successful but user password is incorrect.
		 *		kKeyBlock_IncorrectFormatVersion if id decoding failed.
		 *		Other error codes in exceptional cases.
		 */
		int produce_from_id(const bc_u8* idBuf, unsigned int idSize, const BCPassword* pwd);
	};
}

//bool ParseSecretKey( BYTE *buff, int buffSize, kgghost::type_kff kff, cPassword *pwd, HPKA* pSka );


#endif // KGGHOST_PKE_H