/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 *
 * KGSHA keygen library interface.
 */

#ifndef KGSHA_KGSHA_H
#define KGSHA_KGSHA_H

#include "../../container/keygen.h"
#include "hidden_sector.h"
#include "datablock.h"
#include "../../container/iChannel.h"
#include "../../container/error.h"

/**
 * kgsha keygen interface implementation.
 */
class KGSHA_Header : public IBCHeader {
public:

    KGSHA_Header(HiddenSector* hs, DATA_BLOCK* db);

    virtual bool isEncrypted() {
        return false;
    }

    virtual bool isValid();
    virtual int setBusiedName( const char *compName, int len );
    virtual int getBusiedName( char *compName, int len );

    virtual BCContainerResult encrypt(BCPassword & password) {
        return kBCContainer_Unsupported;
    }

    virtual BCContainerResult decrypt(BCPassword & password) {
        return kBCContainer_Unsupported;
    }

    virtual void mainVolumeInfo(struct BCVolumeInfo* info);
    virtual BCContainerResult init(bc_alg_id algid, bc_mode_id modeid, bc_u64 size, const char* description, BCPassword & password);
    //virtual BCContainerResult createHiddenKey(SpaceMap *map, BCPassword* mainPassword, BCPassword* hiddenPassword);
    virtual BCContainerResult createHiddenKey(SpaceMap *map, cPTB *ptbMainKey, BCPassword* hiddenPassword);
    virtual BCContainerResult verifyPassword(BCPassword & password, struct BCVolumeInfo* out_info, cProgressActor *progresActor, cPTB *keyOut = NULL );
    virtual BCContainerResult decodeKey(BCPassword & password, struct BCVolumeInfo* out_info, bc_key_handle* out_key, CipherBase **cipher_out, cPTB *keyOut = NULL );
	virtual BCContainerResult generateCipherInfo( const cPTB &keyIn, struct BCVolumeInfo* out_info, bc_key_handle* out_handle, CipherBase **cipher_out );
    virtual BCContainerResult addPassword(BCPassword & mainPassword, BCPassword & volumePassword, BCPassword & newPassword);
    virtual BCContainerResult changePassword(BCPassword & mainPassword, BCPassword & volumePassword, BCPassword & newPassword);
    virtual BCContainerResult removePassword(BCPassword & mainPassword, BCPassword & oldPassword);
    virtual BCContainerResult store( iChannel* channel );
    virtual BCContainerResult setDescription(const char * description, const cPTB & mainKey);
    virtual void markForReencryption(bool mark);
    virtual bool isMarkedForReencryption();

    /**
     * Get name of keygen used
     */
    virtual const char * getKeygenName();
    
    /**
     * Returns true if header encryption ot decryption is allowed
     */
    virtual bool isHeaderEncryptionAllowed() { return false; }
    /**
     * Returns true if container can be reencrypted
     */
    virtual bool isReencryptionAllowed() { return true; }

    virtual ~KGSHA_Header();

protected:

    BCContainerResult passwd(BCPassword & mainPassword, BCPassword & volumePassword, BCPassword & password, BCKeyOp opflag);
    BCContainerResult volumeInfoForKey(const kgsha_key* key, struct BCVolumeInfo* out_info);

public:
    HiddenSector    m_hs;

private:
    DATA_BLOCK      m_db;

	int getEncAlgId();
};


#endif

