#include "spacemap.h"

sm_result SpaceMapGetMapOfSize(SpaceMap *map, bc_u64 size, SpaceMap **out_mapOfSize)
{
    SpaceMap *resultMap = 0;
    bc_u64 sizeFound = 0;
    SMExtentNode *cur = 0;
    Extent part;
    sm_result result = kSpaceMap_Success;

    if (!map || !out_mapOfSize)
        return kSpaceMap_InvalidArg;

    resultMap = SpaceMapCreate();
    if (!resultMap)
        return kSpaceMap_NoMem;

    // If one wants an empty map, return it.
    if (!size)
    {
        (*out_mapOfSize) = resultMap;
        return kSpaceMap_Success;
    }

    cur = map->start;
    // Add extents while their total size is less than desired size.
    for (; cur != NULL; cur = cur->next)
    {
        if ((sizeFound + cur->extent.length) <= size)
        {
            // Add full extent.
            result = SpaceMapAddCopy(resultMap, &cur->extent);
            if (kSpaceMap_Success != result)
            {
                SpaceMapFree(resultMap);
                return result;
            }

            sizeFound += cur->extent.length;
        }
        else
        {
            // If not completed, add the part of the extent.
            if (sizeFound < size)
            {
                part.start = cur->extent.start;
                part.length = size - sizeFound;
                result = SpaceMapAddCopy(resultMap, &part);
                if (kSpaceMap_Success != result)
                {
                    SpaceMapFree(resultMap);
                    return result;
                }
                sizeFound += part.length;
            }
            break;
        }
    }

    if (sizeFound == size)
    {
        (*out_mapOfSize) = resultMap;
        return kSpaceMap_Success;
    }

    // Have not enough space in map.
    SpaceMapFree(resultMap);
    return kSpaceMap_NotFound;
}


