#include <QApplication>
#include <QStringList>
#include <QTimer>

#include "CommandLineArgsProcessor.h"
#include "entity/containerdescriptor.h"
#include "operations/bcoperation.h"
#include "operations/MountController.h"


CommandLineArgsProcessor::CommandLineArgsProcessor(const QStringList &args) :
    files(args), exit_code(0)
{
    files.pop_front(); // remove program name from list
}

void CommandLineArgsProcessor::take_one()
{
    if (files.empty()) {
        QApplication::instance()->exit(exit_code);
        return;
    }

    QString containerFile = files.at(0);
    files.pop_front();

    ContainerDescriptor *descriptor = new ContainerDescriptor(this, containerFile);
    descriptor->update();

    BackgroundJob *j = 0;

    switch(descriptor->GetState()) {
    case ContainerDescriptor::kBCContainerState_Linked:
        if (! BCOperationUnlinkContainer(descriptor, false))
            exit_code++;
        break;

    case ContainerDescriptor::kBCContainerState_Mounted:
        if (! BCOperationUmountContainter(descriptor, false))
            exit_code++;
        break;

    case ContainerDescriptor::kBCContainerState_Unlinked:
        j = new MountController(&ew, descriptor);
        connect(j, SIGNAL(finished(bool)), this, SLOT(onContainerOperationFinished(bool)));
        descriptor->runJob(j);
        break;

    default:
        qDebug("Unimplemented case in BCApplication::ProcessArgs");
        break;
    }

    if (! j) {
        QTimer::singleShot(100, this, SLOT(take_one()));
    }
}


void CommandLineArgsProcessor::onContainerOperationFinished(bool success)
{
    if (! success)
        exit_code++;

    // we've done, get next container to process
    QTimer::singleShot(100, this, SLOT(take_one()));
}
