#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include <limits.h>
#include <iconv.h>
#include <locale.h>
#include <langinfo.h>
#include <termios.h>

#include <QMessageBox>
#include <QApplication>
#include <QWaitCondition>
#include <QMutex>
#include <QProgressDialog>

#include "container/user.h"
#include "container/intsize.h"
#include "operations/bcoperation.h"
#include "model/qt_helpers.h"



unsigned GenerateRandom(const char *message, char *out_buf, unsigned total,
                        BCProgressCallback callback, void *callbackArg, int strong)
{
    memset(out_buf, 0, total);

    FILE *f;

    if (strong) {
        f = fopen("/dev/random", "r");
    }
    else {
        f = fopen("/dev/urandom", "r");
    }

    ssize_t bytesRead = 0;
    QApplication::processEvents();

    if (!f)
    {
        BCOperationShowErrorString(QString("Cannot open /dev/%1: %0\n").arg(strerror(errno)).arg((strong)? "random" : "urandom"));
        return 0;
    }

    do
    {
        out_buf[bytesRead++] = fgetc(f);
        QApplication::processEvents();

        if (callback)
        {
            if (!callback(message, bytesRead, total, callbackArg))
            {
                fclose(f);
                return 0;
            }
        }
    }
    while (bytesRead < total);

    fclose(f);
    return total;
}


unsigned BCUserGenerateSeed(const char *message, char *out_buf, unsigned total,
                            BCProgressCallback callback, void *callbackArg)
{
    return GenerateRandom(message, out_buf, total, callback, callbackArg, 1);
}
