#include <QMouseEvent>
#include <QPainter>

#include "containerlistitem.h"
#include "ui_containerlistitem.h"
#include "entity/containerdescriptor.h"
#include "container/intsize.h"
#include "operations/bcoperation.h"
#include "StringUtils.h"

ContainerListItem::ContainerListItem(QWidget *parent, ContainerDescriptor *d) :
    QWidget(parent),
    ui(new Ui::ContainerView),
    descriptor(d)
{
    ui->setupUi(this);

    connect(this->ui->ejectToolButton, SIGNAL(clicked()), this, SLOT(EjectClickSlot()));
    connect(descriptor, SIGNAL(changed()), this, SLOT(onUpdate()));

    onUpdate();
}

ContainerListItem::~ContainerListItem()
{
    delete ui;
}


void ContainerListItem::Select()
{
    if (this->IsSelected())
        return;

    this->setFocus();
    QPalette palette(this->ui->containerNameLabel->palette());
    QPalette defaultPalette(QApplication::palette());
    palette.setColor(QPalette::Foreground, defaultPalette.highlightedText().color());
    this->ui->containerNameLabel->setPalette(palette);
    this->ui->containerInfoLabel->setPalette(palette);
    this->isSelected_ = true;
    this->repaint();

}

void ContainerListItem::Deselect()
{
    if (this->IsSelected())
    {
        QPalette defaultPalette(QApplication::palette());
        this->ui->containerNameLabel->setPalette(defaultPalette);
        this->ui->containerInfoLabel->setPalette(defaultPalette);

        this->isSelected_ = false;
        this->repaint();
    }
}

void ContainerListItem::onUpdate()
{
    this->ui->containerNameLabel->setText(this->descriptor->GetFileName());
    if (this->descriptor->IsExists() && this->descriptor->IsParsed())
    {
        this->ui->containerInfoLabel->setText(
                    QString(tr("%0")).arg(QString::fromStdString(StringUtils::humanSizeString(this->descriptor->size))));
    }
    else
    {
        if (!this->descriptor->IsExists())
            this->ui->containerInfoLabel->setText(tr("File not found."));
        else
            this->ui->containerInfoLabel->setText("Encrypted header or corrupted.");
    }

    if (this->descriptor->IsLinked())
    {
        this->ui->ejectToolButton->setVisible(true);
    }
    else
    {
        this->ui->ejectToolButton->setVisible(false);
    }
}

void ContainerListItem::paintEvent(QPaintEvent *e)
{
    if (!this->IsSelected())
    {
        return this->QWidget::paintEvent(e);
    }

    QPainter p(this);
    QPalette defaultPalette(QApplication::palette());
    p.fillRect(this->rect(), defaultPalette.highlight().color());
}

void ContainerListItem::mouseDoubleClickEvent(QMouseEvent *)
{
    emit this->OnSelected(this);
    emit this->OnMount(this);
}

void ContainerListItem::keyPressEvent(QKeyEvent *event)
{
    switch(event->key())
    {
        case Qt::Key_Return:
        case Qt::Key_Enter:
            if (this->descriptor->IsExists())
            {
                event->accept();
                if (this->descriptor->IsLinked())
                    emit this->OnEject(this);
                else
                    emit this->OnMount(this);
            }
            break;

        case Qt::Key_Up:
            event->accept();
            emit this->OnSelectPrevious(this);
            break;

        case Qt::Key_Down:
            event->accept();
            emit this->OnSelectNext(this);
            break;

        default:
            event->ignore();
            break;
    }
}

void ContainerListItem::mousePressEvent(QMouseEvent */*event*/)
{
    emit this->OnSelected(this);
}

void ContainerListItem::EjectClickSlot()
{
    emit this->OnEject(this);
}
