#include "AssignPasswordDialog.h"
#include "ui_AssignPasswordDialog.h"

#include "keyfilesdialog.h"
#include "operations/QtPwdGet.h"

AssignPasswordDialog::AssignPasswordDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::AssignPasswordDialog)
{
    ui->setupUi(this);
}

AssignPasswordDialog::~AssignPasswordDialog()
{
    delete ui;
}

void AssignPasswordDialog::on_keyFilesButton_clicked()
{
    keyfilesdialog dlg(this, keyfiles);

    if (dlg.exec() != QDialog::Accepted)
        return;

    keyfiles = dlg.getKeyFiles();
    ui->keyFilesButton->setText(dlg.getButtonCaption(keyfiles.size()));
}


void AssignPasswordDialog::setTitle(const char *query, bool prev_failed)
{
    if (query)
        QDialog::setWindowTitle(QString(query));

    if (prev_failed)
        ui->errorLabel->setText("This password is already in use for this container, please enter another one");
}


void AssignPasswordDialog::savePasswordBundleTo(QtPwdGet *pwdGet) const
{
    if (! pwdGet->setPasswordBundle(ui->passEdit->text(), keyfiles)) {
        ui->errorLabel->setText(pwdGet->getLastError());
    }
}


void AssignPasswordDialog::on_passEdit_textChanged(const QString &arg1)
{
    bool password_is_ok = false;

    if (ui->passEdit->text().isEmpty()) {
        ui->errorLabel->setText("");
    }
    else if (ui->passEdit->text().size() < 8) {
        ui->errorLabel->setText("<font color='red'>Password is too short</font>");
    }
    else if (ui->passEdit->text().size() > 255) {
        ui->errorLabel->setText("<font color='red'>Password is too long</font>");
    }
    else if (ui->passEdit->text() != ui->confirmEdit->text()) {
        ui->errorLabel->setText("<font color='red'>Passwords do not match</font>");
    }
    else {
        password_is_ok = true;
        ui->errorLabel->setText("");
    }

    ui->buttonBox->button( QDialogButtonBox::Ok )->setEnabled( password_is_ok );
}
