#include <QDir>
#include <QFileInfo>

#include "BackupHeaderDialog.h"
#include "ui_BackupHeaderDialog.h"

#include "keyfilesdialog.h"
#include "operations/QtPwdGet.h"
#include "model/qt_helpers.h"
#include "bctool.h"


BackupHeaderDialog::BackupHeaderDialog(QWidget *parent, BCToolHeaderParams *hp) :
    QDialog(parent),
    ui(new Ui::BackupHeaderDialog),
    headerParams(hp)
{
    ui->setupUi(this);
    ui->errorLabel->setStyleSheet(QString::fromUtf8("QLabel { color: red; }"));
}

BackupHeaderDialog::~BackupHeaderDialog()
{
    delete ui;
}

void BackupHeaderDialog::setTitle(const char *query, bool prev_failed)
{
    if (query && strlen(query) > 0)
        QDialog::setWindowTitle(QString(query));

    if (prev_failed)
        ui->errorLabel->setText("Incorrect password");
}

void BackupHeaderDialog::setFilename(const QString &path)
{
    ui->filepath->setText(path);

    // default key block backup name: near the original container with .kbb extension
    QFileInfo cnt(path);
    QString kbb_path = cnt.canonicalPath() + QDir::separator() + cnt.baseName() + ".kbb";
    ui->headerPathEdit->setText(kbb_path);
}

void BackupHeaderDialog::savePasswordBundleTo(QtPwdGet *pwdGet) const
{
    if (result() == Accepted) {
        headerParams->headerFilepath = ui->headerPathEdit->text().toStdString();
    }
    else
        ui->passwordEdit->setText("");

    if (! pwdGet->setPasswordBundle(ui->passwordEdit->text(), keyfiles)) {
        ui->errorLabel->setText(pwdGet->getLastError());
    }
}

void BackupHeaderDialog::on_keyFilesButton_clicked()
{
    keyfilesdialog dlg(this, keyfiles);

    if (dlg.exec() != QDialog::Accepted)
        return;

    keyfiles = dlg.getKeyFiles();
    ui->keyFilesButton->setText(dlg.getButtonCaption(keyfiles.size()));
}

void BackupHeaderDialog::on_passwordEdit_textChanged(const QString &arg1)
{
    ui->errorLabel->setText("");
}

void BackupHeaderDialog::on_browseBtn_clicked()
{
    QString p = BCHelperSaveFile(ui->errorLabel->text(), "Enter name for backup file", "*.kbb");
    if (! p.isEmpty())
        ui->keyFilesButton->setText(p);
}
