#include "ContainerAccessDialog.h"
#include "ui_ContainerAccessDialog.h"

#include "keyfilesdialog.h"
#include "operations/QtPwdGet.h"

ContainerAccessDialog::ContainerAccessDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ContainerAccessDialog)
{
    ui->setupUi(this);

    onAdvancedTrigger();

    ui->askMoreCheckBox->hide();
}

ContainerAccessDialog::~ContainerAccessDialog()
{
    delete ui;
}

void ContainerAccessDialog::clear()
{
    keyfiles.clear();
    ui->noteLabel->setText("");
    ui->passwordEdit->setText("");
}

void ContainerAccessDialog::setTitle(const char *query, bool prev_failed)
{
    if (query && strlen(query) > 0)
        QDialog::setWindowTitle(QString(query));

    if (prev_failed)
        ui->errorLabel->setText("Incorrect password");
}

void ContainerAccessDialog::setFilename(const QString &path)
{
    ui->filepath->setText(path);
}


void ContainerAccessDialog::setTextNote(const std::string &txt)
{
    ui->noteLabel->setText(QString(txt.c_str()));
}


void ContainerAccessDialog::savePasswordBundleTo(QtPwdGet *pwdGet) const
{
    if (result() == Rejected)
        ui->passwordEdit->setText("");

    if (! pwdGet->setPasswordBundle(ui->passwordEdit->text(), keyfiles)) {
        ui->errorLabel->setText(pwdGet->getLastError());
    }
}

void ContainerAccessDialog::on_keyFilesButton_clicked()
{
    keyfilesdialog dlg(this, keyfiles);

    if (dlg.exec() != QDialog::Accepted)
        return;

    keyfiles = dlg.getKeyFiles();
    ui->keyFilesButton->setText(dlg.getButtonCaption(keyfiles.size()));
}

void ContainerAccessDialog::on_passwordEdit_textChanged(const QString &arg1)
{
    ui->errorLabel->setText("");
}

void ContainerAccessDialog::onAdvancedTrigger()
{
    if (ui->keyFilesButton->isVisible()) {
        ui->keyFilesButton->hide();
        ui->AdvancedTrigger->setText("<a href=\"#\">Show Anvanced Settings</a>");
    }
    else {
        ui->keyFilesButton->show();
        ui->AdvancedTrigger->setText("<a href=\"#\">Hide Anvanced Settings</a>");
    }
}


void ContainerAccessDialog::enableAskMore()
{
    ui->askMoreCheckBox->show();
}


bool ContainerAccessDialog::needToAskMore() const
{
    return (ui->askMoreCheckBox->checkState() == Qt::Checked);
}


void ContainerAccessDialog::on_AdvancedTrigger_linkActivated(const QString &link)
{
    onAdvancedTrigger();
}

