#ifndef CONTAINERCREATEDIALOG_H
#define CONTAINERCREATEDIALOG_H

#include <QDialog>

#include "kernel/BCKernel.h"
#include "QtPwdGetGlue.h"

namespace Ui {
class ContainerCreateDialog;
}

class ContainerDescriptor;
class BCToolCreateParams;
class BCContainerContext;
class BCToolMountParams;

class ContainerCreateDialog : public QDialog
{
    Q_OBJECT
public:
    explicit ContainerCreateDialog(QWidget *parent);
    ~ContainerCreateDialog();

    ContainerDescriptor *getData();

    // when we create dialog in MainWindow there is no CreationWorker yet
    void dumpParamsTo(BCToolCreateParams *createParams, BCContainerContext &cc, BCToolMountParams *mountParams);

    void setQtPwdGet(QtPwdGet *obj);

protected slots:
    void onNeedPassword(bool prev_failed);

protected:
    // For creation dialog just connect directly QtPwdGet::needPassword signal
    // with onNeedPassword  slot. Do not use QtPwdGetGlue.
    // void savePasswordBundleTo(QtPwdGet * /*pwdGet*/) const {}

private slots:
    void on_mountPointBrowse_clicked();

    void onShowBriefTab();
    void onShowBodyTab();
    void onShowHeaderTab();

    void onContainerSizeChanged();

    void onPassEdited(const QString &val);
    void onConfirmEdited(const QString &val);

    void onUpdateFreeSpaceInfo();
    void onFilesystemOptionsChanged(int /*unused*/);

    void on_algCombo_currentIndexChanged(int index);
    void on_hashAlgBox_currentIndexChanged(int index);
    void on_keyFilesButton_clicked();
    void onGenerateSalt();
    void onUpdateMountBenchmark(int /*unused*/);

    void on_encryptHeaderCheckbox_stateChanged(int arg1);

    void on_filepathBrowseButton_clicked();
    void on_filepathEdit_textChanged(const QString &arg1);

private:
    Ui::ContainerCreateDialog *ui;
    QList<BCAlgInfo> *algList;
    QByteArray salt;
    QtPwdGet *pwdGet;

    QString getFilename() const;
    QString getFilesystemType() const;
    QString getVolumeLabel() const;
    bc_alg_id getEncryptionAlgId() const;
    void fillHashAlgs();
    void setDefaultFilename();
    void setDefaultSize();
    void setUnderpasswordMessage(QString text, bool create_btn_enabled);
    void fillContainerContext(BCContainerContext &cc);
    void validatePassword();

    bool password_is_ok;
    QStringList keyfiles;
    bc_u64 container_size;  //<! in bytes
    int hash_alg_id;
    uint64_t freespace_bytes;
    bool mount_point_is_specified_by_user;
};

#endif // CONTAINERCREATEDIALOG_H
