#include <stdint.h>

#include "HiddenCreateDialog.h"
#include "ui_HiddenCreateDialog.h"

#include "bctool.h"
#include "filesystemwidget.h"
#include "StringUtils.h"


HiddenCreateDialog::HiddenCreateDialog(QWidget *parent, BCToolCreateParams *creatparams) :
    QDialog(parent),
    ui(new Ui::HiddenCreateDialog),
    createParams(creatparams)
{
    ui->setupUi(this);

    ui->progressBar->hide();

    FileSystemWidget::ReloadFilesystems(ui->filesystemCombo, false /* not to include 'no fs' option */);

    connect(ui->protectExistingButton, SIGNAL(clicked()), this, SIGNAL(protectExisting()));
}

HiddenCreateDialog::~HiddenCreateDialog()
{
    delete ui;
}

void HiddenCreateDialog::setHiddenPartSizesRangeInMB(int min_mb, int max_mb)
{
    ui->sizeSpinBox->setMinimum(min_mb);
    ui->sizeSpinBox->setMaximum(max_mb);

    ui->sizeSlider->setMinimum(min_mb);
    ui->sizeSlider->setMaximum(max_mb);

    ui->minSizeLabel->setText( QString::fromStdString(StringUtils::humanSizeString(uint64_t(min_mb) * _1MB)));
    ui->maxSizeLabel->setText( QString::fromStdString(StringUtils::humanSizeString(uint64_t(max_mb) * _1MB)));

    int middle = (min_mb + max_mb) / 2;
    ui->sizeSpinBox->setValue( middle);

    ui->createButton->setEnabled(true);
}


void HiddenCreateDialog::on_createButton_clicked()
{
    // dump params
    createParams->fsType = FileSystemWidget::GetFileSystem(ui->filesystemCombo).toStdString();
    createParams->volumeLabel = ui->labelEdit->text().toStdString();
    createParams->hiddenPartSize = ui->sizeSpinBox->value() * _1MB;
    createParams->mount_after_creation = ui->mountCheckBox->isChecked();

    // just signal, without closing the dialog
    accepted();
}
