#include <QDialog>
#include <QDebug>

#include "QtPwdGetGlue.h"
#include "operations/QtPwdGet.h"


QtPwdGetGlue::QtPwdGetGlue(QObject *parent, QDialog *dlg, QtPwdGet *pwdget) :
    QObject(parent), dialog(dlg), pwdGet(pwdget)
{
    connect(pwdGet, SIGNAL(needPassword(bool)), this, SLOT(onNeedPassword(bool)));
    connect(dialog, SIGNAL(accepted()), this, SLOT(onDlgAccepted()));
    connect(dialog, SIGNAL(rejected()), this, SLOT(onDlgRejected()));
}


void QtPwdGetGlue::onNeedPassword(bool prev_failed)
{
    qDebug() << "QtPwdGetGlue: onNeedPassword";

    PasswordProvider *pp = dynamic_cast<PasswordProvider *>(dialog);
    assert(pp);

    // case of container creation or mount (first attempt) password is already here
    if (! prev_failed && pp->havePasswordAlready()) {
        onDlgAccepted();
    }
    else {
        pp->setTitle(pwdGet->m_query, prev_failed);
        dialog->show();
    }
}


void QtPwdGetGlue::onDlgAccepted()
{
    qDebug() << "QtPwdGetGlue: user clicked OK in dialog";

    PasswordProvider *pp = dynamic_cast<PasswordProvider *>(dialog);
    assert(pp);

    // do this the last: cross-thread call, unpause Worker thread
    pp->savePasswordBundleTo(pwdGet);
    if (pp->needToAskMore())
        pwdGet->getContainerContext().askMore = true;
}


void QtPwdGetGlue::onDlgRejected()
{
    qDebug() << "QtPwdGetGlue: user clicked Cancel in dialog";

    // direct cross-thread call, unpause worker thread
    pwdGet->setPasswordBundle(QString(), QStringList());
}
