#include <QDebug>
#include <QDir>
#include <QFileInfo>

#include "RestoreHeaderDialog.h"
#include "ui_RestoreHeaderDialog.h"

#include "keyfilesdialog.h"
#include "operations/QtPwdGet.h"
#include "model/qt_helpers.h"
#include "bctool.h"

#define BC_BACKUP_FILENAMES "Key Block Backup (*.kbb);; Backup files (*.bak);;BestCrypt Containers (*.jbc);; All Files (*)"


RestoreHeaderDialog::RestoreHeaderDialog(QWidget *parent, BCToolHeaderParams *hp) :
    QDialog(parent),
    ui(new Ui::RestoreHeaderDialog),
    headerParams(hp)
{
    ui->setupUi(this);
    ui->errorLabel->setStyleSheet(QString::fromUtf8("QLabel { color: red; }"));
}

RestoreHeaderDialog::~RestoreHeaderDialog()
{
    delete ui;
}

void RestoreHeaderDialog::setTitle(const char *query, bool prev_failed)
{
    if (query && strlen(query) > 0)
        QDialog::setWindowTitle(QString(query));

    if (prev_failed)
        ui->errorLabel->setText("Incorrect password");
}

void RestoreHeaderDialog::setFilename(const QString &path)
{
    qDebug() << "Entry of " << Q_FUNC_INFO;

    ui->filepath->setText(path);

    // default key block backup name: near the original container with .kbb extension
    QFileInfo cnt(path);
    QString kbb_path = cnt.canonicalPath() + QDir::separator() + cnt.baseName() + ".kbb";
    QFileInfo kbb(kbb_path);
    if (kbb.exists())
        ui->headerPathEdit->setText(kbb_path);
}


bool RestoreHeaderDialog::havePasswordAlready() const
{
    if (ui->passwordEdit->text().isEmpty())
        return false;
    return true;
}


void RestoreHeaderDialog::savePasswordBundleTo(QtPwdGet *pwdGet) const
{
    if (result() == Accepted) {
        headerParams->headerFilepath = ui->headerPathEdit->text().toStdString();
    }
    else
        ui->passwordEdit->setText("");

    if (! pwdGet->setPasswordBundle(ui->passwordEdit->text(), keyfiles)) {
        ui->errorLabel->setText(pwdGet->getLastError());
    }
}

void RestoreHeaderDialog::on_keyFilesButton_clicked()
{
    keyfilesdialog dlg(this, keyfiles);

    if (dlg.exec() != QDialog::Accepted)
        return;

    keyfiles = dlg.getKeyFiles();
    ui->keyFilesButton->setText(dlg.getButtonCaption(keyfiles.size()));
}


void RestoreHeaderDialog::on_passwordEdit_textChanged(const QString &arg1)
{
    ui->errorLabel->setText("");
}


void RestoreHeaderDialog::on_browseBtn_clicked()
{
    QString p = BCHelperSaveFile(ui->errorLabel->text(), "Enter name for backup file", "*.kbb");

    if (! p.isEmpty())
        ui->headerPathEdit->setText(p);
}

