#ifndef BC_SETTINGS_H
#define BC_SETTINGS_H

#include "entity/hotkey.h"

#include <QList>
#include <QSize>
#include <QPoint>
#include <QVariant>
#include <QSettings>
#include <QKeySequence>

class ContainerDescriptor;


/**
  @brief Used for accessing application settings. All property changes
        are reflecting in configuration file.
*/
class BCSettings : QObject
{
    Q_OBJECT

public:

    // General Properties.

    /**
      @brief Gets value indicating whether settings are stored in config.
    */
    bool IsConfigUsed();

    /**
      @brief Sets value indicating whether settings should be saved in config.
            If @param store is set to false, config file will be cleared and
            only this setting will be saved.
    */
    void SetConfigUsed(bool used);

    /**
      @brief Clears all saved settings.
    */
    void Clear();

    bool IsEjectAllHotKeyEnabled();
    void SetEjectAllHotKeyEnabled(bool enabled);

    BCHotkey EjectAllHotKey(const BCHotkey& defaultHotkey = BCHotkey());
    void SetEjectAllHotKey(const BCHotkey& hotkey);

    void SetEjectAllForced(bool forced);
    bool IsEjectAllForced();

    void SetContainerListSaving(bool save);
    bool IsContainerListSaving();

    void SetMinimizeToTray(bool use);
    bool IsMinimizingToTray();


    // Main Form Properties.


    bool MainFormIsMaximized();
    void SetMainFormMaximized(bool maximized);

    QSize MainFormSize(const QSize& defaultSize);
    void SetMainFormSize(const QSize& size);

    QPoint MainFormLocation(const QPoint& defaultLocation);
    void SetMainFormLocation(const QPoint& location);

    //! expose QSettings object
    QSettings &settings() { return userSettings_; }

private:
    QSettings       userSettings_;

    void SaveVariant(const QString& elementName, const QVariant& val, const QString& group = QString());
};
#endif // BC_SETTINGS_H
