#include "filesystemwidget.h"
#include "ui_filesystemwidget.h"

#include "model/qt_helpers.h"

#define NO_FILESYSTEM_STRING (QObject::tr("<No, I will format it later>"))

FileSystemWidget::FileSystemWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::FileSystemWidget)
{
    ui->setupUi(this);

    ReloadFilesystems(ui->fsCombo, true);
}

FileSystemWidget::~FileSystemWidget()
{
    delete ui;
}

QString FileSystemWidget::GetFileSystem()
{
    return GetFileSystem(ui->fsCombo);
}

QString FileSystemWidget::GetFileSystem(QComboBox *fsCombo)
{
    int idx = fsCombo->currentIndex();
    return fsCombo->itemData(idx).toString();
}


void FileSystemWidget::ReloadFilesystems(bool isNoFSVisible)
{
    ReloadFilesystems(ui->fsCombo, isNoFSVisible);
}

void FileSystemWidget::ReloadFilesystems(QComboBox *fsCombo, bool include_no_fs_option)
{
    QStringList filesystem_ids;
    bool succeeded = BCHelperGetSupportedFS(&filesystem_ids);
    if (!succeeded)
        return;

    QList<QString> ignored;
    ignored << "cramfs" << "fuseblk" << "msdos" << "fat";

    fsCombo->clear();

    if (include_no_fs_option) {
        fsCombo->addItem(NO_FILESYSTEM_STRING, QVariant::fromValue(QString("")));
    }

    for (int i = 0; i < filesystem_ids.count(); i++)
    {
        if (ignored.contains(filesystem_ids[i]))
            continue;

        if (filesystem_ids[i] == "vfat") {
            fsCombo->addItem("FAT", "vfat");
        }
        else if (filesystem_ids[i] == "exfat") {
            fsCombo->addItem("exFAT", "exfat");
        }
        else {
            // human-readable name matches with filesystem id
            fsCombo->addItem(filesystem_ids[i], filesystem_ids[i]);
        }
    }

    // Select exFAT or ext3 by default
    int defaultFs = fsCombo->findText("exFAT");
    if (defaultFs == -1)
        defaultFs = fsCombo->findText("ext3");
    if (defaultFs == -1)
        defaultFs = fsCombo->findText("FAT");
    if (defaultFs >= 0) {
        fsCombo->setCurrentIndex(defaultFs);
    }
}
