#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>


#include "entity/settings.h"

class QSystemSemaphore;
class ContainerListModel;
class QItemSelection;
class ContainerInfoWidget;
class ContainerDescriptor;
class ContainerTreeWidget;
class MainToolbarWidget;
class BCToolUI;
class HotkeyListener;
class TrayIconManager;
class WelcomeWidget;
class ContainerCreateDialog;

namespace Ui {
class MainWindow;

}

class MainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

public slots:
    void showError(QString str);

protected:
    void closeEvent(QCloseEvent * event);

private:
    Ui::MainWindow      *ui;
    ContainerListModel      *containerModel_;
    WelcomeWidget       *welcomeWidget;
    HotkeyListener      *hotkeyListener_;
    TrayIconManager     *trayIcon_;
    QSystemSemaphore    *singleInstSemaphore_;
    bool                isRestoring_;

    BCSettings          settings_;

    ContainerCreateDialog *creationDialog;



    void changeEvent(QEvent *e);

    /// @brief Updates enability of tool buttons depending of currently selected descriptor.
    void UpdateToolButtonState();

    ContainerInfoWidget * getContainerInfoWidgetFor(ContainerDescriptor *descriptor);

    /// @brief Updates all necessary controls after some changes have occured in selected descriptor.
    void ProcessDescriptorChanges();

    /// @brief Loads stored coords and sizes (if any) and sets them to widgets.
    void LoadPreviousViewSettings();

    /// @brief Saves necessary coords and sizes.
    void SaveViewSettings();

    /// @brief Updates the hot-key listener state from the application config.
    void UpdateHotkeyListener();

private slots:

    void SelectedDescriptorChangedSlot(ContainerDescriptor *);
    void ContainerIsAboutToRemove(ContainerDescriptor * descriptor);

    void TrayIconDescriptorSelectedSlot(ContainerDescriptor * descriptor);
    void TrayIconShowControlPanelSlot();

    void on_createAction_triggered();
    void onCreationDlgCanceled();
    void onCreationDlgOK();

    void on_locateAction_triggered();
    void on_linkAction_triggered();
    void on_rawFileLinkAction_triggered();

    void on_mountAction_triggered();

    void onContainerOperationFinished(bool success);

    void on_settingsAction_triggered();
    void on_forceEjectAction_triggered();
    void on_ejectAction_triggered();
    void on_ejectAllAction_triggered();
    void on_exitAction_triggered();
    void on_removeAction_triggered();
    void on_headerEncryptAction_triggered();
    void on_headerDecryptAction_triggered();
    void on_headerBackupAction_triggered();
    void on_headerRestoreAction_triggered();
    void on_reencryptAction_triggered();
    void on_createHiddenAction_triggered();
    void on_passwordAddAction_triggered();
    void on_passwordRemoveAction_triggered();
    void on_passwordChangeAction_triggered();
    void on_formatAction_triggered();
    void on_checkFilesystemAction_triggered();
    void on_showLogAction_triggered();
    void on_aboutAction_triggered();
    void on_aboutQtAction_triggered();
    void on_wipeHeaderAction_triggered();
    void on_helpAction_triggered();
    void on_changeDescriptionAction_triggered();
};

#endif // MAINWINDOW_H
