#include <QStringList>

#include "KeyFilesModel.h"
#include "StringUtils.h"

KeyFilesModel::KeyFilesModel(QObject *parent) :
    QAbstractTableModel(parent)
{
}

void KeyFilesModel::appendFile(QString path)
{
    beginInsertRows(QModelIndex(), files.count(), files.count());
    files.append(QFileInfo(path));
    endInsertRows();
}


void KeyFilesModel::clear()
{
    beginRemoveRows(QModelIndex(), 0, files.count() - 1);
    files.clear();
    endRemoveRows();
}


int KeyFilesModel::rowCount(const QModelIndex &parent) const
{
    if (parent.isValid())  // we have simple 2D data structure here, not a tree!
        return 0;

    return files.count();
}

QVariant KeyFilesModel::data(const QModelIndex &i, int role) const
{
    if (role != Qt::DisplayRole)
        return QVariant();

    if (i.parent().isValid())  // we have simple 2D data structure here, not a tree!
        return QVariant();

    if (i.column() > files.size() || i.row() > 2)
        return QVariant();

    switch (i.column()) {
    case 0:
        return QVariant(files[i.row()].absoluteFilePath());
    case 1:
        return QVariant(QString::fromStdString(StringUtils::humanSizeString(files[i.row()].size())));
    }

    return QVariant(); // XXX
}

QVariant KeyFilesModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role != Qt::DisplayRole)
        return QVariant();

    if (orientation == Qt::Horizontal) {
        switch(section) {
        case 0: return "Path";
        case 1: return "Size";
        }
    }

    return QVariant();
}

QStringList KeyFilesModel::keyFiles() const
{
    QStringList ret;
    for (int i = 0 ; i < files.size() ; i++)
        ret.append(files[i].absoluteFilePath());
    return ret;
}


