#ifndef CONTAINERLISTMODEL_H
#define CONTAINERLISTMODEL_H

#include <QObject>
#include <QStringList>

class ContainerDescriptor;

/**
    @brief Model for containers' list.
    TODO: use Qt Model/View framework here: QAbstractListModel
*/
class ContainerListModel :public QObject
{
    Q_OBJECT

public:

    typedef QList<ContainerDescriptor*> list_t;

    ContainerListModel(QObject *parent = 0);
    ~ContainerListModel();

    list_t GetDescriptors() const { return descriptors; }

    list_t getRecentDescriptors() const;

    /// @brief Adds specified container descriptor to model.
    void Add(ContainerDescriptor *descriptor);

    /// @brief Removes specified container descriptor from model.
    void Remove(ContainerDescriptor *descriptor);

    /// @brief Checks whether specified container is already present in the model.
    int IndexOfContainer(const QString &fullPath) const;

    /// @brief Updates all information about each container in the model.
    void Update();

    /// @brief Removes all containers from the model.
    void Clear();

    /// @brief Saves list of containers stored in the model, to config file.
    void SaveToConfig();

    void putOnTopOfRecent(QString path);

signals:
    void changed();

private slots:
    void onDiskEvent();

private:
    list_t descriptors;
    bool ignoreEvents;
    QStringList recents;

    /// @brief Loads list of existing container files from config file.
    void LoadFromConfig();

    /// @brief Loads list of currently mounted and existing containers and adds them to model.
    void LoadFromDevices();

    void InternalAdd(ContainerDescriptor *descriptor);

    void EmitDataChanged();

    // Recent containers list maintenance
    void loadRecentList();
    void storeRecentList();
};

#endif // CONTAINERLISTMODEL_H
