#include <QDebug>

#include "AddPasswordController.h"
#include "AddPasswordWorker.h"
#include "QtCertlibProgressActor.h"
#include "entity/containerdescriptor.h"
#include "container/BCFileChannel.h"
#include "dialogs/ContainerAccessDialog.h"
#include "dialogs/AssignPasswordDialog.h"


AddPasswordController::AddPasswordController(QWidget *parentWidget, ContainerDescriptor *d) :
    BackgroundJob(parentWidget, d), worker(0)
{
    worker = new AddPasswordWorker();

    worker->m_containerFileChannel = new BCFileChannel(descriptor->GetPath().toStdString().c_str());

    worker->cProgressActor = new QtCertlibProgressActor(worker);
    connect(worker->cProgressActor, SIGNAL(progress(QString,int,int)), this, SIGNAL(progress(QString,int,int)));

    mainAccessDialog = new ContainerAccessDialog(parentWidget);
    mainAccessDialog->setFilename(descriptor->GetPath());
    mainAccessDialog->enableAskMore();
    mainGlue = new QtPwdGetGlue(this, mainAccessDialog, &worker->mainPasswordGetter);

    addAccessDialog = new ContainerAccessDialog(parentWidget);
    addAccessDialog->setFilename(descriptor->GetPath());
    addAccessDialog->setTextNote("Please enter a password for an existing hidden part "
                                 "you are going to add a new password to.");
    addGlue = new QtPwdGetGlue(this, addAccessDialog, &worker->additionalPasswordGetter);

    assignPasswordDlg = new AssignPasswordDialog(parentWidget);
    assignGlue = new QtPwdGetGlue(this, assignPasswordDlg, &worker->newPasswordGetter);
}

AddPasswordController::~AddPasswordController()
{
}

void AddPasswordController::on_finished(bool success)
{
    qDebug() << "AddPasswordController::on_finished(" << success << ")";

    emit finished(success);
}

Worker* AddPasswordController::getWorker()
{
    return worker;
}


