#include <QDebug>

#include "BackgroundJob.h"
#include "Worker.h"
#include "bcoperation.h"
#include "QtBCTool.h"
#include "QtCertlibProgressActor.h"
#include "entity/containerdescriptor.h"


BackgroundJob::BackgroundJob(QWidget *parentWid, ContainerDescriptor *d) :
    QObject(d), descriptor(d), parentWidget(parentWid)
{
    thread.setObjectName("BackgroundJob");
}

BackgroundJob::~BackgroundJob()
{
    thread.exit();  // send 'exit' to an event loop
    thread.wait();  // wait for the thread is actually finished
}

void BackgroundJob::start()
{
    connect(getWorker()->bctool, SIGNAL(needAnswer(QString,bool)), this, SLOT(on_YesNoAsked(QString,bool)));
    connect(getWorker()->bctool, SIGNAL(showMessage(int,QString)), this, SLOT(on_showMessage(int,QString)));
    connect(getWorker(), SIGNAL(finished(bool)), this, SLOT(on_finished(bool)));
    connect(getWorker(), SIGNAL(volume_info_updated()), this, SLOT(on_container_info_updated()));

    getWorker()->moveToThread(&thread);

    // async run the method in another thread
    QMetaObject::invokeMethod(getWorker(), "on_entry_point");

    thread.start();
}


void BackgroundJob::onCancelation()
{
    qDebug() << "PROGRESS: " << Q_FUNC_INFO;
    if (getWorker()->cProgressActor) {
        getWorker()->cProgressActor->on_canceled();
    }
}


void BackgroundJob::on_YesNoAsked(QString question, bool defaultAnswer)
{
    bool answer = BCOperationAskUser(question, defaultAnswer);
    getWorker()->bctool->answerReceived(answer);
}

void BackgroundJob::on_showMessage(int level, QString msg)
{
    BCOperationShowMessageString(bctool_msg_level(level), msg);
}

void BackgroundJob::on_container_info_updated()
{
    descriptor->getInfoFrom(getWorker()->m_volumeInfo);
}
