#ifndef BACKGROUNDJOB_H
#define BACKGROUNDJOB_H

#include <QObject>
#include <QThread>

class Worker;
class ContainerDescriptor;

//! Background Job Controller class (executed in UI thread)
class BackgroundJob : public QObject
{
    Q_OBJECT
public:
    explicit BackgroundJob(QWidget *parentWidget, ContainerDescriptor *d);
    ~BackgroundJob();

    //! Execute 'on_entry_point()' slot in a 'Worker'
    void start();

    const ContainerDescriptor* container() const { return descriptor; }

signals:
    // emited in 'on_finished(bool)
    void finished(bool success);

    void progress(QString text, int n, int max);

protected slots:
    //! a 'Worker' should call it when it has done
    virtual void on_finished(bool success) = 0;

    //! BCTool asks 'yes/no' question
    virtual void on_YesNoAsked(QString question, bool defaultAnswer);

    //! BCTool wants to display a message
    virtual void on_showMessage(int level, QString msg);

    //! Container descriptor was updated (after decoding a header for main part)
    void on_container_info_updated();

    virtual void onCancelation();

protected:
    QThread thread;

    ContainerDescriptor *descriptor;
    QWidget *parentWidget;

    virtual Worker* getWorker() = 0;
};

#endif // BACKGROUNDJOB_H
