#include <QDebug>

#include "ChangeDescriptionController.h"
#include "ChangeDescriptionWorker.h"
#include "entity/containerdescriptor.h"
#include "container/BCFileChannel.h"
#include "dialogs/ContainerAccessDialog.h"
#include "QtCertlibProgressActor.h"

ChangeDescriptionController::ChangeDescriptionController(QWidget *parentWidget, ContainerDescriptor *d, QString new_description) :
    BackgroundJob(parentWidget, d), worker(0)
{
    worker = new ChangeDescriptionWorker();

    worker->new_description = new_description.toUtf8().constData();

    worker->m_containerFileChannel = new BCFileChannel(descriptor->GetPath().toStdString().c_str());
    worker->cProgressActor = new QtCertlibProgressActor(worker);
    connect(worker->cProgressActor, SIGNAL(progress(QString,int,int)), this, SIGNAL(progress(QString,int,int)));

    mainAccessDialog = new ContainerAccessDialog(parentWidget);
    mainAccessDialog->setFilename(descriptor->GetPath());
    mainGlue = new QtPwdGetGlue(this, mainAccessDialog, &worker->mainPasswordGetter);
}


void ChangeDescriptionController::on_finished(bool success)
{
    qDebug() << "ChangeDescripionController::on_finished(" << success << ")";

    if (success)
        descriptor->update();  // container file was changed, need to re-read

    emit finished(success);
}

Worker* ChangeDescriptionController::getWorker()
{
    return worker;
}

