#include <QDebug>

#include "ChangePasswordController.h"
#include "ChangePasswordWorker.h"
#include "entity/containerdescriptor.h"
#include "container/BCFileChannel.h"
#include "dialogs/ContainerAccessDialog.h"
#include "dialogs/AssignPasswordDialog.h"
#include "QtCertlibProgressActor.h"


ChangePasswordController::ChangePasswordController(QWidget *parentWidget, ContainerDescriptor *d) :
    BackgroundJob(parentWidget, d), worker(0)
{
    worker = new ChangePasswordWorker();
    connect(worker, SIGNAL(finished(bool)), this, SIGNAL(finished(bool)));

    worker->m_containerFileChannel = new BCFileChannel(descriptor->GetPath().toStdString().c_str());
    worker->cProgressActor = new QtCertlibProgressActor(worker);
    connect(worker->cProgressActor, SIGNAL(progress(QString,int,int)), this, SIGNAL(progress(QString,int,int)));

    mainAccessDialog = new ContainerAccessDialog(parentWidget);
    mainAccessDialog->setFilename(descriptor->GetPath());
    mainAccessDialog->enableAskMore();
    mainGlue = new QtPwdGetGlue(this, mainAccessDialog, &worker->mainPasswordGetter);

    addAccessDialog = new ContainerAccessDialog(parentWidget);
    addAccessDialog->setFilename(descriptor->GetPath());
    addGlue = new QtPwdGetGlue(this, addAccessDialog, &worker->additionalPasswordGetter);

    assignPasswordDlg = new AssignPasswordDialog(parentWidget);
    assignGlue = new QtPwdGetGlue(this, assignPasswordDlg, &worker->newPasswordGetter);
}


void ChangePasswordController::on_finished(bool success)
{
    qDebug() << "ChangePasswordController::on_finished(" << success << ")";

    emit finished(success);
}


Worker* ChangePasswordController::getWorker()
{
    return worker;
}
