#include <QDir>
#include <QDebug>

#include "HiddenCreateController.h"
#include "HiddenCreateWorker.h"
#include "entity/containerdescriptor.h"
#include "src/container/BCFileChannel.h"
#include "dialogs/ContainerAccessDialog.h"
#include "dialogs/HiddenCreateDialog.h"
#include "dialogs/AssignPasswordDialog.h"
#include "bcoperation.h"
//#include "model/qt_helpers.h"
#include "dialogs/QtPwdGetGlue.h"

#define _1MB    uint64_t((1024 * 1024))


HiddenCreateController::HiddenCreateController(QWidget *parentWid, ContainerDescriptor *d)
    : BackgroundJob(parentWid, d)
{
    worker = new HiddenCreateWorker();
    connect(worker, SIGNAL(spacemap_changed()), this, SLOT(onFreeSpaceMapChanged()));

    worker->mountParams.filename = descriptor->GetPath().toStdString();

    // TODO: ???  Why do we need to mount it at all???
    QString mountLocation = getDefaultMountPointFor(descriptor->GetPath());
    worker->mountParams.mountPoint = mountLocation.toStdString();
    createMountPointDirectory(mountLocation);

    strncpy(worker->newHiddenPwdGet.getContainerContext().path,
            descriptor->GetPath().toStdString().c_str(),
            sizeof(worker->newHiddenPwdGet.getContainerContext().path));

    hiddenCreateDlg = new HiddenCreateDialog(parentWidget, &worker->createParams);
    connect(hiddenCreateDlg, SIGNAL(protectExisting()), worker, SLOT(onProtectExistingHiddenPart()));
    connect(hiddenCreateDlg, SIGNAL(accepted()), worker, SLOT(onCreateNewHiddenPart()));
    connect(hiddenCreateDlg, SIGNAL(rejected()), this, SLOT(onCanceled()));
    hiddenCreateDlg->show();

    mainAccessDialog = new ContainerAccessDialog(parentWidget);
    mainAccessDialog->setTitle("Enter password for main part");
    mainAccessDialog->setFilename(descriptor->GetPath());
    mainGlue = new QtPwdGetGlue(this, mainAccessDialog, &worker->mainPwdGet);

    existingHiddenAccessDialog = new ContainerAccessDialog(parentWidget);
    existingHiddenAccessDialog->setTitle("Please enter existing hidden part password");
    existingHiddenAccessDialog->setFilename(descriptor->GetPath());
    existingGlue = new QtPwdGetGlue(this, existingHiddenAccessDialog, &worker->existingHiddenPwdGet);

    assignPasswordDlg = new AssignPasswordDialog(parentWidget);
    assignPasswordDlg->setTitle("Please enter password for new hidden part");
    assignGlue = new QtPwdGetGlue(this, assignPasswordDlg, &worker->newHiddenPwdGet);
}

HiddenCreateController::~HiddenCreateController()
{
    delete worker;
    hiddenCreateDlg->deleteLater();
    mainAccessDialog->deleteLater();
    existingHiddenAccessDialog->deleteLater();
    assignPasswordDlg->deleteLater();
}


void HiddenCreateController::onFreeSpaceMapChanged()
{
    uint64_t freeBytes;
    sm_result r = SpaceMapGetLength(worker->spaceMap, &freeBytes);
    assert (r == kSpaceMap_Success);
    hiddenCreateDlg->setHiddenPartSizesRangeInMB(BCTool::minimalContainerSize / _1MB, freeBytes / _1MB);

    // reset entered password for existing hidden part in dialog
    existingHiddenAccessDialog->clear();
}


void HiddenCreateController::onCanceled()
{
    on_finished(false);
}


void HiddenCreateController::on_finished(bool success)
{
    // if mounted after creation we should display it
    descriptor->update();

    emit finished(success);
}


Worker* HiddenCreateController::getWorker()
{
    return worker;
}

