#include <QDebug>

#include "HiddenCreateWorker.h"
#include "QtBCTool.h"

HiddenCreateWorker::HiddenCreateWorker() :
    spaceMap(0)
{
}

HiddenCreateWorker::~HiddenCreateWorker()
{
    if (spaceMap)
        SpaceMapFree(spaceMap);
}


void HiddenCreateWorker::on_entry_point()
{
    // first step- reading free space map from main part

    qDebug() << "   >>> HiddenCreateWorker::on_entry_point()";

    // Check write permissions if requested.
    if(! bctool->CanWrite(mountParams.filename.c_str())) {
        bctool->ShowError("You do not have write permissions to the container");
        finished(false);
        return;
    }

    /* We use this helper context all the times:
     *  - on read free space map
     *  - on decoding existing hidden parts slots
     *  - on write back new hidden part slot
     */
    m_containerFileChannel = new BCFileChannel(mountParams.filename.c_str());
    bc_file_mode file_mode = kBCFileMode_ReadWrite | kBCFileMode_Exclude;
    m_mainVolumeHelper = new BCHelperContext(m_containerFileChannel, file_mode, &mainPwdGet);
    // m_mainVolumeHelper->progressActor = new BCDecodeProgressActor(_object);

    qDebug() << "Getting a spacemap...";
    spaceMap = bctool->GetFreeSpaceFromMainPart(*m_mainVolumeHelper, &mountParams);

    if (! spaceMap) {
        qDebug() << "Error: unable to get space map";
        finished(false);
        return;
    }

    qDebug() << "Here is spacemap:";
    SpaceMapPrint(spaceMap);

    spacemap_changed();
}

void HiddenCreateWorker::onProtectExistingHiddenPart()
{
    // second (optional, multiple times) step
    qDebug() << "HiddenCreateWorker::onProtectExistingHiddenPart(): entry";

    existingHiddenPwdGet.clear();
    bool res = bctool->ProtectExistingHiddenPart(*m_mainVolumeHelper, &existingHiddenPwdGet, spaceMap);

    if (res)
        spacemap_changed();

    qDebug() << "HiddenCreateWorker::onProtectExistingHiddenPart(): ended with " << res;
}


void HiddenCreateWorker::onCreateNewHiddenPart()
{
    // third step- create new hidden part
    qDebug() << "HiddenCreateWorker::onCreateNewHiddenPart(): entry";

    bool res = bctool->CreateHiddenPart(*m_mainVolumeHelper, &newHiddenPwdGet, spaceMap, &createParams, &mountParams);

    qDebug() << "bctool->CreateHiddenPart returned " << res;
    finished(res);
}

