#include <QDebug>

#include "LinkController.h"
#include "LinkWorker.h"
#include "entity/containerdescriptor.h"
#include "container/BCFileChannel.h"
#include "dialogs/ContainerAccessDialog.h"
#include "QtCertlibProgressActor.h"
#include "QtBCTool.h"
#include "bcoperation.h"


LinkController::LinkController(QWidget *mainWindow, ContainerDescriptor *d) :
    BackgroundJob(mainWindow, d)
    , worker(0)
{
    worker = new LinkWorker();

    worker->mountParams.filename = descriptor->GetPath().toStdString();

    worker->m_containerFileChannel = new BCFileChannel(descriptor->GetPath().toStdString().c_str());
    worker->cProgressActor = new QtCertlibProgressActor(worker);
    connect(worker->cProgressActor, SIGNAL(progress(QString,int,int)), this, SIGNAL(progress(QString,int,int)));

    mainAccessDialog = new ContainerAccessDialog(parentWidget);
    mainAccessDialog->setFilename(descriptor->GetPath());
    mainGlue = new QtPwdGetGlue(this, mainAccessDialog, &worker->mainPasswordGetter);

    // Check write permissions if requested.
    if(! worker->mountParams.mountReadOnly &&
            ! worker->bctool->CanWrite( worker->mountParams.filename.c_str())) {
        // TODO: ???  BCOperationShowWarningString call makes LinkWorker start
        worker->mountParams.mountReadOnly = true;
        BCOperationShowWarningString("You do not have write permissions to the container, it can only be linked for read only.");
    }
}

void LinkController::on_finished(bool success)
{
    qDebug() << "LinkController::on_finished(" << success << ")";

    if (success)
        descriptor->update();  // container file was changed, need to re-read

    emit finished(success);
}

Worker* LinkController::getWorker()
{
    return worker;
}
