#include <QDir>
#include <QMessageBox>
#include <QDebug>

#include "MountController.h"
#include "MountWorker.h"
#include "dialogs/ContainerMountDialog.h"
#include "entity/containerdescriptor.h"
#include "QtCertlibProgressActor.h"
#include "model/qt_helpers.h"
#include "QtBCTool.h"
#include "bcoperation.h"


MountController::MountController(QWidget *parentWidget, ContainerDescriptor *d) :
    BackgroundJob(parentWidget, d), worker(0)
    , canceled(false)
{
    QDir mountLocation(descriptor->lastMountPoint);

    if (descriptor->lastMountPoint.isEmpty())
    {
        // Like removable media default location is
        //  /media/<user name>/<container_basename>
        mountLocation = QDir::homePath() + QDir::separator() + descriptor->GetFileBaseName();
    }

    worker = new MountWorker();

    worker->mountParams.filename = descriptor->GetPath().toStdString();
    worker->mountParams.mountPoint = mountLocation.path().toStdString();
    worker->mountParams.mountReadOnly = descriptor->mountReadOnly;
    worker->mountParams.fsType = descriptor->fileSystem.toStdString();
    worker->mountParams.useDefaultMountPoint = false;

    // Check write permissions if requested.
    if(! worker->mountParams.mountReadOnly
            && ! worker->bctool->CanWrite(worker->mountParams.filename.c_str())) {
        BCOperationShowWarningString("You do not have write permissions to the container, it can only be mount for read only.");
        worker->mountParams.mountReadOnly = true;
    }

    mountDlg = new ContainerMountDialog(parentWidget, &worker->mountParams);
    mountDlgPwdGetGlue = new QtPwdGetGlue(this, mountDlg, &worker->mountPwdGet);
    mountDlg->show();

    worker->cProgressActor = new QtCertlibProgressActor(worker);
    connect(worker->cProgressActor, SIGNAL(progress(QString,int,int)), this, SIGNAL(progress(QString,int,int)));
    // ERROR: this signal will never be delivered because of busy event loop in worker thread!
    // connect(this, SIGNAL(need_cancel()), worker->cProgressActor, SLOT(on_canceled()));
}

MountController::~MountController()
{
    delete worker;
    mountDlg->deleteLater();
}

void MountController::onCancelation()
{
    BackgroundJob::onCancelation();
    canceled = true;
}

void MountController::on_finished(bool success)
{
    qDebug() << "MountController: 'finished' signal (success=" << success << ") is received from worker thread";

    if (success) {
        descriptor->lastMountPoint = QString::fromStdString(worker->mountParams.mountPoint);
    }
    else {
        if (canceled) {
            descriptor->error = "mount canceled";
        }
        else {
            descriptor->error = "Error on mount attempt: " + descriptor->error;
        }
    }

    descriptor->update();

    emit finished(success);
}


Worker* MountController::getWorker()
{
    return worker;
}

