#include <QDebug>

#include "RestoreHeaderController.h"
#include "RestoreHeaderWorker.h"
#include "entity/containerdescriptor.h"
#include "container/BCFileChannel.h"
#include "dialogs/RestoreHeaderDialog.h"
#include "QtCertlibProgressActor.h"


RestoreHeaderController::RestoreHeaderController(QWidget *parentWidget, ContainerDescriptor *d) :
    BackgroundJob(parentWidget, d), worker(0)
{
    worker = new RestoreHeaderWorker();

    worker->m_containerFileChannel = new BCFileChannel(descriptor->GetPath().toStdString().c_str());
    worker->cProgressActor = new QtCertlibProgressActor(worker);
    connect(worker->cProgressActor, SIGNAL(progress(QString,int,int)), this, SIGNAL(progress(QString,int,int)));

    worker->headerParams.containerFilename = descriptor->GetPath().toStdString();

    backupHeaderDialog = new RestoreHeaderDialog(parentWidget, &worker->headerParams);
    backupHeaderDialog->setFilename(descriptor->GetPath());
    mainGlue = new QtPwdGetGlue(this, backupHeaderDialog, &worker->mainPasswordGetter);

    // we should choose keyblock backup file first
    backupHeaderDialog->show();

    // immitate one-shot connection here
    connect(backupHeaderDialog, SIGNAL(accepted()), this, SLOT(on_headerbackup_choosen()));
}


void RestoreHeaderController::on_headerbackup_choosen()
{
    qDebug() << "Entry of " << Q_FUNC_INFO;

    disconnect(backupHeaderDialog, SIGNAL(accepted()), this, SLOT(on_headerbackup_choosen()));

    QMetaObject::invokeMethod(worker, "on_headerbackup_choosen", Qt::QueuedConnection);
}


void RestoreHeaderController::on_finished(bool success)
{
    qDebug() << "RestoreHeaderController::on_finished(" << success << ")";

    if (success)
        descriptor->update();  // container file was changed, need to re-read

    emit finished(success);
}

Worker* RestoreHeaderController::getWorker()
{
    return worker;
}
