#include <QDebug>

#include "selectcipherwidget.h"
#include "ui_selectcipherwidget.h"

#include "model/qt_helpers.h"
#include "operations/bcoperation.h"


SelectCipherWidget::SelectCipherWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::SelectCipherWidget)
{
    ui->setupUi(this);

    this->algList_ = new QList<BCAlgInfo>();
    FillAlgList(this->algList_, ui->algCombo);
}

SelectCipherWidget::~SelectCipherWidget()
{
    delete this->algList_;
    delete ui;
}


//! Loading list of ciphers
void SelectCipherWidget::FillAlgList(QList<BCAlgInfo> *algList, QComboBox *algCombo)
{
    if (! BCOperationGetAlgList(algList))
        return;

    for (int i = 0 ; i < algList->count() ; i++)
    {
        if (BCTool::isDeprecatedEncryptionAlgorithm(algList->at(i).name)) {
            qDebug() << "Hide deprecated algorithm: " << algList->at(i).name;
            continue;
        }

        algCombo->addItem(QString(algList->at(i).name), QVariant::fromValue(algList->at(i).id));
    }

    int defaultAlg = algCombo->findText("RIJNDAEL");

    if (defaultAlg >= 0)
        algCombo->setCurrentIndex(defaultAlg);
}

QString SelectCipherWidget::GetModeName()
{
    return this->ui->modeCombo->currentText();
}

QString SelectCipherWidget::GetAlgorythmName()
{
    int algIndex = this->ui->algCombo->currentIndex();
    if (algIndex == -1)
        return QString();
    return QString(this->algList_->at(algIndex).name);
}

bc_alg_id SelectCipherWidget::GetAlgorythmId()
{
    int algIndex = this->ui->algCombo->currentIndex();
    if (algIndex == -1)
        return BC_INVALID_ALG;
    bc_alg_id algId = this->algList_->at(algIndex).id;
    return algId;
}

namespace {
void AddModeIfSupported(BCAlgInfo &alg, bc_mode_id mode, QComboBox *modeCombo)
{
    if (BCOperationIsModeSupported(alg, mode)) {
        modeCombo->addItem(BCOperationGetModeName(mode), QVariant::fromValue(static_cast<int>(mode)));
    }
}
}

void SelectCipherWidget::updateModeCombo(QList<BCAlgInfo> *algList, QComboBox *modeCombo, int alg_id)
{
    BCAlgInfo selectedAlg;
    selectedAlg.id = -1;

    for(int i = 0; i < algList->count(); ++i )
    {
        if (algList->at(i).id == alg_id) {
            selectedAlg = algList->at(i);
            break;
        }
    }

    modeCombo->clear();

    if (selectedAlg.id == -1)
        return;

    AddModeIfSupported(selectedAlg, kBCMode_CBC, modeCombo);
    AddModeIfSupported(selectedAlg, kBCMode_LRW, modeCombo);
    AddModeIfSupported(selectedAlg, kBCMode_XTS, modeCombo);
    modeCombo->setEnabled(true);

    // automatically select the last (the strongest)
    int modeCount = modeCombo->count();
    if (modeCount)
        modeCombo->setCurrentIndex(modeCount - 1);
}
