#include "trayiconmanager.h"

#include "model/containerlistmodel.h"
#include "operations/bcoperation.h"
#include "model/qt_helpers.h"
#include "entity/containerdescriptor.h"

#include <QList>
#include <QMenu>
#include <QAction>
#include <QSystemTrayIcon>

TrayIconManager::TrayIconManager(ContainerListModel *model)
{
    this->containerModel_ = model;
    connect(this->containerModel_, SIGNAL(changed()), this, SLOT(ContainerModelChangedSlot()));

    QIcon icon;
    BCHelperLoadIcon(":/icons/Main48", &icon);
    this->trayIcon_ = new QSystemTrayIcon(icon);
    this->menu = new QMenu();
    this->trayIcon_->setContextMenu(this->menu);
    this->UpdateMenu();

    connect(this->menu, SIGNAL(triggered(QAction*)), this, SLOT(MenuActionTriggeredSlot(QAction*)));
    connect(this->trayIcon_, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(TrayIconActivatedSlot(QSystemTrayIcon::ActivationReason)));
}

TrayIconManager::~TrayIconManager()
{
    delete this->trayIcon_;
    delete this->menu;
}

void TrayIconManager::ShowIcon()
{
    this->trayIcon_->show();
}

void TrayIconManager::HideIcon()
{
    this->trayIcon_->hide();
}

void TrayIconManager::UpdateMenu()
{
    // Reload container list and add them to menu.
    this->menu->clear();

    // these action objects are destroyed by menu->clear()
    this->showControlPanelAction_ = 0;
    this->exitAction_ = 0;

    const ContainerListModel::list_t descriptors = containerModel_->getRecentDescriptors();

    // TODO:   QIcon mountIcon(":/icons/Mount");
    QIcon mountIcon;
    QIcon unlinkIcon;

    bool haveMountIcon = BCHelperLoadIcon(":/icons/Mount", &mountIcon);
    bool haveUnlinkIcon = BCHelperLoadIcon(":/icons/Eject", &unlinkIcon);

    // Actions are placed in the same order as descriptors.
    for (int i = 0; i < descriptors.count() ; i++)
    {
        ContainerDescriptor *descriptor = descriptors.at(i);

        QAction *action = this->menu->addAction(descriptor->GetFileName());

        if ((descriptor->IsMounted() || descriptor->IsLinked()) && haveUnlinkIcon)
            action->setIcon(unlinkIcon);
        else if (haveMountIcon)
            action->setIcon(mountIcon);
    }

    this->menu->addSeparator();

    this->showControlPanelAction_ = this->menu->addAction(this->trayIcon_->icon(), tr("Show Control Panel"));
    this->ejectAllAction = menu->addAction(tr("Eject All"));
    this->exitAction_ = this->menu->addAction(tr("Quit"));
}

void TrayIconManager::ContainerModelChangedSlot()
{
    this->UpdateMenu();
}

void TrayIconManager::MenuActionTriggeredSlot(QAction *action)
{
    if (action == this->showControlPanelAction_)
    {
        emit this->showControlPanel();
        return;
    }

    if (action == this->exitAction_)
    {
        emit this->quit();
        return;
    }

    if (action == this->ejectAllAction) {
        emit ejectAll();
        return;
    }

    // Suppose here, that actions are placed in the same order as descriptors.
    QList<QAction*> actions = this->menu->actions();
    int actionIndex = actions.indexOf(action);
    if (actionIndex != -1)
    {
        const ContainerListModel::list_t descriptors = this->containerModel_->GetDescriptors();
        ContainerDescriptor *descriptor = descriptors.at(actionIndex);
        emit this->selectDescriptor(descriptor);
        this->UpdateMenu();
    }
}

void TrayIconManager::TrayIconActivatedSlot(QSystemTrayIcon::ActivationReason reason)
{
    switch (reason)
    {
    case QSystemTrayIcon::DoubleClick:
        emit this->showControlPanel();
        break;

    default:
        break;
    }
}
