#ifndef TRAYICONMANAGER_H
#define TRAYICONMANAGER_H

#include <QObject>
#include <QIcon>
#include <QSystemTrayIcon>

class ContainerDescriptor;
class ContainerListModel;
class QAction;
class QMenu;

class TrayIconManager : public QObject
{
    Q_OBJECT
public:
    /**
      @brief Initializes new instance of TrayIconManager.
      @param model      Pointer to the container model.
      @param icon       Tray icon.
    */
    TrayIconManager(ContainerListModel *model);
    ~TrayIconManager();

    void ShowIcon();
    void HideIcon();
    void UpdateMenu();

signals:
    void showControlPanel();
    void quit();
    void ejectAll();
    void selectDescriptor(ContainerDescriptor *descriptor);

private:
    QSystemTrayIcon     *trayIcon_;
    ContainerListModel      *containerModel_;

    // weak pointers to actions (do not own the objects)
    QAction             *showControlPanelAction_;
    QAction             *exitAction_;
    QAction             *ejectAllAction;
    QMenu               *menu;

private slots:
    void ContainerModelChangedSlot();
    void MenuActionTriggeredSlot(QAction *action);
    void TrayIconActivatedSlot(QSystemTrayIcon::ActivationReason reason);
};

#endif // TRAYICONMANAGER_H
