#include "StringUtils.h"

#include <cstdlib>
#include <cstdio>

#define ONE_KB 1024.0L
#define ONE_MB (ONE_KB * 1024.0L)
#define ONE_GB (ONE_MB * 1024.0L)
#define ONE_TB (ONE_GB * 1024.0L)

#define ROUND2(x) (((unsigned long)(x * 100)) / 100.0)
#define ROUND3(x) (((unsigned long)(x * 1000)) / 1000.0)



uint64_t StringUtils::sizeStringToBytes(const char *string)
{
    char *endptr;
    uint64_t result;

    result = strtoul(string, &endptr, 0);
    if (string == endptr)
    {
        return 0;
    }

    if ((string[0] != '\0') && (endptr[0] == '\0'))
    {
        return result;
    }

    if ((endptr[0] == 'K' || endptr[0] == 'k') && (endptr[1]=='\0'))
    {
        result <<= 10;
        return result;
    }

    if ((endptr[0] == 'M' || endptr[0] == 'm') && (endptr[1] == '\0'))
    {
        result <<= 20;
        return result;
    }

    if ((endptr[0] == 'G' || endptr[0] == 'g') && (endptr[1] == '\0'))
    {
        result <<= 30;
        return result;
    }

    if ((endptr[0] == 'T' || endptr[0] == 't') && (endptr[1] == '\0'))
    {
        result <<= 40;
        return result;
    }

    return 0L;
}


std::string StringUtils::humanSizeString(uint64_t size)
{
    char buf[32];

    if (size == 0)
        return "zero";

    // TODO: rewrite with shift >>
    long double d = (long double) size;

    if (d < ONE_KB) {
        snprintf(buf, sizeof(buf), "%d bytes", (int)size);
        return buf;
    }
    else if (d < ONE_MB) {
        snprintf(buf, sizeof(buf), "%.2lf KB", ROUND2(d / ONE_KB));
        return buf;
    }
    else if (d < ONE_GB) {
        snprintf(buf, sizeof(buf), "%.2lf MB", ROUND2(d / ONE_MB));
        return buf;
    }
    else if (d < ONE_TB) {
        snprintf(buf, sizeof(buf), "%.2lf GB", ROUND2(d / ONE_GB));
        return buf;
    }
    else {
        snprintf(buf, sizeof(buf), "%.2lf TB", ROUND2(d / ONE_TB));
        return buf;
    }

    return "way too much"; // XXX
}

std::string StringUtils::humanTimeInterval(double _time)
{
    char buf[32];

    if (_time < 1.0) {
        return "less then a second";
    }

    if (_time < 5.0) {
        snprintf(buf, sizeof(buf), "%.1f seconds", _time);
        return buf;
    }

    if (_time < 60.0) {
        int seconds = _time;
        int remain = seconds % 5;
        if (remain < 3) seconds -= remain;
        else seconds += (5 - remain);
        snprintf(buf, sizeof(buf), "%i seconds", seconds);
        return buf;
    }

    if (_time < 3600.0) {
        int minutes = _time / 60;
        int seconds = _time - minutes * 60;
        int remain = seconds % 5;
        if (remain < 3) seconds -= remain;
        else seconds += (5 - remain);
        snprintf(buf, sizeof(buf), "%i Mins %i Secs", minutes, seconds);
        return buf;
    }

    if (_time < 86400) {
        int hours = _time / 3600;
        int minutes = (_time - hours * 3600) / 60;
        int remain = minutes % 5;
        if (remain < 3) minutes -= remain;
        else minutes += (5 - remain);
        snprintf(buf, sizeof(buf), "%i Hours %i Mins", hours, minutes);
        return buf;
    }

    int days = _time / 86400;
    int hours = (_time - days * 86400) / 3600;
    snprintf(buf, sizeof(buf), "%i Days %i Hours", days, hours);
    return buf;
}
