//
//  BCFile.cpp
//  bcmac
//
//  Created by Jetico Inc. Oy on 23/09/15.
//
//

/*#include <locale>
#include <codecvt>*/
#include <string>
#include <istream>
#include <sys/stat.h>
#include <unistd.h>

#include "BCFileChannel.h"

using namespace std;


BCFileChannel::BCFileChannel(const string & _filePath)
  : m_lastError(feNone)
{
    //if (_filePath == NULL) return;

    /*std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> converter;
    m_filePath = converter.from_bytes(std::string(_filePath));*/
    m_fullPath = _filePath;
}

std::ios_base::seekdir BCFileChannel::ios_dir_FromBCFile(bc_file_seek _seek)
{
    if (_seek == kBCFileSeek_Current) {
        return ios_base::cur;
    } else if (_seek == kBCFileSeek_End) {
        return ios_base::end;
    }
    return ios_base::beg;
}

bc_u64 BCFileChannel::setReadPosition(bc_u64 _offset, bc_file_seek _seek)
{
    m_lastError = feNone;
    if (!m_fileStream.is_open()) {
        m_lastError = feFileClosed;
        return 0;
    }

    ios_base::seekdir seekDirection = ios_dir_FromBCFile(_seek);

    m_fileStream.seekg((_seek == kBCFileSeek_End)? -(bc_64)_offset : _offset, seekDirection);
    if (m_fileStream.fail()) {
        m_fileStream.clear();
        m_lastError = feInternalError;
        return 0;
    }
    return getPosition();
}

bc_u64 BCFileChannel::setWritePosition(bc_u64 _offset, bc_file_seek _seek)
{
    m_lastError = feNone;
    if (!m_fileStream.is_open()) {
        m_lastError = feFileClosed;
        return 0;
    }

    ios_base::seekdir seekDirection = ios_dir_FromBCFile(_seek);

    m_fileStream.seekp(seekDirection == ios_base::end ? -(bc_64)_offset : _offset, seekDirection);
    if (m_fileStream.fail()) {
        m_fileStream.clear();
        m_lastError = feInternalError;
        return 0;
    }
    return getPosition(false);
}

bc_u64 BCFileChannel::setPosition( bc_u64 _offset, bc_file_seek _seek )
{
    return setReadPosition(_offset, _seek);
    /*if (setReadPosition(_offset, _seek) == FC_ERROR) {
        return FC_ERROR;
    }
    if (setWritePosition(_offset, _seek) == FC_ERROR) {
        return FC_ERROR;
    }
    return getPosition();*/
}

bc_u64 BCFileChannel::getPosition(bool _forReading)
{
    m_lastError = feNone;
    if (!m_fileStream.is_open()) {
        m_lastError = feFileClosed;
        return 0;
    }

    streampos position;
    if (_forReading) {
        position = m_fileStream.tellg();
    } else {
        position = m_fileStream.tellp();
    }
    if (m_fileStream.fail()) {
        m_fileStream.clear();
        m_lastError = feInternalError;
        return 0;
    }
    return position;
}

bc_u64 BCFileChannel::readBytes( void * _buffer, bc_u64 _byteCount, bc_u64 _offset )
{
    m_lastError = feNone;
    bool wasNotOpened = false;
    if (!m_fileStream.is_open()) {
        if (!open(kBCFileMode_Read)) {
            m_lastError = feUnableToOpen;
            return 0;
        }
        wasNotOpened = true;
        //m_lastError = feFileClosed;
        //return 0;
    }
    if (_offset != -1) {
        if (setReadPosition(_offset, kBCFileSeek_Begin) != _offset) {
            if (wasNotOpened) close();
            return 0;
        }
    }

    m_fileStream.read((char *)_buffer, _byteCount);
    if (!m_fileStream) {
        _byteCount = m_fileStream.gcount();
        m_fileStream.clear();
        setReadPosition(_byteCount, kBCFileSeek_End);
        m_fileStream.read((char *)_buffer, _byteCount);
        if (!m_fileStream) {
            m_fileStream.clear();
            m_lastError = feInternalError;
            if (wasNotOpened) close();
            return 0;
        }
    }
    if (wasNotOpened) close();

    return m_fileStream.gcount();
}

bc_u64 BCFileChannel::writeBytes( void * _buffer, bc_u64 _byteCount, bc_u64 _offset )
{
    m_lastError = feNone;
    if (!m_fileStream.is_open()) {
        m_lastError = feFileClosed;
        return 0;
    }
    if (_offset != -1) {
        if (setWritePosition(_offset, kBCFileSeek_Begin) != _offset) {
            return 0;
        }
    }

    m_fileStream.write((const char *)_buffer, _byteCount);
    if (!m_fileStream) {
        m_fileStream.clear();
        m_lastError = feInternalError;
        return 0;
    }
    return _byteCount;
}

bc_u64 BCFileChannel::readBytes( void * _buffer, bc_u64 _byteCount )
{
    return readBytes(_buffer, _byteCount, -1);
}

bc_u64 BCFileChannel::writeBytes( void * _buffer, bc_u64 _byteCount )
{
    return writeBytes(_buffer, _byteCount, -1);
}

bool BCFileChannel::open(int _mode)
{
    if (m_fullPath.empty()) {
        return false;
    }
    if (m_fileStream.is_open()) {
        m_fileStream.close();
        if (!m_fileStream) {
            m_fileStream.clear();
            return false;
        }
    }

    /*std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> converter;
    std::string filePath = converter.to_bytes(m_filePath);*/

    if ((_mode & kBCFileMode_Exclude) && !exist()) {
        return false;
    }

    std::ios_base::openmode mode;
    if (_mode & kBCFileMode_Read) {
        mode = std::ios_base::in;
    } else if (_mode & kBCFileMode_ReadWrite) {
        mode = ios_base::in | ios_base::out;
        bool fileExist = exist();
        if ((fileExist && (_mode & kBCFileMode_Exclude)) || !fileExist) {
            mode |= ios_base::trunc;
        }
    } else {
        return false;
    }

    m_fileStream.open(m_fullPath.c_str(), mode);
    if (!m_fileStream) {
        m_fileStream.clear();
        return false;
    }

    return true;
}

bool BCFileChannel::exist() const
{
    m_lastError = feNone;
    if (m_fullPath.empty()) {
        return false;
    }

    ifstream f(m_fullPath.c_str());
    if (f.good()) {
        f.close();
        return true;
    }
    f.close();
    return false;
}

void BCFileChannel::sync()
{
    if (m_fileStream.is_open()) {
        m_fileStream.sync();
    }
}

void BCFileChannel::close()
{
    if (m_fileStream.is_open()) {
        m_fileStream.close();
    }
}

bc_u64 BCFileChannel::getSize()
{
    m_lastError = feNone;
    bool wasNotOpened = false;
    if (!m_fileStream.is_open()) {
        if (!open(kBCFileMode_Read)) {
            m_lastError = feUnableToOpen;
            return 0;
        }
        wasNotOpened = true;
    }

    bc_u64 currentPosition = getPosition();
    if (getLastError() != feNone) {
        if (wasNotOpened) close();
        return 0;
    }

    m_fileStream.seekg (0, m_fileStream.end);
    bc_u64 length = m_fileStream.tellg();
    if (m_fileStream.fail()) {
        m_fileStream.clear();
        if (wasNotOpened) close();
        m_lastError = feInternalError;
        return 0;
    }
    m_fileStream.seekg (currentPosition);

    if (wasNotOpened) close();

    return length;
}

bc_u64 BCFileChannel::truncate(bc_u64 _size)
{
    m_lastError = feNone;
    if (m_fileStream.is_open()) {
        m_lastError = feMustBeClosed;
        return 0;
    }

    if (!exist()) {
        m_lastError = feDoesNotExist;
        return 0;
    }

    if (::truncate(m_fullPath.c_str(), _size) != 0) {
        m_lastError = feInternalError;
        return 0;
    }
    return _size;
}

bool BCFileChannel::isDirectory() const
{
    m_lastError = feNone;
    if (!exist()) {
        m_lastError = feDoesNotExist;
        return false;
    }

    struct stat fileStat;
    if (stat(m_fullPath.c_str(), &fileStat) != 0) {
        m_lastError = feInternalError;
        return false;
    }

    if ((fileStat.st_mode & S_IFDIR) != 0) {
        return true;
    }

    /*ifstream fileOrDir(m_fullPath);
    fileOrDir.seekg(0, ios::end);
    if (!fileOrDir.good()) {
        return true;
    }*/
    return false;
}

std::string BCFileChannel::getPath() const
{
    std::size_t position = m_fullPath.find_last_of("/\\");
    if (position != string::npos) {
        return m_fullPath.substr(0, position);
    }
    return "";
}

std::string BCFileChannel::getName() const
{
    std::size_t position = m_fullPath.find_last_of("/\\");
    if (position != string::npos) {
        return m_fullPath.substr(position + 1);
    }
    return "";
}

void BCFileChannel::deleteFile()
{
    close();
    remove(m_fullPath.c_str());
}

BCFileChannel::~BCFileChannel()
{
    close();
}
