#ifndef __BCFILECHANNEL_H__
#define __BCFILECHANNEL_H__

#include <fstream>

#include "iChannel.h"

class BCFileChannel: public iChannel
{
public:
    //static const bc_u64 FC_ERROR = (bc_u64)-1;
    enum BCFileError {
        feNone,
        feDoesNotExist,
        feFileClosed,
        feMustBeClosed,
        feUnableToOpen,
        feInternalError
    };

    BCFileChannel(const std::string & _filePath);
    ~BCFileChannel();

    bc_u64 setPosition(bc_u64 offset, bc_file_seek seek);
    bc_u64 setReadPosition(bc_u64 offset, bc_file_seek seek);
    bc_u64 setWritePosition(bc_u64 offset, bc_file_seek seek);
    bc_u64 getPosition(bool _forReading = true);

    bc_u64 readBytes( void *buffer, bc_u64 byteCount, bc_u64 offset );
    bc_u64 writeBytes( void *buffer, bc_u64 byteCount, bc_u64 offset );
    bc_u64 readBytes( void *buffer, bc_u64 byteCount );
    bc_u64 writeBytes( void *buffer, bc_u64 byteCount );

    bool open(int _mode);
    void sync();
    void close();
    bool exist() const;
    bool isDirectory() const;

    bc_u64 getSize();
    bc_u64 truncate(bc_u64 _size);
    void deleteFile();

    const std::string & getFullPath() const {
        return m_fullPath;
    }

    std::string getPath() const;
    std::string getName() const;

    BCFileError getLastError() const {
        return m_lastError;
    }

private:
    BCFileChannel();
    BCFileChannel(const BCFileChannel&);

    //std::wstring m_filePath;
    std::string m_fullPath;
    std::fstream m_fileStream;
    mutable BCFileError m_lastError;

    std::ios_base::seekdir ios_dir_FromBCFile(bc_file_seek _seek);
};

#endif // __BCFILECHANNEL_H__
